import 'package:sqflite/sqflite.dart';

import 'package:peertube_mobile/features/account/connected_account/local_account/local_account_model.dart';
import 'package:peertube_mobile/features/video/video_model.dart';

class LocalAccountService {
  final Database database;
  final tableWatchLater = 'LocalWatchLaterPlaylistElement';
  final tableHistory = 'LocalHistoryElement';

  LocalAccountService({
    required this.database,
  });

  Future<List<LocalWatchLaterPlaylistElementSQL>> getWatchLater() async {
    final List<Map<String, dynamic>> maps = await database.query(
      tableWatchLater,
      orderBy: 'positionInPlaylist ASC',
    );

    return maps
        .map((map) => LocalWatchLaterPlaylistElementSQL.fromMap(map))
        .toList();
  }

  Future<void> addToWatchLater({
    required String domain,
    required VideoAbstract video,
  }) async {
    final List<Map<String, dynamic>> result = await database.rawQuery('''
      SELECT MAX(positionInPlaylist) AS maxPosition FROM $tableWatchLater
    ''');

    final lastPosition =
        result.isNotEmpty && result.first['maxPosition'] != null
            ? result.first['maxPosition'] as int
            : 0;

    final newPosition = lastPosition + 1;

    final watchLaterElement = LocalWatchLaterPlaylistElementSQL(
      videoUuid: video.uuid,
      domain: domain,
      name: video.name,
      previewPath: video.previewPath,
      duration: video.duration,
      channelDisplayName: video.channel.displayName,
      positionInPlaylist: newPosition,
    );

    await database.insert(
      tableWatchLater,
      watchLaterElement.toMap(),
      conflictAlgorithm: ConflictAlgorithm.replace,
    );
  }

  Future<void> increasePositionOf({
    required int fromPosition,
    required Transaction transaction,
    int by = 1,
  }) async {
    const query = '''
    UPDATE LocalWatchLaterPlaylistElement
    SET positionInPlaylist = positionInPlaylist + ?
    WHERE positionInPlaylist >= ?
  ''';

    await transaction.rawUpdate(
      query,
      [by, fromPosition],
    );
  }

  Future<void> reorderWatchLaterElements({
    required int oldIndex,
    required int newIndex,
  }) async {
    if (oldIndex == newIndex) {
      return;
    }

    // TODO
  }

  Future<void> removeFromWatchLater({
    required String videoUuid,
  }) async {
    await database.transaction((txn) async {
      final List<Map<String, dynamic>> result = await txn.query(
        tableWatchLater,
        where: 'videoUuid = ?',
        whereArgs: [videoUuid],
        limit: 1,
      );

      if (result.isNotEmpty) {
        final int currentPosition = result.first['positionInPlaylist'] as int;

        await txn.delete(
          tableWatchLater,
          where: 'videoUuid = ?',
          whereArgs: [videoUuid],
        );

        await increasePositionOf(
          fromPosition: currentPosition,
          transaction: txn,
          by: -1,
        );
      }
    });
  }

  Future<void> deleteAllWatchLater() async =>
      await database.delete(tableWatchLater);

  Future<void> addVideoToHistory({
    required String domain,
    required VideoAbstract video,
    required int currentPosition,
  }) async {
    final historyElement = LocalHistoryElementSQL(
      videoUuid: video.uuid,
      domain: domain,
      name: video.name,
      previewPath: video.previewPath,
      duration: video.duration,
      channelDisplayName: video.channel.displayName,
      currentPosition: currentPosition,
      watchedAt: DateTime.now(),
    );

    await database.insert(
      tableHistory,
      historyElement.toMap(),
      conflictAlgorithm: ConflictAlgorithm.replace,
    );
  }

  Future<void> removeFromHistory({
    required LocalHistoryElementSQL element,
  }) async =>
      await database.delete(
        tableHistory,
        where: 'id = ?',
        whereArgs: [element.id],
      );

  Future<void> deleleAllHistory() async => await database.delete(
        tableHistory,
      );

  Future<LocalHistoryElementSQL?> getHistoryElement({
    required String videoUuid,
  }) async {
    final List<Map<String, dynamic>> maps = await database.query(
      tableHistory,
      where: 'videoUuid = ?',
      whereArgs: [videoUuid],
    );

    if (maps.isNotEmpty) {
      return LocalHistoryElementSQL.fromMap(maps.first);
    }
    return null;
  }

  Future<void> updateCurrentPosition({
    required String videoUuid,
    required int currentPosition,
  }) async {
    await database.update(
      tableHistory,
      {
        'currentPosition': currentPosition,
        'watchedAt': DateTime.now().millisecondsSinceEpoch,
      },
      where: 'videoUuid = ?',
      whereArgs: [videoUuid],
    );
  }

  Future<List<LocalHistoryElementSQL>> getHistory() async {
    final List<Map<String, dynamic>> maps = await database.query(
      tableHistory,
      orderBy: 'watchedAt DESC',
    );

    return maps.map((map) => LocalHistoryElementSQL.fromMap(map)).toList();
  }
}
