import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';

import 'package:peertube_mobile/features/filters/filters_providers.dart';
import 'package:peertube_mobile/routes/settings/interests_screen.dart';
import 'package:peertube_mobile/shared/filters/filters_providers.dart';
import 'package:peertube_mobile/shared/filters/search/search_video_params_model.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/button.dart';

class FilterByMyInterestsSwitch extends ConsumerWidget {
  const FilterByMyInterestsSwitch({
    super.key,
  });

  @override
  Widget build(context, ref) {
    final textTheme = Theme.of(context).textTheme;
    return ListTile(
      contentPadding: const EdgeInsets.all(0),
      title: Text(t(context)!.filterByMyInterests),
      subtitle: Padding(
        padding: const EdgeInsets.symmetric(vertical: 4.0),
        child: UnderlinedTextButton(
          onPressed: () => GoRouter.of(context).push('/settings/preferences'),
          text: t(context)!.edit,
          textStyle: textTheme.displaySmall!.copyWith(
            color: Theme.of(context).colorScheme.onSecondary,
          ),
        ),
      ),
      trailing: Semantics(
        label: t(context)!.filterByMyInterests,
        child: Switch(
          value: ref.watch(filterByMyInterestsSwitchProvider),
          onChanged: (value) => ref
              .read(filterByMyInterestsSwitchProvider.notifier)
              .state = value,
        ),
      ),
    );
  }
}

class ContentFilterChipsWidget extends ConsumerWidget {
  const ContentFilterChipsWidget({super.key});

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final contentFilterSettings = ref.watch(exploreSearchVideoFilterProvider);

    return SizedBox(
      height: 48,
      child: ListView(
        scrollDirection: Axis.horizontal,
        children: [
          Row(
            children: [
              Padding(
                padding: const EdgeInsets.symmetric(horizontal: 8.0),
                child: FilterChip(
                  label: Text(
                    getVideoSearchSortCriteriaString(
                      context,
                      VideoSearchSortCriteria.viewsDesc,
                    ),
                  ),
                  selected: contentFilterSettings.sort ==
                      VideoSearchSortCriteria.viewsDesc,
                  onSelected: (isSelected) {
                    ref
                        .read(exploreSearchVideoFilterProvider.notifier)
                        .update(sort: VideoSearchSortCriteria.viewsDesc);
                  },
                ),
              ),
              Padding(
                padding: const EdgeInsets.symmetric(horizontal: 8.0),
                child: FilterChip(
                  label: Text(
                    getVideoSearchSortCriteriaString(
                      context,
                      VideoSearchSortCriteria.likesDesc,
                    ),
                  ),
                  selected: contentFilterSettings.sort ==
                      VideoSearchSortCriteria.likesDesc,
                  onSelected: (isSelected) {
                    ref
                        .read(exploreSearchVideoFilterProvider.notifier)
                        .update(sort: VideoSearchSortCriteria.likesDesc);
                  },
                ),
              ),
              Padding(
                padding: const EdgeInsets.symmetric(horizontal: 8.0),
                child: FilterChip(
                  label: Text(
                    getVideoSearchSortCriteriaString(
                      context,
                      VideoSearchSortCriteria.publishedAtDesc,
                    ),
                  ),
                  selected: contentFilterSettings.sort ==
                      VideoSearchSortCriteria.publishedAtDesc,
                  onSelected: (isSelected) {
                    ref
                        .read(exploreSearchVideoFilterProvider.notifier)
                        .update(sort: VideoSearchSortCriteria.publishedAtDesc);
                  },
                ),
              ),
              const SizedBox(width: 8),
              UnderlinedTextButton(
                onPressed: () {
                  GoRouter.of(context).push('/explore/filters/contents');
                },
                text: t(context)!.moreFilters,
                alignment: Alignment.centerLeft,
                textStyle: Theme.of(context)
                    .textTheme
                    .displaySmall!
                    .copyWith(color: Theme.of(context).colorScheme.onSecondary),
              ),
            ],
          ),
        ],
      ),
    );
  }
}

class AllContentFiltersScreen extends ConsumerStatefulWidget {
  const AllContentFiltersScreen({super.key});

  @override
  ConsumerState<AllContentFiltersScreen> createState() =>
      _AllContentFiltersScreenState();
}

class _AllContentFiltersScreenState
    extends ConsumerState<AllContentFiltersScreen> {
  bool? _isLive;
  Map<String, bool> _selectedCategories = {};
  bool _applyButtonVisible = false;

  @override
  void initState() {
    super.initState();

    final filterSettings = ref.read(exploreSearchVideoFilterProvider);

    _isLive = filterSettings.isLive;
    _selectedCategories = {
      for (var index in categoriesMap.keys)
        index: filterSettings.categoryOneOf?.contains(index) ?? false,
    };
  }

  void _onCategoryTapped(String index) {
    setState(() {
      _selectedCategories[index] = !_selectedCategories[index]!;
      _applyButtonVisible = true;
    });
  }

  void _onVideoTypeChanged(bool? value) {
    setState(() {
      _isLive = value;
      _applyButtonVisible = true;
    });
  }

  void _applyFilters() {
    ref.read(exploreSearchVideoFilterProvider.notifier).update(
          isLive: _isLive,
          categoryOneOf: _selectedCategories.keys
              .where((key) => _selectedCategories[key]!)
              .toList(),
        );

    Navigator.pop(context);
  }

  void _resetFilters() {
    if (ref.read(filterByMyInterestsSwitchProvider)) {
      ref.read(exploreSearchVideoFilterProvider.notifier).applyInterests();
    } else {
      ref.read(exploreSearchVideoFilterProvider.notifier).reset();
    }
    final filterSettings = ref.read(exploreSearchVideoFilterProvider);
    setState(() {
      _isLive = filterSettings.isLive;
      _selectedCategories = {
        for (var index in categoriesMap.keys)
          index: filterSettings.categoryOneOf?.contains(index) ?? false,
      };
    });
  }

  @override
  Widget build(BuildContext context) {
    final textTheme = Theme.of(context).textTheme;
    return Scaffold(
      appBar: AppBar(
        title: Text(t(context)!.videoFilters),
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: SingleChildScrollView(
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              // const FilterByMyInterestsSwitch(),
              const SizedBox(height: 16),
              Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Text(
                        t(context)!.displayOnly,
                        style: textTheme.headlineLarge,
                      ),
                      if (_isLive != null)
                        UnderlinedTextButton(
                          onPressed: () {
                            _onVideoTypeChanged(
                              null,
                            );
                          },
                          text: t(context)!.reset,
                          textStyle: textTheme.displayMedium!,
                        ),
                    ],
                  ),
                  RadioListTile<bool>(
                    title: Text(t(context)!.liveVideos),
                    value: true,
                    groupValue: _isLive,
                    onChanged: (value) => _onVideoTypeChanged(value),
                  ),
                  RadioListTile<bool>(
                    title: Text(t(context)!.vodVideos),
                    value: false,
                    groupValue: _isLive,
                    onChanged: (value) => _onVideoTypeChanged(value),
                  ),
                ],
              ),

              const SizedBox(height: 16),
              Text(
                t(context)!.categories,
                style: textTheme.headlineLarge,
              ),
              const SizedBox(
                height: 8,
              ),
              UnderlinedTextButton(
                onPressed: () =>
                    GoRouter.of(context).push('/settings/preferences'),
                text: t(context)!.updateMyInterests,
                textStyle: textTheme.labelMedium!,
              ),
              const SizedBox(
                height: 16,
              ),
              Wrap(
                spacing: 8.0,
                children: categoriesMap.keys.map((index) {
                  return FilterChip(
                    label: Text(getCategoryString(context, index)),
                    selected: _selectedCategories[index]!,
                    onSelected: (bool selected) {
                      _onCategoryTapped(index);
                    },
                  );
                }).toList(),
              ),
            ],
          ),
        ),
      ),
      bottomNavigationBar: Offstage(
        offstage: ref.watch(exploreSearchVideoFilterProvider).isDefault() &&
            !_applyButtonVisible,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            if (!ref.read(exploreSearchVideoFilterProvider).isDefault() ||
                _applyButtonVisible)
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: CustomElevatedButton(
                  onPressed: _resetFilters,
                  child: Text(t(context)!.resetFilters),
                ),
              ),
            if (_applyButtonVisible)
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: CustomElevatedButton(
                  onPressed: _applyFilters,
                  child: Text(t(context)!.apply),
                ),
              ),
          ],
        ),
      ),
    );
  }
}
