import 'package:sqflite/sqflite.dart';

import 'package:peertube_mobile/features/account/connected_account/connected_account_model.dart';
import 'package:peertube_mobile/features/platform/history/platform_history_model.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences.dart';

class PlatformHistoryService {
  final SharedUtility sharedUtility;
  final Database database;

  final tablePlatormHistory = 'PlatformHistoryElement';

  PlatformHistoryService({
    required this.sharedUtility,
    required this.database,
  });

  Future<void> addPlatformToHistory({
    required String domain,
    required String name,
    required ConnectedAccountSQL account,
    String? avatarUrl,
  }) async {
    final platform = PlatformHistoryElementSQL(
      domain: domain,
      name: name,
      avatarUrl: avatarUrl,
      bookmarked: false,
      lastAccess: DateTime.now(),
      accountId: account.id,
    );

    await database.insert(
      tablePlatormHistory,
      platform.toMap(),
      conflictAlgorithm: ConflictAlgorithm.replace,
    );
  }

  Future<void> _updateBookmarkStatus(
    PlatformHistoryElementSQL platform,
    bool isBookmarked,
  ) async =>
      await database.update(
        tablePlatormHistory,
        {'bookmarked': isBookmarked ? 1 : 0},
        where: 'id = ?',
        whereArgs: [platform.id],
      );

  Future<void> addToBookmarks(PlatformHistoryElementSQL platform) async {
    await _updateBookmarkStatus(platform, true);
  }

  Future<void> removeFromBookmarks(PlatformHistoryElementSQL platform) async {
    await _updateBookmarkStatus(platform, false);
  }

  Future<void> delete(int platformHistoryElementId) async =>
      await database.delete(
        tablePlatormHistory,
        where: 'id = ?',
        whereArgs: [platformHistoryElementId],
      );

  Future<List<PlatformHistoryElementSQL>> getBookmarks() async {
    final List<Map<String, dynamic>> maps = await database.query(
      tablePlatormHistory,
      where: 'bookmarked = ?',
      whereArgs: [1],
    );

    return maps.map((map) => PlatformHistoryElementSQL.fromMap(map)).toList();
  }

  Future<PlatformHistoryElementSQL?> getPlatformByDomain(
    String domain,
    ConnectedAccountSQL account,
  ) async {
    final List<Map<String, dynamic>> maps = await database.query(
      tablePlatormHistory,
      where: 'domain = ? AND accountId = ?',
      whereArgs: [domain, account.id],
    );

    if (maps.isNotEmpty) {
      return PlatformHistoryElementSQL.fromMap(maps.first);
    }
    return null;
  }

  Future<List<PlatformHistoryElementSQL>> getPlatforms(
    ConnectedAccountSQL account,
  ) async {
    final List<Map<String, dynamic>> maps = await database.query(
      tablePlatormHistory,
      where: 'accountId = ?',
      whereArgs: [account.id],
      orderBy: 'bookmarked DESC, lastAccess DESC',
    );

    return maps.map((map) => PlatformHistoryElementSQL.fromMap(map)).toList();
  }

  Future<PlatformHistoryElementSQL?> getLastPlatform() async {
    final List<Map<String, dynamic>> maps = await database.query(
      tablePlatormHistory,
      orderBy: 'lastAccess DESC',
      limit: 1,
    );

    if (maps.isNotEmpty) {
      return PlatformHistoryElementSQL.fromMap(maps.first);
    }
    return null;
  }

  Future<void> updateLastAccess(PlatformHistoryElementSQL platform) async =>
      await database.update(
        tablePlatormHistory,
        {
          'lastAccess': DateTime.now().millisecondsSinceEpoch,
        },
        where: 'id = ?',
        whereArgs: [platform.id],
      );

  Future<void> updateLastAccessByDomain(
    String domain,
    ConnectedAccountSQL account,
  ) async {
    await database.update(
      tablePlatormHistory,
      {
        'lastAccess': DateTime.now().millisecondsSinceEpoch,
      },
      where: 'domain = ? AND accountId = ?',
      whereArgs: [domain, account.id],
    );
  }
}
