import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/features/platform/history/platform_history_model.dart';
import 'package:peertube_mobile/features/platform/history/platform_history_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences.dart';
import 'package:peertube_mobile/shared/utils/time.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/avatar_builder.dart';
import 'package:peertube_mobile/shared/widgets/dialog.dart';

final showDialogRemovePlatformProvider = StateProvider<bool>(
  (ref) {
    SharedUtility sharedUtility = ref.watch(sharedUtilityProvider);
    bool? showDialogRemovePlatform = sharedUtility.showDialogRemovePlatform;

    ref.listenSelf((bool? prev, bool curr) {
      sharedUtility.setShowDialogRemovePlatform(curr);
    });

    return showDialogRemovePlatform ?? true;
  },
  name: 'showDialogRemovePlatformProvider',
);

class PlatformHistoryModalItem extends ConsumerWidget {
  final PlatformHistoryElementSQL platform;
  final bool isCurrentPlatform;
  final VoidCallback onTap;
  final VoidCallback? onDeleted;

  const PlatformHistoryModalItem({
    super.key,
    required this.platform,
    this.isCurrentPlatform = false,
    required this.onTap,
    this.onDeleted,
  });

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    ColorScheme colorScheme = Theme.of(context).colorScheme;
    return Container(
      decoration: BoxDecoration(
        color: isCurrentPlatform
            ? colorScheme.surfaceContainerHigh
            : Colors.transparent,
        borderRadius: const BorderRadius.only(
          topRight: Radius.circular(50.0),
          bottomRight: Radius.circular(50.0),
        ),
      ),
      margin: const EdgeInsets.only(right: 8),
      padding: const EdgeInsets.symmetric(vertical: 8),
      child: ListTile(
        onTap: onTap,
        isThreeLine: true,
        leading: Container(
          width: 54,
          height: 54,
          decoration: BoxDecoration(
            border: Border.all(
              width: 2,
              color:
                  isCurrentPlatform ? colorScheme.primary : Colors.transparent,
            ),
            borderRadius: BorderRadius.circular(16),
          ),
          child: Center(
            child: AvatarBuilder(
              name: platform.name,
              domain: platform.domain,
              avatarUrl: platform.avatarUrl,
              size: 48.0,
              borderRadius: 16,
              borderColor: Colors.transparent,
            ),
          ),
        ),
        title: Text(
          platform.name,
          style: Theme.of(context).textTheme.headlineLarge,
        ),
        subtitle: Row(
          children: [
            Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text(
                  platform.domain.toString(),
                  style: Theme.of(context)
                      .textTheme
                      .displaySmall!
                      .copyWith(color: colorScheme.onTertiary),
                ),
                const SizedBox(
                  height: 8,
                ),
                Row(
                  children: [
                    Icon(
                      isCurrentPlatform
                          ? LucideIcons.rocket
                          : LucideIcons.history,
                      size: 18,
                      color: colorScheme.onSecondary,
                    ),
                    const SizedBox(
                      width: 4,
                    ),
                    Text(
                      isCurrentPlatform
                          ? t(context)!.runningPlatform
                          : timeago(platform.lastAccess, context),
                      style: Theme.of(context)
                          .textTheme
                          .displayMedium!
                          .copyWith(color: colorScheme.onSecondary),
                    ),
                  ],
                ),
              ],
            ),
            const Spacer(),
          ],
        ),
        trailing: PopupMenuButton<String>(
          offset: const Offset(-10, 48),
          onSelected: (value) async {
            if (value == 'delete') {
              final confirmed = await showDeleteConfirmationDialog(
                context,
                ref,
                platform.name,
              );
              if (confirmed != null && confirmed) {
                ref.read(platformHistoryProvider.notifier).delete(platform);
              }
            } else if (value == 'pin') {
              platform.bookmarked
                  ? ref
                      .read(platformHistoryProvider.notifier)
                      .removeBookmark(platform)
                  : ref
                      .read(platformHistoryProvider.notifier)
                      .addBookmark(platform);
            }
          },
          itemBuilder: (context) => <PopupMenuEntry<String>>[
            if (!isCurrentPlatform)
              PopupMenuItem<String>(
                value: 'delete',
                child: Row(
                  children: [
                    const Icon(LucideIcons.trash2),
                    const SizedBox(width: 8),
                    Text(t(context)!.delete),
                  ],
                ),
              ),
            PopupMenuItem<String>(
              value: 'pin',
              child: Row(
                children: [
                  Icon(
                    platform.bookmarked ? LucideIcons.pinOff : LucideIcons.pin,
                  ),
                  const SizedBox(width: 8),
                  Text(
                    platform.bookmarked ? t(context)!.unpin : t(context)!.pin,
                  ),
                ],
              ),
            ),
          ],
        ),
      ),
    );
  }

  Future<bool?> showDeleteConfirmationDialog(
    BuildContext context,
    WidgetRef ref,
    String platformName,
  ) async {
    if (ref.read(showDialogRemovePlatformProvider) == false) return true;
    return showDialog<bool>(
      context: context,
      builder: (BuildContext context) {
        return CustomAlertDialog(
          title: t(context)!.confirmation,
          content: t(context)!.platformDeletionConfirmation(platformName),
          cancelText: t(context)!.platformKeepItInMyList,
          confirmText: t(context)!.platformRemove,
          showAgain: ref.read(showDialogRemovePlatformProvider),
          onShowAgain: (value) => ref
              .read(showDialogRemovePlatformProvider.notifier)
              .state = value ?? true,
        );
      },
    );
  }
}
