import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/features/platform/joinpeertube/joinpeertube_models.dart';
import 'package:peertube_mobile/features/platform/platform_miniature.dart';
import 'package:peertube_mobile/shared/actor/actor_image_model.dart';
import 'package:peertube_mobile/shared/config/config_models.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/utils/tools.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/avatar_builder.dart';
import 'package:peertube_mobile/shared/widgets/dialog.dart';
import 'package:peertube_mobile/shared/widgets/image.dart';

class PlatformBanner extends ConsumerWidget {
  final String name;
  final String version;
  final String? bannerUrl;
  final String shortDescription;
  final String domain;
  final List<ActorImage>? avatarActorImageList;
  final String? avatarUrl;
  final String? entityName;

  const PlatformBanner({
    required this.name,
    required this.version,
    this.bannerUrl,
    required this.shortDescription,
    required this.domain,
    this.avatarActorImageList,
    this.avatarUrl,
    this.entityName,
    super.key,
  });

  @override
  Widget build(context, ref) {
    final colorScheme = Theme.of(context).colorScheme;
    final textTheme = Theme.of(context).textTheme;
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 4.0),
      child: Column(
        children: [
          if (entityName != null)
            Row(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                Text(
                  t(context)!
                      .entitynameFederatedFromThisPlatform(entityName ?? ''),
                ),
                IconButton(
                  onPressed: () => showDialog(
                    context: context,
                    builder: (context) => TitleContentDialog(
                      title: t(context)!.federationInformationTitle,
                      content: t(context)!.federationInformationDescription(
                        entityName ?? '',
                        name,
                        ref.watch(currentPlatformNonNullableProvider).name,
                      ),
                    ),
                  ),
                  icon: const Icon(LucideIcons.helpCircle),
                ),
              ],
            ),
          const SizedBox(
            height: 8,
          ),
          GestureDetector(
            onTap: () => onTapPlatform(
              context,
              ref,
              version: version,
              name: name,
              domain: domain,
              avatarUrl: avatarUrl,
            ),
            child: Container(
              decoration: BoxDecoration(
                color: colorScheme.surfaceContainerHighest,
                borderRadius: BorderRadius.circular(6.0),
                boxShadow: const [
                  BoxShadow(
                    color: Colors.black12,
                    blurRadius: 8.0,
                  ),
                ],
              ),
              child: Column(
                children: [
                  if (bannerUrl != null)
                    ClipRRect(
                      borderRadius: const BorderRadius.vertical(
                        top: Radius.circular(6.0),
                      ),
                      child: CustomCachedNetworkImage(
                        imageUrl: bannerUrl!,
                      ),
                    ),
                  Padding(
                    padding: const EdgeInsets.all(16.0),
                    child: Row(
                      children: [
                        AvatarBuilder(
                          domain: domain,
                          name: name,
                          size: 44,
                          actorImageList: avatarActorImageList,
                          avatarUrl: avatarUrl,
                          borderRadius: 16,
                          borderWidth: 3,
                        ),
                        const SizedBox(width: 8.0),
                        Expanded(
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text(
                                name,
                                style: textTheme.headlineMedium!
                                    .copyWith(fontWeight: FontWeight.w700),
                              ),
                              Text(
                                shortDescription,
                                maxLines: 2,
                                overflow: TextOverflow.ellipsis,
                                style: textTheme.displayMedium!.copyWith(
                                  color: colorScheme.onSecondary,
                                ),
                              ),
                            ],
                          ),
                        ),
                      ],
                    ),
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class PlatformBannerFromConfig extends ConsumerWidget {
  final String host;
  final ConfigServer config;
  final String? entityName;

  const PlatformBannerFromConfig({
    required this.host,
    required this.config,
    this.entityName,
    super.key,
  });
  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final platform = config.instance;

    final bannerUrl = platform.banners == null || platform.banners!.isEmpty
        ? null
        : Uri.parse(
            'https://$host',
          )
            .resolve(
              getImagePathForSize(
                platform.banners!,
                MediaQuery.of(context).size.width,
              )!,
            )
            .toString();
    return PlatformBanner(
      name: platform.name,
      version: config.serverVersion,
      bannerUrl: bannerUrl,
      shortDescription: platform.shortDescription,
      domain: 'https://$host',
      avatarActorImageList: platform.avatars,
      entityName: entityName,
    );
  }
}

class PlatformBannerFromJPT extends ConsumerWidget {
  final PlatformJPT platformJPT;

  const PlatformBannerFromJPT({
    required this.platformJPT,
    super.key,
  });

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final bannerUrl =
        platformJPT.banners.isEmpty ? null : platformJPT.banners[0].url;
    final avatarUrl =
        platformJPT.avatars.isEmpty ? null : platformJPT.avatars[0].url;
    return PlatformBanner(
      bannerUrl: bannerUrl,
      version: platformJPT.version,
      name: platformJPT.name,
      shortDescription: platformJPT.shortDescription ?? '',
      domain: 'https://${platformJPT.host}',
      avatarUrl: avatarUrl,
    );
  }
}
