import 'dart:async';

import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:intl/intl.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/features/account/connected_account/connected_account_providers.dart';
import 'package:peertube_mobile/features/platform/joinpeertube/joinpeertube_models.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/checkbox_list_tile.dart';
import 'package:peertube_mobile/shared/widgets/dialog.dart';
import 'package:peertube_mobile/shared/widgets/image.dart';

class PlatformMiniature extends ConsumerWidget {
  final PlatformJPT platformJPT;

  const PlatformMiniature({
    super.key,
    required this.platformJPT,
  });

  @override
  Widget build(context, ref) {
    final theme = Theme.of(context);
    return Semantics(
      label: t(context)!.platformInstanceName(platformJPT.name),
      onTapHint: t(context)!.goToThisPlatform,
      child: GestureDetector(
        onTap: () async => await onTapPlatform(
          context,
          ref,
          version: platformJPT.version,
          name: platformJPT.name,
          domain: 'https://${platformJPT.host}',
          avatarUrl:
              (platformJPT.avatars.isEmpty ? null : platformJPT.avatars[0].url),
        ),
        child: Container(
          margin: const EdgeInsets.only(right: 16.0, top: 8.0, bottom: 8.0),
          width: 220,
          decoration: BoxDecoration(
            color: theme.colorScheme.surfaceContainerHighest,
            borderRadius: BorderRadius.circular(6.0),
          ),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              ClipRRect(
                borderRadius: const BorderRadius.only(
                  topLeft: Radius.circular(6.0),
                  topRight: Radius.circular(6.0),
                ),
                child: platformJPT.banners.isEmpty
                    ? const SizedBox.shrink()
                    : CustomCachedNetworkImage(
                        imageUrl: platformJPT.banners[0].url,
                        height: 59,
                      ),
              ),
              const SizedBox(height: 8.0),
              Padding(
                padding: const EdgeInsets.all(12.0),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                      platformJPT.name,
                      style: theme.textTheme.headlineMedium!.copyWith(
                        fontWeight: FontWeight.w600,
                      ),
                      overflow: TextOverflow.ellipsis,
                    ),
                    const SizedBox(height: 4.0),
                    SizedBox(
                      height: 48,
                      child: Text(
                        platformJPT.shortDescription ?? '',
                        style: theme.textTheme.displayMedium!.copyWith(
                          color: theme.colorScheme.onSecondary,
                        ),
                        overflow: TextOverflow.ellipsis,
                        maxLines: 3,
                      ),
                    ),
                    const SizedBox(height: 8.0),
                    Row(
                      children: [
                        Icon(
                          LucideIcons.youtube,
                          size: 15,
                          color: theme.colorScheme.onTertiary,
                        ),
                        const SizedBox(width: 4.0),
                        Text(
                          t(context)!.totalVideos(
                            NumberFormat.decimalPattern(
                              ref.watch(localeProvider).languageCode,
                            ).format(platformJPT.totalLocalVideos),
                          ),
                          style: theme.textTheme.displayMedium!.copyWith(
                            color: theme.colorScheme.onTertiary,
                          ),
                        ),
                      ],
                    ),
                    const SizedBox(height: 4.0),
                    Row(
                      children: [
                        Icon(
                          LucideIcons.users2,
                          size: 15,
                          color: theme.colorScheme.onTertiary,
                        ),
                        const SizedBox(width: 4.0),
                        Text(
                          t(context)!.totalUsers(
                            NumberFormat.decimalPattern(
                              ref.watch(localeProvider).languageCode,
                            ).format(platformJPT.totalUsers),
                          ),
                          style: theme.textTheme.displayMedium!.copyWith(
                            color: theme.colorScheme.onTertiary,
                          ),
                        ),
                      ],
                    ),
                  ],
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}

Future<void> onTapPlatform(
  BuildContext context,
  WidgetRef ref, {
  required String version,
  required String name,
  required String domain,
  String? avatarUrl,
}) async {
  bool showAgain = ref.read(showDialogGoPlatformProvider);

  if (!version.startsWith('6') && !version.startsWith('7')) {
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        return TitleContentDialog(
          title: t(context)!.unableToConnect,
          content: t(context)!.versionTooOld,
        );
      },
    );
  } else if (!showAgain) {
    await handlePlatformUpdate(context, ref, name, domain, avatarUrl);
  } else {
    await showDialog(
      context: context,
      builder: (BuildContext context) {
        return PlatformDialog(
          version: version,
          name: name,
          domain: domain,
          avatarUrl: avatarUrl,
        );
      },
    );
  }
}

Future<bool> handlePlatformUpdate(
  BuildContext context,
  WidgetRef ref,
  String name,
  String domain,
  String? avatarUrl,
) async {
  final bool success =
      await ref.read(currentConnectedAccountProvider).maybeWhen(
            data: (account) async {
              if (context.mounted) {
                return await ref
                    .read(currentPlatformServiceProvider)
                    .updateCurrentPlatform(
                      ref,
                      domain,
                      account,
                    );
              }
              return true;
            },
            orElse: () => false,
          );

  if (context.mounted) {
    if (success) {
      GoRouter.of(context).go('/');
    } else {
      await showPlatformNotRespondingDialog(context, name);
    }
  }
  return success;
}

Future<dynamic> showPlatformNotRespondingDialog(
  BuildContext context,
  String name,
) {
  return showDialog(
    context: context,
    builder: (BuildContext context) {
      return TitleContentDialog(
        title: t(context)!.unableToConnect,
        content: t(context)!.platformIsNotResponding(name),
      );
    },
  );
}

class PlatformDialog extends ConsumerWidget {
  final String version;
  final String name;
  final String domain;
  final String? avatarUrl;

  const PlatformDialog({
    super.key,
    required this.version,
    required this.name,
    required this.domain,
    this.avatarUrl,
  });

  @override
  Widget build(context, ref) {
    final initialShowAgain = ref.read(showDialogGoPlatformProvider);
    return AlertDialog(
      title: Text(t(context)!.redirection(name)),
      content: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(t(context)!.redirectionPlatform(name)),
          CustomCheckboxListTile(
            value: ref.read(showDialogGoPlatformProvider),
            onChanged: (value) => ref
                .read(showDialogGoPlatformProvider.notifier)
                .state = value ?? true,
            title: 'Show me this again',
          ),
        ],
      ),
      actions: [
        TextButton(
          onPressed: () {
            ref.read(showDialogGoPlatformProvider.notifier).state =
                initialShowAgain;
            Navigator.of(context).pop(false);
          },
          child: Text(t(context)!.cancel),
        ),
        TextButton(
          onPressed: () async {
            ref.read(showDialogGoPlatformProvider.notifier).state =
                ref.read(showDialogGoPlatformProvider);
            final success = await handlePlatformUpdate(
              context,
              ref,
              name,
              domain,
              avatarUrl,
            );
            if (context.mounted) Navigator.of(context).pop(success);
          },
          child: Text(t(context)!.confirm),
        ),
      ],
    );
  }
}

final showDialogGoPlatformProvider = StateProvider<bool>(
  (ref) {
    SharedUtility sharedUtility = ref.watch(sharedUtilityProvider);
    bool? showDialogGoPlatform = sharedUtility.showDialogGoPlatform;

    ref.listenSelf((bool? prev, bool curr) {
      sharedUtility.setShowDialogGoPlatform(curr);
    });

    return showDialogGoPlatform ?? true;
  },
  name: 'showDialogGoPlatformProvider',
);
