import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:infinite_scroll_pagination/infinite_scroll_pagination.dart';

import 'package:peertube_mobile/features/video/channel/channel_miniature.dart';
import 'package:peertube_mobile/features/video/channel/channel_model.dart';
import 'package:peertube_mobile/shared/config/config_models.dart';
import 'package:peertube_mobile/shared/filters/account/account_params_model.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';

class VideoChannelListView extends ConsumerStatefulWidget {
  final Future<List<VideoChannelDetails>> Function(AccountParams params)
      fetchVideoChannels;
  final AccountParams params;
  final bool isHorizontal;
  final bool isNested;
  final bool wantKeepAlive;
  final ConfigServer config;

  const VideoChannelListView({
    super.key,
    required this.fetchVideoChannels,
    required this.config,
    this.params = const AccountParams(),
    this.isHorizontal = false,
    this.isNested = false,
    this.wantKeepAlive = false,
  });

  @override
  ConsumerState<VideoChannelListView> createState() =>
      _VideoChannelListViewState();
}

class _VideoChannelListViewState extends ConsumerState<VideoChannelListView>
    with AutomaticKeepAliveClientMixin<VideoChannelListView> {
  final PagingController<int, VideoChannelDetails> _pagingController =
      PagingController(firstPageKey: 0);

  @override
  bool get wantKeepAlive => widget.wantKeepAlive;

  @override
  void initState() {
    _pagingController.addPageRequestListener((pageKey) {
      _fetchPage(pageKey);
    });
    super.initState();
  }

  Future<void> _fetchPage(int pageKey) async {
    int pageSize = 15;

    try {
      final newItems = await widget.fetchVideoChannels(
        widget.params.copyWith(
          start: pageKey,
          count: pageSize,
        ),
      );

      if (!mounted) {
        return;
      }

      final isLastPage = newItems.length < pageSize;

      if (isLastPage) {
        _pagingController.appendLastPage(newItems);
      } else {
        final nextPageKey = pageKey + newItems.length;
        _pagingController.appendPage(newItems, nextPageKey);
      }
    } catch (error) {
      _pagingController.error = error;
    }
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);

    Future<void> refresh() async => _pagingController.refresh();

    return !widget.isHorizontal
        ? Semantics(
            label: t(context)!.videoList,
            onTap: refresh,
            onTapHint: t(context)!.refreshList,
            child: RefreshIndicator(
              onRefresh: refresh,
              child: PagedListView<int, VideoChannelDetails>(
                pagingController: _pagingController,
                physics: widget.isNested
                    ? const NeverScrollableScrollPhysics()
                    : null,
                shrinkWrap: widget.isNested,
                builderDelegate: PagedChildBuilderDelegate<VideoChannelDetails>(
                  itemBuilder: (context, item, index) => VideoChannelMiniature(
                    videoChannel: item,
                    config: widget.config,
                  ),
                  noItemsFoundIndicatorBuilder: (_) => const SizedBox.shrink(),
                ),
              ),
            ),
          )
        : Column(
            children: [
              SizedBox(
                height: 200,
                child: PagedListView<int, VideoChannelDetails>(
                  scrollDirection: Axis.horizontal,
                  pagingController: _pagingController,
                  builderDelegate:
                      PagedChildBuilderDelegate<VideoChannelDetails>(
                    itemBuilder: (context, item, index) =>
                        VideoChannelMiniature(
                      videoChannel: item,
                      config: widget.config,
                    ),
                    noItemsFoundIndicatorBuilder: (_) =>
                        const SizedBox.shrink(),
                  ),
                ),
              ),
            ],
          );
  }

  @override
  void dispose() {
    _pagingController.dispose();
    super.dispose();
  }
}
