import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:intl/intl.dart';
import 'package:lucide_icons/lucide_icons.dart';
import 'package:markdown/markdown.dart' as md;

import 'package:peertube_mobile/features/video/channel/channel_model.dart';
import 'package:peertube_mobile/features/video/video_providers.dart';
import 'package:peertube_mobile/shared/config/config_models.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/avatar_builder.dart';
import 'package:peertube_mobile/shared/widgets/button.dart';
import 'package:peertube_mobile/shared/widgets/dialog.dart';
import 'package:peertube_mobile/shared/widgets/html.dart';

class VideoChannelMiniature extends ConsumerWidget {
  final VideoChannelDetails videoChannel;
  final ConfigServer config;
  final bool isLocalAccount;
  final bool showStatsAndDescription;

  const VideoChannelMiniature({
    super.key,
    required this.videoChannel,
    required this.config,
    this.isLocalAccount = false,
    this.showStatsAndDescription = true,
  });

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final colorScheme = Theme.of(context).colorScheme;
    final textTheme = Theme.of(context).textTheme;
    return Padding(
      padding: const EdgeInsets.all(8.0),
      child: GestureDetector(
        onTap: () {
          GoRouter.of(context).pushReplacement(
            '/video-channel/${videoChannel.name}${videoChannel.isLocal ? '' : '@${videoChannel.host}'}',
          );
        },
        child: Stack(
          children: [
            Container(
              height: showStatsAndDescription ||
                      (isLocalAccount && videoChannel.support == null)
                  ? 93
                  : 130,
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(2.0),
                color: colorScheme.surfaceContainerHighest,
              ),
              margin: const EdgeInsets.only(left: 16),
              padding: const EdgeInsets.only(
                left: 56,
                top: 16,
                right: 16,
                bottom: 16,
              ),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text(
                    videoChannel.displayName,
                    style: textTheme.displayLarge,
                  ),
                  const SizedBox(
                    height: 2,
                  ),
                  if (showStatsAndDescription)
                    Row(
                      children: [
                        Icon(
                          LucideIcons.youtube,
                          size: 12,
                          color: colorScheme.onTertiary,
                        ),
                        const SizedBox(
                          width: 4,
                        ),
                        AsyncValueWidget(
                          value: ref.watch(
                            totalVideosProvider(
                              '/video-channels/${videoChannel.name}${videoChannel.isLocal ? '' : '@${videoChannel.host}'}/videos',
                            ),
                          ),
                          dataBuilder: (total) => Text(
                            t(context)!.totalVideos(
                              NumberFormat.decimalPattern(
                                ref.watch(localeProvider).languageCode,
                              ).format(total),
                            ),
                            style: textTheme.displayMedium!.copyWith(
                              color: colorScheme.onTertiary,
                            ),
                          ),
                        ),
                        const SizedBox(
                          width: 12,
                        ),
                        Icon(
                          LucideIcons.users,
                          size: 12,
                          color: colorScheme.onTertiary,
                        ),
                        const SizedBox(
                          width: 4,
                        ),
                        Text(
                          t(context)!.videochannelFollowersCount(
                            NumberFormat.decimalPattern(
                              ref.watch(localeProvider).languageCode,
                            ).format(videoChannel.followersCount),
                          ),
                          style: textTheme.displayMedium!.copyWith(
                            color: colorScheme.onTertiary,
                          ),
                        ),
                      ],
                    ),
                  if (!showStatsAndDescription)
                    Text(
                      t(context)!.onThePlatform(config.instance.name),
                      style: textTheme.displayMedium!
                          .copyWith(color: colorScheme.onSecondary),
                    ),
                  SizedBox(
                    height: showStatsAndDescription ? 6 : 14,
                  ),
                  if (videoChannel.description != null &&
                      showStatsAndDescription)
                    Text(
                      videoChannel.description!,
                      style: textTheme.displayMedium!.copyWith(
                        color: colorScheme.onSecondary,
                      ),
                      maxLines: 1,
                      overflow: TextOverflow.ellipsis,
                    ),
                  if (!showStatsAndDescription)
                    Row(
                      children: [
                        if (!isLocalAccount)
                          Expanded(
                            flex: 1,
                            child: TextButton(
                              onPressed: () {},
                              child: Text(t(context)!.subscribe),
                            ),
                          ),
                        if (videoChannel.support != null) ...{
                          const SizedBox(
                            width: 48,
                          ),
                          Expanded(
                            flex: 1,
                            child: UnderlinedTextButton(
                              onPressed: () => showDialog(
                                context: context,
                                builder: (context) => CustomDialog(
                                  content: Padding(
                                    padding: const EdgeInsets.only(top: 4.0),
                                    child: CustomHtmlWidget(
                                      rawHtml: md.markdownToHtml(
                                        videoChannel.support!,
                                      ),
                                    ),
                                  ),
                                ),
                              ),
                              text: t(context)!.support,
                              textStyle: textTheme.displayLarge!,
                              alignment: Alignment.centerLeft,
                            ),
                          ),
                        },
                      ],
                    ),
                ],
              ),
            ),
            Padding(
              padding: const EdgeInsets.only(top: 16.0),
              child: AvatarBuilder(
                domain: ref
                    .watch(
                      currentPlatformNonNullableProvider,
                    )
                    .domain,
                name: videoChannel.displayName,
                size: 55,
                borderRadius: 2,
                actorImageList: videoChannel.avatars,
              ),
            ),
            Padding(
              padding: const EdgeInsets.only(left: 32.0, top: 48.0),
              child: AvatarBuilder(
                domain: 'https://${videoChannel.host}',
                name: config.instance.name,
                size: 28,
                borderRadius: 6,
                actorImageList: config.instance.avatars,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
