import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:infinite_scroll_pagination/infinite_scroll_pagination.dart';

import 'package:peertube_mobile/features/video/comments/comment_model.dart';
import 'package:peertube_mobile/features/video/comments/comment_providers.dart';
import 'package:peertube_mobile/features/video/comments/comment_tile.dart';
import 'package:peertube_mobile/shared/filters/comment/comment_params_model.dart';

class VideoCommentListView extends ConsumerStatefulWidget {
  final CommentParams params;
  final bool isLocalAccount;
  final String videoUuid;
  final bool isNested;

  const VideoCommentListView({
    super.key,
    required this.params,
    required this.isLocalAccount,
    required this.videoUuid,
    this.isNested = true,
  });

  @override
  ConsumerState<VideoCommentListView> createState() => _InstanceListViewState();
}

class _InstanceListViewState extends ConsumerState<VideoCommentListView> {
  static const _pageSize = 20;

  final PagingController<int, VideoCommentThread> _pagingController =
      PagingController(firstPageKey: 0);

  @override
  Widget build(BuildContext context) {
    return PagedListView<int, VideoCommentThread>(
      pagingController: _pagingController,
      physics: widget.isNested ? NeverScrollableScrollPhysics() : null,
      shrinkWrap: widget.isNested,
      builderDelegate: PagedChildBuilderDelegate<VideoCommentThread>(
        itemBuilder: (_, item, index) => CommentTile(
          commentThread: item,
          isLocalAccount: widget.isLocalAccount,
          index: index,
        ),
        noItemsFoundIndicatorBuilder: (context) => Padding(
          padding: const EdgeInsets.all(16.0),
          child: Text('No comments.'),
        ),
      ),
    );
  }

  @override
  void didUpdateWidget(VideoCommentListView oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.params != widget.params) {
      _pagingController.refresh();
    }
  }

  @override
  void dispose() {
    _pagingController.dispose();
    super.dispose();
  }

  @override
  void initState() {
    _pagingController.addPageRequestListener((pageKey) {
      _fetchPage(pageKey);
    });
    super.initState();
  }

  Future<void> _fetchPage(int pageKey) async {
    try {
      final newItems =
          (await ref.read(videoCommentDatasourceProvider).fetchThreads(
                    widget.videoUuid,
                    commonParams: widget.params
                        .copyWith(start: pageKey, count: _pageSize),
                  ))
              .where((item) => !item.isDeleted)
              .toList();
      if (!mounted) {
        return;
      }
      final isLastPage = newItems.length < _pageSize;
      if (isLastPage) {
        _pagingController.appendLastPage(newItems);
      } else {
        final nextPageKey = pageKey + newItems.length;
        _pagingController.appendPage(newItems, nextPageKey);
      }
    } catch (error) {
      _pagingController.error = error;
    }
  }
}
