import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:lucide_icons/lucide_icons.dart';
import 'package:markdown/markdown.dart' as md;

import 'package:peertube_mobile/features/video/comments/comment_model.dart';
import 'package:peertube_mobile/features/video/comments/comment_providers.dart';
import 'package:peertube_mobile/features/video/comments/thread_modal.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/utils/time.dart';
import 'package:peertube_mobile/shared/widgets/avatar_builder.dart';
import 'package:peertube_mobile/shared/widgets/html.dart';

class CommentTile extends ConsumerWidget {
  final dynamic commentThread;
  final bool isLocalAccount;
  final int index;
  final bool isParent;

  const CommentTile({
    super.key,
    required this.commentThread,
    required this.isLocalAccount,
    required this.index,
    this.isParent = false,
  });

  @override
  Widget build(context, ref) {
    final VideoCommentThread comment = commentThread is VideoCommentThreadTree
        ? commentThread.comment
        : commentThread as VideoCommentThread;
    final colorScheme = Theme.of(context).colorScheme;
    final textTheme = Theme.of(context).textTheme;
    final processedHtml = md.markdownToHtml(comment.text);
    return Container(
      decoration: BoxDecoration(
        color: colorScheme.surfaceContainerHigh,
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(16),
          topRight: Radius.circular(16),
          bottomRight: Radius.circular(index % 2 == 0 ? 0 : 16),
          bottomLeft: Radius.circular(index % 2 == 0 ? 16 : 0),
        ),
      ),
      margin: const EdgeInsets.symmetric(vertical: 8, horizontal: 16),
      padding: const EdgeInsets.only(bottom: 16, left: 16, right: 16),
      child: Column(
        children: [
          ListTile(
            contentPadding: EdgeInsets.all(0),
            leading: comment.account != null
                ? AvatarBuilder(
                    domain:
                        ref.watch(currentPlatformNonNullableProvider).domain,
                    name: comment.account!.displayName,
                    size: 41,
                    borderRadius: 25,
                    actorImageList: comment.account!.avatars,
                  )
                : SizedBox.shrink(),
            trailing: IconButton(
              onPressed: () => GoRouter.of(context).push(
                '/report/comments/${comment.id}?isLocalAccount=$isLocalAccount&host=${comment.account?.host}',
              ),
              icon: const Icon(LucideIcons.flag),
            ),
            title: Text(
              comment.account?.displayName ?? '',
              style: textTheme.headlineMedium,
            ),
            subtitle: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Row(
                  children: [
                    Text(
                      timeago(comment.createdAt, context),
                      style: textTheme.displaySmall!
                          .copyWith(color: colorScheme.onSecondary),
                    ),
                  ],
                ),
              ],
            ),
          ),
          if (comment.isDeleted)
            Text(
              'Comment deleted',
              style: textTheme.displayMedium!.copyWith(
                fontStyle: FontStyle.italic,
              ),
            )
          else
            CustomHtmlWidget(
              rawHtml: processedHtml,
            ),
          if (!isParent && comment.totalReplies > 0) ...[
            SizedBox(width: 16),
            Row(
              children: [
                Padding(
                  padding: const EdgeInsets.only(top: 2.0),
                  child: Icon(
                    LucideIcons.messageSquare,
                    size: 16,
                  ),
                ),
                TextButton(
                  style: TextButton.styleFrom(
                    side: BorderSide.none,
                  ),
                  onPressed: () async {
                    final threadTree = await ref
                        .read(videoCommentDatasourceProvider)
                        .fetchThread(comment.videoId, comment.id);

                    if (context.mounted) {
                      showModalBottomSheet(
                        context: context,
                        isScrollControlled: true,
                        builder: (context) => CommentThreadModal(
                          threadTree: threadTree,
                          isLocalAccount: isLocalAccount,
                        ),
                      );
                    }
                  },
                  child: Container(
                    decoration: BoxDecoration(
                      border: Border(
                        bottom: BorderSide(
                          width: 1.0,
                          color: colorScheme.onPrimary,
                        ),
                      ),
                    ),
                    child: Padding(
                      padding: const EdgeInsets.all(1.0),
                      child: Text(
                        '${comment.totalReplies} replies',
                        style: textTheme.displayLarge!,
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ],
        ],
      ),
    );
  }
}
