import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:video_player/video_player.dart';
import 'package:wakelock_plus/wakelock_plus.dart';

import 'package:peertube_mobile/features/account/connected_account/local_account/local_account_providers.dart';
import 'package:peertube_mobile/features/video/captions/captions_providers.dart';
import 'package:peertube_mobile/features/video/chapters/chapters_providers.dart';
import 'package:peertube_mobile/features/video/player/video_player_controls.dart';
import 'package:peertube_mobile/features/video/player/video_progress_indicator.dart';
import 'package:peertube_mobile/features/video/storyboards/storyboards_providers.dart';
import 'package:peertube_mobile/features/video/video_model.dart';
import 'package:peertube_mobile/features/video/video_providers.dart';
import 'package:peertube_mobile/routes/settings/playback_settings_screen.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';

final StateProvider<VideoPlayerController?> videoPlayerControllerProvider =
    StateProvider<VideoPlayerController?>(
  (Ref<VideoPlayerController?> ref) => null,
);

class PeerTubeVideoPlayer extends ConsumerWidget {
  const PeerTubeVideoPlayer({
    super.key,
    required this.videoPlayerController,
    this.isLive = false,
    this.showControls = true,
  });

  final VideoPlayerController videoPlayerController;

  final bool isLive;
  final bool showControls;

  @override
  Widget build(context, ref) {
    if (videoPlayerController.value.isInitialized) {
      double aspectRatio = videoPlayerController.value.aspectRatio;
      if (aspectRatio < 1) {
        aspectRatio = 16 / 9;
      }

      return Container(
        alignment: Alignment.topCenter,
        child: Stack(
          children: [
            AspectRatio(
              aspectRatio: aspectRatio,
              child: Stack(
                children: [
                  Container(
                    margin: EdgeInsets.only(
                      bottom:
                          showControls ? videoProgressIndicatorHeight / 2 : 0.0,
                    ),
                    color: Colors.black,
                    child: Center(
                      child: AspectRatio(
                        aspectRatio: videoPlayerController.value.aspectRatio,
                        child: VideoPlayer(videoPlayerController),
                      ),
                    ),
                  ),
                  ValueListenableBuilder<VideoPlayerValue>(
                    valueListenable: videoPlayerController,
                    builder: (context, value, child) => ClosedCaption(
                      text: value.caption.text,
                      textStyle: Theme.of(context).textTheme.displayLarge,
                    ),
                  ),
                  if (showControls)
                    CustomControls(
                      videoPlayerController: videoPlayerController,
                      isLive: isLive,
                    ),
                ],
              ),
            ),
          ],
        ),
      );
    } else {
      return Container();
    }
  }
}

Future<void> initializePlayer(
  BuildContext context,
  WidgetRef ref,
  VideoDetails video,
) async {
  String videoUrl = '';

  if (video.streamingPlaylists.isNotEmpty) {
    videoUrl = video.streamingPlaylists[0].playlistUrl.toString();
  } else if (video.files.isNotEmpty) {
    videoUrl = video.files[0].fileUrl.toString();
  }

  final videoPlayerController = VideoPlayerController.networkUrl(
    Uri.parse(videoUrl),
    videoPlayerOptions: VideoPlayerOptions(
      allowBackgroundPlayback: true,
    ),
  );
  await videoPlayerController.initialize().then((_) {
    if (ref.read(autoPlayVideoProvider)) videoPlayerController.play();
  });
  if (context.mounted) {
    ref.read(videoPlayerControllerProvider.notifier).state =
        videoPlayerController;
  }
  videoPlayerController.addListener(() async {
    if (videoPlayerController.value.isPlaying &&
        !(await WakelockPlus.enabled)) {
      WakelockPlus.enable();
    } else if (!videoPlayerController.value.isPlaying &&
        await WakelockPlus.enabled) {
      WakelockPlus.disable();
    }
  });

  final historyElement = await ref
      .read(localAccountServiceProvider)
      .getHistoryElement(videoUuid: video.uuid);
  if (historyElement != null) {
    await videoPlayerController.seekTo(
      Duration(
        seconds: historyElement.currentPosition,
      ),
    );
  } else {
    await ref.read(localHistoryProvider.notifier).addToHistory(
          video,
          ref.read(currentPlatformNonNullableProvider).domain,
          0,
        );
  }

  final captionsList =
      await ref.read(videoDatasourceProvider).fetchCaptions(video.uuid);
  if (context.mounted) {
    ref.read(currentCaptionsListProvider.notifier).state = captionsList;
  }

  final chapters =
      await ref.read(videoDatasourceProvider).fetchChapters(video.uuid);
  if (context.mounted) {
    ref.read(currentChapterListProvider.notifier).state = chapters;
  }

  final storyboards =
      await ref.read(videoDatasourceProvider).fetchStoryboards(video.uuid);
  if (context.mounted) {
    ref.read(currentStoryboardProvider.notifier).state = storyboards;
  }
}

class FullScreenPlayer extends ConsumerWidget {
  const FullScreenPlayer({super.key});

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final videoController = ref.watch(videoPlayerControllerProvider);

    if (videoController != null) {
      if (videoController.value.aspectRatio >= 1.0) {
        SystemChrome.setPreferredOrientations([
          DeviceOrientation.landscapeRight,
          DeviceOrientation.landscapeLeft,
        ]);
      } else {
        SystemChrome.setPreferredOrientations([
          DeviceOrientation.portraitUp,
          DeviceOrientation.portraitDown,
        ]);
      }
    }

    SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersiveSticky);

    return PopScope(
      canPop: false,
      onPopInvokedWithResult: (didPop, __) async {
        if (didPop) {
          return;
        }
        SystemChrome.setPreferredOrientations([
          DeviceOrientation.portraitUp,
        ]);
        SystemChrome.setEnabledSystemUIMode(
          SystemUiMode.manual,
          overlays: SystemUiOverlay.values,
        );
        Navigator.pop(context);
      },
      child: Scaffold(
        body: videoController == null
            ? const Center(
                child: Text('No video selected'),
              )
            : Stack(
                children: [
                  Center(
                    child: AspectRatio(
                      aspectRatio: videoController.value.aspectRatio,
                      child: VideoPlayer(videoController),
                    ),
                  ),
                  ValueListenableBuilder<VideoPlayerValue>(
                    valueListenable: videoController,
                    builder: (context, value, child) => ClosedCaption(
                      text: value.caption.text,
                      textStyle:
                          Theme.of(context).textTheme.titleMedium!.copyWith(
                                fontWeight: FontWeight.normal,
                                color: Colors.white,
                              ),
                    ),
                  ),
                  CustomControls(
                    videoPlayerController: videoController,
                    isLive: ref.watch(selectedVideoProvider)!.isLive,
                    isFullScreen: true,
                  ),
                ],
              ),
      ),
    );
  }
}
