import 'dart:async';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:http/http.dart' as http;
import 'package:lucide_icons/lucide_icons.dart';
import 'package:video_player/video_player.dart';

import 'package:peertube_mobile/features/video/captions/captions_model.dart';
import 'package:peertube_mobile/features/video/captions/captions_providers.dart';
import 'package:peertube_mobile/features/video/chapters/chapters_providers.dart';
import 'package:peertube_mobile/features/video/miniplayer/widgets/play_button.dart';
import 'package:peertube_mobile/features/video/player/controls_notifier.dart';
import 'package:peertube_mobile/features/video/player/video_player_duration.dart';
import 'package:peertube_mobile/features/video/player/video_player_providers.dart';
import 'package:peertube_mobile/features/video/player/video_progress_indicator.dart';
import 'package:peertube_mobile/features/video/player/video_storyboards.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/themes/color_schemes.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';

class CustomControls extends ConsumerStatefulWidget {
  const CustomControls({
    super.key,
    required this.videoPlayerController,
    this.isLive = false,
    this.isFullScreen = false,
  });
  final VideoPlayerController videoPlayerController;
  final bool isLive;
  final bool isFullScreen;

  @override
  ConsumerState createState() => _CustomControlsState();
}

class _CustomControlsState extends ConsumerState<CustomControls> {
  bool _showSeekAnimation = false;
  String _seekDirection = 'left';
  int _totalSeekSeconds = 0;
  double _playbackSpeed = 1.0;
  bool _controllerWasPlaying = false;

  final List<double> _playbackSpeeds = [
    0.25,
    0.5,
    0.75,
    1.0,
    1.5,
    1.75,
    2.0,
    3.0,
  ];

  void _seekVideo(int seconds) {
    final currentPosition = widget.videoPlayerController.value.position;
    final targetPosition = currentPosition + Duration(seconds: seconds);
    widget.videoPlayerController.seekTo(targetPosition);

    setState(() {
      _showSeekAnimation = true;
      _seekDirection = seconds < 0 ? 'left' : 'right';
    });

    // Hide the seek animation after 1 second, then reset total seek seconds
    Future.delayed(const Duration(milliseconds: 500), () {
      setState(() {
        _showSeekAnimation = false;
      });

      Future.delayed(const Duration(milliseconds: 350), () {
        setState(() {
          _totalSeekSeconds = 0;
        });
      });
    });
  }

  void _handleDoubleTap(TapDownDetails details) {
    double screenWidth = MediaQuery.of(context).size.width;
    double tapPosition = details.localPosition.dx;
    int seekSeconds = 10;

    if (tapPosition < screenWidth / 3) {
      // Left double tap, rewind 10 seconds
      _totalSeekSeconds -= seekSeconds;
      _seekVideo(_totalSeekSeconds);
    } else if (tapPosition > 2 * screenWidth / 3) {
      // Right double tap, forward 10 seconds
      _totalSeekSeconds += seekSeconds;
      _seekVideo(_totalSeekSeconds);
    } else {
      // Center double tap to pause/play
      widget.videoPlayerController.value.isPlaying
          ? widget.videoPlayerController.pause()
          : widget.videoPlayerController.play();
      ref.read(showControlsProvider.notifier).showTemporarily();
    }
  }

  void _handleTap(TapDownDetails details) {
    final showControls = ref.watch(showControlsProvider);
    final showControlsNotifier = ref.read(showControlsProvider.notifier);
    final widgetSize = context.size!;
    double tapX = details.localPosition.dx;
    double tapY = details.localPosition.dy;

    if (showControls &&
        tapY > widgetSize.height - videoProgressIndicatorHeight) {
      seekToRelativePosition(details.globalPosition);
      ref.read(dragPositionProvider.notifier).state = null;
    } else if (showControls &&
        tapX > widgetSize.width / 3 &&
        tapX < 2 * widgetSize.width / 3 &&
        tapY > widgetSize.height / 3 &&
        tapY < 2 * widgetSize.height / 3) {
      if (widget.videoPlayerController.value.isPlaying) {
        widget.videoPlayerController.pause();
        ref.read(showControlsProvider.notifier).showPermanently();
      } else {
        widget.videoPlayerController.play();
        ref.read(showControlsProvider.notifier).showTemporarily();
      }
    } else {
      showControls
          ? showControlsNotifier.hide()
          : showControlsNotifier.showTemporarily();
    }
  }

  void _handleHorizontalDragStart(DragStartDetails details) {
    if (ref.watch(showControlsProvider) &&
        details.localPosition.dy >
            context.size!.height - videoProgressIndicatorHeight) {
      ref.read(showControlsProvider.notifier).showPermanently();
      if (!widget.videoPlayerController.value.isInitialized) {
        return;
      }
      _controllerWasPlaying = widget.videoPlayerController.value.isPlaying;
      if (_controllerWasPlaying) {
        widget.videoPlayerController.pause();
      }
    }
  }

  void _handleHorizontalDragEnd(DragEndDetails details) {
    if (ref.watch(showControlsProvider) &&
        details.localPosition.dy >
            context.size!.height - videoProgressIndicatorHeight) {
      ref.read(dragPositionProvider.notifier).state = null;
      if (_controllerWasPlaying &&
          widget.videoPlayerController.value.position !=
              widget.videoPlayerController.value.duration) {
        widget.videoPlayerController.play();
      }
      ref.read(showControlsProvider.notifier).showTemporarily();
    }
  }

  void _handleHorizontalDragUpdate(DragUpdateDetails details) {
    if (ref.watch(showControlsProvider) &&
        details.localPosition.dy >
            context.size!.height - videoProgressIndicatorHeight) {
      if (!widget.videoPlayerController.value.isInitialized) {
        return;
      }
      seekToRelativePosition(details.globalPosition);
    }
  }

  @override
  Widget build(context) {
    final textTheme = Theme.of(context).textTheme;
    final colorScheme = Theme.of(context).colorScheme;
    final chapters = ref.watch(currentChapterListProvider);
    final currentCaptions = ref.watch(currentCaptionsProvider);
    final captionsList = ref.watch(currentCaptionsListProvider);
    final showControls = ref.watch(showControlsProvider);
    final showControlsNotifier = ref.read(showControlsProvider.notifier);
    return GestureDetector(
      behavior: showControls
          ? HitTestBehavior.deferToChild
          : HitTestBehavior.translucent,
      onDoubleTapDown: _handleDoubleTap,
      onTapDown: _handleTap,
      onHorizontalDragStart: _handleHorizontalDragStart,
      onHorizontalDragUpdate: _handleHorizontalDragUpdate,
      onHorizontalDragEnd: _handleHorizontalDragEnd,
      child: Stack(
        children: [
          AbsorbPointer(
            absorbing: !showControls,
            child: AnimatedOpacity(
              opacity: showControls ? 1.0 : 0.0,
              duration: const Duration(milliseconds: 300),
              child: Stack(
                children: [
                  LayoutBuilder(
                    builder: (context, constraints) {
                      return Container(
                        height: widget.isFullScreen
                            ? constraints.maxHeight
                            : constraints.maxHeight -
                                videoProgressIndicatorHeight / 2,
                        decoration: BoxDecoration(
                          color: const Color(0xff121212).withValues(alpha: 0.8),
                        ),
                      );
                    },
                  ),
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 16.0),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: [
                        if (chapters.isNotEmpty) ...{
                          const Icon(
                            LucideIcons.bookOpen,
                            size: 24,
                            color: Colors.white,
                          ),
                          const SizedBox(
                            width: 12,
                          ),
                          ValueListenableBuilder(
                            valueListenable: widget.videoPlayerController,
                            builder: (context, VideoPlayerValue value, _) {
                              int index = chapters.lastIndexWhere(
                                (chapter) =>
                                    value.position.inSeconds >=
                                    chapter.timecode,
                              );
                              if (index == -1) return SizedBox.shrink();
                              return Text(
                                '${index + 1}. ${chapters[index].title}',
                                style: textTheme.displaySmall!.copyWith(
                                  color: Colors.white,
                                ),
                              );
                            },
                          ),
                        },
                        const Spacer(),
                        Row(
                          children: [
                            if (captionsList.isNotEmpty)
                              IconButton(
                                icon: Icon(
                                  currentCaptions == null
                                      ? Icons.closed_caption_outlined
                                      : Icons.closed_caption,
                                  color: Colors.white,
                                  semanticLabel:
                                      t(context)!.enableClosedCaptions,
                                ),
                                onPressed: () {
                                  showControlsNotifier.showTemporarily();
                                  if (currentCaptions == null) {
                                    Captions? captions;
                                    try {
                                      captions = captionsList.firstWhere(
                                        (caption) =>
                                            caption.language.id ==
                                            ref
                                                .read(localeProvider)
                                                .languageCode,
                                      );
                                    } catch (e) {
                                      captions = null;
                                    }
                                    if (captions != null) {
                                      ref
                                          .read(
                                            currentCaptionsProvider.notifier,
                                          )
                                          .state = captions;
                                      widget.videoPlayerController
                                          .setClosedCaptionFile(
                                        _loadCaptions(captions),
                                      );
                                    } else {
                                      showControlsNotifier.showPermanently();
                                      _showCaptionsModal(context);
                                    }
                                  } else {
                                    ref
                                        .read(
                                          currentCaptionsProvider.notifier,
                                        )
                                        .state = null;
                                    widget.videoPlayerController
                                        .setClosedCaptionFile(null);
                                  }
                                },
                              ),
                            IconButton(
                              icon: Icon(
                                LucideIcons.settings,
                                color: Colors.white,
                                semanticLabel: t(context)!.videoSettings,
                              ),
                              onPressed: () {
                                showControlsNotifier.showPermanently();

                                _showSettingsModal(context);
                              },
                            ),
                          ],
                        ),
                      ],
                    ),
                  ),
                  Center(
                    child: IgnorePointer(
                      ignoring: true,
                      child: PlayButton(
                        videoPlayerController: widget.videoPlayerController,
                        size: 50,
                        backgroundColor: beige300.withValues(alpha: 0.14),
                        padding: const EdgeInsets.all(10.0),
                      ),
                    ),
                  ),
                  Positioned(
                    bottom: 30,
                    width: MediaQuery.of(context).size.width,
                    child: Padding(
                      padding: const EdgeInsets.symmetric(horizontal: 16.0),
                      child: Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: [
                          ref.watch(dragPositionProvider) == null
                              ? widget.isLive
                                  ? Row(
                                      children: [
                                        Center(
                                          child: Container(
                                            width: 6,
                                            height: 6,
                                            decoration: BoxDecoration(
                                              shape: BoxShape.circle,
                                              color: colorScheme.primary,
                                            ),
                                          ),
                                        ),
                                        const SizedBox(
                                          width: 5,
                                        ),
                                        Text(
                                          t(context)!.live.toUpperCase(),
                                          style: Theme.of(context)
                                              .textTheme
                                              .labelMedium!
                                              .copyWith(
                                                fontWeight: FontWeight.bold,
                                                color: Colors.white,
                                              ),
                                        ),
                                      ],
                                    )
                                  : DurationDisplay(
                                      controller: widget.videoPlayerController,
                                    )
                              : const SizedBox.shrink(),
                          IconButton(
                            icon: Icon(
                              widget.isFullScreen
                                  ? LucideIcons.minimize2
                                  : LucideIcons.maximize2,
                              color: Colors.white,
                              semanticLabel: widget.isFullScreen
                                  ? t(context)!.exitFullScreen
                                  : t(context)!.enterFullScreen,
                            ),
                            onPressed: () {
                              if (widget.isFullScreen) {
                                SystemChrome.setPreferredOrientations([
                                  DeviceOrientation.portraitUp,
                                ]);
                                SystemChrome.setEnabledSystemUIMode(
                                  SystemUiMode.manual,
                                  overlays: SystemUiOverlay.values,
                                );
                                GoRouter.of(context).pop();
                              } else {
                                GoRouter.of(context).push('/full-screen');
                              }
                            },
                          ),
                        ],
                      ),
                    ),
                  ),
                  StoryboardsLayer(
                    controller: widget.videoPlayerController,
                    showControls: showControls,
                  ),
                  Positioned(
                    bottom: 0,
                    child: CustomVideoProgressIndicator(
                      widget.videoPlayerController,
                      chapters:
                          chapters.map((chapter) => chapter.timecode).toList(),
                    ),
                  ),
                ],
              ),
            ),
          ),
          Positioned.fill(
            child: Center(
              child: AnimatedOpacity(
                opacity: _showSeekAnimation ? 1.0 : 0.0,
                duration: const Duration(milliseconds: 500),
                child: Row(
                  mainAxisAlignment: _seekDirection == 'left'
                      ? MainAxisAlignment.start
                      : MainAxisAlignment.end,
                  children: [
                    Container(
                      width: MediaQuery.of(context).size.width / 2,
                      margin: const EdgeInsets.only(
                        bottom: 20.0,
                      ),
                      decoration: BoxDecoration(
                        color: Colors.grey.withValues(alpha: 0.3),
                        borderRadius: _seekDirection == 'left'
                            ? const BorderRadius.only(
                                topRight: Radius.elliptical(50, 200),
                                bottomRight: Radius.elliptical(50, 200),
                              )
                            : const BorderRadius.only(
                                topLeft: Radius.elliptical(50, 200),
                                bottomLeft: Radius.elliptical(50, 200),
                              ),
                      ),
                      child: Column(
                        mainAxisAlignment: MainAxisAlignment.center,
                        children: [
                          Text(
                            _seekDirection == 'left'
                                ? '$_totalSeekSeconds seconds'
                                : '+$_totalSeekSeconds seconds',
                            style: const TextStyle(
                              color: Colors.white,
                              fontSize: 16,
                            ),
                          ),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  void _showSettingsModal(BuildContext context) {
    final textTheme = Theme.of(context).textTheme;
    Completer<bool> showTemporarilyControlsOnDismiss = Completer<bool>();
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      builder: (BuildContext context) {
        return PopScope(
          canPop: false,
          onPopInvokedWithResult: (didPop, __) async {
            if (didPop) {
              return;
            }
            showTemporarilyControlsOnDismiss.complete(true);
            Navigator.pop(context);
          },
          child: Container(
            padding: const EdgeInsets.all(16.0),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                if (widget.isLive == false)
                  ListTile(
                    leading: const Icon(LucideIcons.playCircle),
                    title: const Text('Speed'),
                    trailing: Text(
                      _playbackSpeed == 1.0 ? 'Normal' : '${_playbackSpeed}x',
                      style: textTheme.displayMedium,
                    ),
                    onTap: () {
                      Navigator.pop(context);
                      showTemporarilyControlsOnDismiss.complete(false);
                      _showPlaybackSpeedModal(context);
                    },
                  ),
                if (ref.watch(currentCaptionsListProvider).isNotEmpty)
                  ListTile(
                    leading: const Icon(LucideIcons.playCircle),
                    title: const Text('Subtitles/CC'),
                    trailing: Text(
                      ref.watch(currentCaptionsProvider) == null
                          ? 'Off'
                          : ref.watch(currentCaptionsProvider)!.language.label,
                      style: textTheme.displayMedium,
                    ),
                    onTap: () {
                      Navigator.pop(context);
                      showTemporarilyControlsOnDismiss.complete(false);
                      _showCaptionsModal(context);
                    },
                  ),
              ],
            ),
          ),
        );
      },
    ).whenComplete(() async {
      bool shouldShowTemp = await showTemporarilyControlsOnDismiss.future;
      if (shouldShowTemp) {
        ref.read(showControlsProvider.notifier).showTemporarily();
      }
    });
  }

  void _showPlaybackSpeedModal(BuildContext context) {
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      builder: (BuildContext context) {
        return SingleChildScrollView(
          child: Container(
            padding: const EdgeInsets.all(16.0),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              children: [
                const Text(
                  'Select Playback Speed',
                  style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
                ),
                ..._playbackSpeeds.map((speed) {
                  return ListTile(
                    title: Text(
                      speed == 1.0 ? 'Normal' : '${speed}x',
                    ),
                    trailing: _playbackSpeed == speed
                        ? const Icon(Icons.check)
                        : null,
                    onTap: () {
                      widget.videoPlayerController.setPlaybackSpeed(speed);
                      setState(() {
                        _playbackSpeed = speed;
                      });
                      Navigator.pop(context);
                    },
                  );
                }),
              ],
            ),
          ),
        );
      },
    ).whenComplete(() {
      ref.read(showControlsProvider.notifier).showTemporarily();
    });
  }

  void _showCaptionsModal(BuildContext context) {
    showModalBottomSheet(
      context: context,
      isScrollControlled: true,
      useSafeArea: true,
      builder: (BuildContext context) {
        return Container(
          padding: const EdgeInsets.all(16.0),
          child: ListView(
            children: [
              const Text(
                'Select language',
                style: TextStyle(fontSize: 18, fontWeight: FontWeight.bold),
              ),
              ListTile(
                title: const Text('Off'),
                trailing: ref.watch(currentCaptionsProvider) == null
                    ? const Icon(Icons.check)
                    : null,
                onTap: () {
                  ref.read(currentCaptionsProvider.notifier).state = null;
                  widget.videoPlayerController.setClosedCaptionFile(null);
                  Navigator.pop(context);
                },
              ),
              ...ref.watch(currentCaptionsListProvider).map((captions) {
                return ListTile(
                  title: Text(
                    captions.language.label,
                  ),
                  trailing: ref.watch(currentCaptionsProvider)?.language.id ==
                          captions.language.id
                      ? const Icon(Icons.check)
                      : null,
                  onTap: () {
                    ref.read(currentCaptionsProvider.notifier).state = captions;
                    widget.videoPlayerController
                        .setClosedCaptionFile(_loadCaptions(captions));
                    Navigator.pop(context);
                  },
                );
              }),
            ],
          ),
        );
      },
    ).whenComplete(() {
      ref.read(showControlsProvider.notifier).showTemporarily();
    });
  }

  Future<ClosedCaptionFile> _loadCaptions(Captions captions) async {
    final String url =
        Uri.parse(ref.read(currentPlatformNonNullableProvider).domain)
            .resolve(captions.captionPath)
            .toString();

    final http.Response response = await http.get(Uri.parse(url));

    if (response.statusCode == 200) {
      final String fileContents = response.body;
      return WebVTTCaptionFile(
        fileContents,
      );
    } else {
      throw Exception('Failed to load captions');
    }
  }

  void seekToRelativePosition(Offset globalPosition) {
    final RenderBox box = context.findRenderObject()! as RenderBox;
    final Offset tapPos = box.globalToLocal(globalPosition);
    final double relative = tapPos.dx / box.size.width;
    final Duration position =
        widget.videoPlayerController.value.duration * relative;
    widget.videoPlayerController.seekTo(position);
    ref.read(dragPositionProvider.notifier).state = relative;
  }
}
