import 'dart:async';
import 'dart:ui' as ui;

import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:video_player/video_player.dart';

import 'package:peertube_mobile/features/video/chapters/chapters_providers.dart';
import 'package:peertube_mobile/features/video/player/video_player_providers.dart';
import 'package:peertube_mobile/features/video/player/video_progress_indicator.dart';
import 'package:peertube_mobile/features/video/storyboards/storyboard_model.dart';
import 'package:peertube_mobile/features/video/storyboards/storyboards_providers.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/utils/time.dart';

class StoryboardsLayer extends ConsumerStatefulWidget {
  const StoryboardsLayer({
    super.key,
    required this.controller,
    required this.showControls,
  });

  final VideoPlayerController controller;
  final bool showControls;

  @override
  ConsumerState<StoryboardsLayer> createState() => _StoryboardsLayerState();
}

class _StoryboardsLayerState extends ConsumerState<StoryboardsLayer> {
  ui.Image? _image;
  bool _isLoading = true;

  VideoPlayerController get controller => widget.controller;
  bool get showControls => widget.showControls;

  Future<void> _loadImage() async {
    final storyboards = ref.read(currentStoryboardProvider);
    if (storyboards.isEmpty) return;
    final storyboard = storyboards[0];

    final image = await getImage(
      Uri.parse(
        ref.read(currentPlatformNonNullableProvider).domain,
      ).resolve(storyboard.storyboardPath).toString(),
    );
    if (!mounted) {
      return;
    }

    setState(() {
      _image = image;
      _isLoading = false;
    });
  }

  Widget? buildStoryboardOverlay(double relativePosition) {
    final storyboards = ref.watch(currentStoryboardProvider);
    Storyboard? storyboard;
    if (storyboards.isNotEmpty) {
      storyboard = storyboards[0];
      _loadImage();
    }
    final position = controller.value.position;

    final chapters = ref.watch(currentChapterListProvider);

    final currentChapter = chapters.lastIndexWhere(
      (chapter) => position.inSeconds >= chapter.timecode,
    );
    return Container(
      margin: EdgeInsets.only(
        bottom: showControls ? videoProgressIndicatorHeight / 2 : 0.0,
      ),
      color: Colors.black,
      child: Center(
        child: AspectRatio(
          aspectRatio: controller.value.aspectRatio,
          child: Stack(
            children: [
              _isLoading && storyboard != null
                  ? const CircularProgressIndicator()
                  : _image == null
                      ? const SizedBox.shrink()
                      : CustomPaint(
                          size: Size(
                            controller.value.size.width,
                            controller.value.size.width *
                                storyboard!.spriteHeight.toDouble() /
                                storyboard.spriteWidth.toDouble(),
                          ),
                          painter: SpriteImagePainter(
                            image: _image!,
                            storyboard: storyboard,
                            position: position.inSeconds,
                            duration: controller.value.duration.inSeconds,
                          ),
                        ),
              Column(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Center(
                    child: Container(
                      padding: const EdgeInsets.symmetric(
                        vertical: 4,
                        horizontal: 16,
                      ),
                      margin: const EdgeInsets.only(
                        bottom: videoProgressIndicatorHeight / 2 + 10,
                      ),
                      decoration: BoxDecoration(
                        color: Theme.of(context).colorScheme.secondary,
                        borderRadius: const BorderRadius.horizontal(
                          left: Radius.circular(30),
                          right: Radius.circular(30),
                        ),
                      ),
                      child: Text(
                        '${formatDuration(position)}  ${(chapters.isNotEmpty && currentChapter != -1) ? chapters[currentChapter].title : ''}',
                      ),
                    ),
                  ),
                ],
              ),
            ],
          ),
        ),
      ),
    );
  }

  Future<ui.Image> getImage(String path) async {
    final completer = Completer<ImageInfo>();
    final img = NetworkImage(path);
    img.resolve(ImageConfiguration.empty).addListener(
      ImageStreamListener((info, _) {
        completer.complete(info);
      }),
    );
    final imageInfo = await completer.future;

    return imageInfo.image;
  }

  @override
  Widget build(context) {
    final storyboards = ref.watch(currentStoryboardProvider);
    if (storyboards.isNotEmpty) {
      _loadImage();
    }
    final dragPosition = ref.watch(dragPositionProvider);
    return dragPosition != null
        ? buildStoryboardOverlay(dragPosition)!
        : const SizedBox.shrink();
  }
}

class SpriteImagePainter extends CustomPainter {
  final ui.Image image;
  final Storyboard storyboard;
  final int position;
  final int duration;

  SpriteImagePainter({
    required this.image,
    required this.storyboard,
    required this.position,
    required this.duration,
  });

  @override
  void paint(Canvas canvas, Size size) {
    int spritesPerRow = storyboard.totalWidth ~/ storyboard.spriteWidth;
    int totalSprites =
        spritesPerRow * (storyboard.totalHeight ~/ storyboard.spriteHeight);

    final int currentSpriteIndex = (position * totalSprites / duration).floor();

    final srcX = (currentSpriteIndex % spritesPerRow) * storyboard.spriteWidth;
    final srcY =
        (currentSpriteIndex ~/ spritesPerRow) * storyboard.spriteHeight;

    final srcRect = Rect.fromLTWH(
      srcX.toDouble(),
      srcY.toDouble(),
      storyboard.spriteWidth.toDouble(),
      storyboard.spriteHeight.toDouble(),
    );

    final dstRect = Rect.fromLTWH(0, 0, size.width, size.height);

    canvas.drawImageRect(image, srcRect, dstRect, Paint());
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}
