import 'dart:io';

import 'package:dio/dio.dart';

import 'package:peertube_mobile/features/account/account_model.dart';
import 'package:peertube_mobile/features/video/channel/channel_model.dart';
import 'package:peertube_mobile/features/video/playlist/playlist_model.dart';
import 'package:peertube_mobile/features/video/playlist/playlist_privacy_enum.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_models.dart';
import 'package:peertube_mobile/shared/filters/playlist/playlist_params_model.dart';
import 'package:peertube_mobile/shared/network/http_service.dart';

class PlaylistDatasource {
  final CurrentPlatform currentPlatform;
  final HttpService httpService;

  PlaylistDatasource({
    required this.currentPlatform,
    required this.httpService,
  });

  Future<List<Playlist>> fetchPlaylists(
    PlaylistParams? commonParams,
  ) async =>
      _fetchPlaylistLists(
        commonParams,
        '/video-playlists',
      );

  Future<List<Playlist>> fetchPlaylistsFromAccount(
    Account account, {
    PlaylistParams? commonParams,
  }) async {
    final accountName = account.host == Uri.parse(currentPlatform.domain).host
        ? account.name
        : '${account.name}@${account.host}';

    return _fetchPlaylistLists(
      commonParams,
      '/accounts/$accountName/video-playlists',
    );
  }

  Future<List<Playlist>> fetchPlaylistsFromChannel(
    VideoChannel channel, {
    PlaylistParams? commonParams,
  }) async {
    final channelName = channel.host == Uri.parse(currentPlatform.domain).host
        ? channel.name
        : '${channel.name}@${channel.host}';

    return _fetchPlaylistLists(
      commonParams,
      '/video-channels/$channelName/video-playlists',
    );
  }

  Future<Playlist> fetchPlaylist(String playlistId) async {
    final response = await httpService.get(
      path: currentPlatform.getApiBaseUrl('/video-playlists/$playlistId'),
    );

    dynamic playlitJson = response.data;

    try {
      return Playlist.fromJson(playlitJson);
    } catch (e) {
      throw Exception('An error has occurred : $e');
    }
  }

  Future<List<VideoPlaylistElement>> fetchPlaylistVideos(
    int playlistId, {
    PlaylistParams? commonParams,
  }) async {
    final PlaylistParams params = commonParams ?? const PlaylistParams();
    final Map<String, dynamic> queryParams = params.toJson();

    final Response response = await httpService.get(
      path:
          currentPlatform.getApiBaseUrl('/video-playlists/$playlistId/videos'),
      queryParameters: queryParams,
      expectedStatusCode: HttpStatus.ok,
    );

    return response.data['data'].map<VideoPlaylistElement>((json) {
      try {
        return VideoPlaylistElement.fromJson(json);
      } catch (e) {
        throw Exception('Error parsing playlist element: $e');
      }
    }).toList();
  }

  Future<Map<int, List<PlaylistElement>>> checkVideoExistsInPlaylist(
    List<int> videoIds,
  ) async {
    final queryParams = {'videoIds': videoIds.join(',')};

    final response = await httpService.get(
      path: currentPlatform
          .getApiBaseUrl('/users/me/video-playlists/videos-exist'),
      queryParameters: queryParams,
      expectedStatusCode: HttpStatus.ok,
    );

    try {
      final elementsJson = response.data as Map<String, dynamic>;

      final Map<int, List<PlaylistElement>> elements = {};

      for (final videoId in videoIds) {
        final elementList = elementsJson[videoId.toString()] as List<dynamic>?;

        elements[videoId] =
            elementList?.map((e) => PlaylistElement.fromJson(e)).toList() ?? [];
      }

      return elements;
    } catch (e) {
      throw Exception('Unexpected error: $e');
    }
  }

  Future<int> createPlaylist({
    required String displayName,
    required PlaylistPrivacy privacy,
    String? thumbnailPath,
    String? description,
    int? videoChannelId,
  }) async {
    final formData = {
      'displayName': displayName,
      'privacy': privacy,
      if (thumbnailPath != null) 'thumbnailPath': thumbnailPath,
      if (description != null) 'description': description,
      if (videoChannelId != null) 'videoChannelId': videoChannelId,
    };

    final response = await httpService.post(
      path: currentPlatform.getApiBaseUrl('/video-playlists'),
      body: FormData.fromMap(formData),
      expectedStatusCode: HttpStatus.ok,
    );

    return response.data['videoPlaylist']['id'];
  }

  Future<void> updatePlaylist({
    required int playlistId,
    String? displayName,
    PlaylistPrivacy? privacy,
    String? thumbnailPath,
    String? description,
    int? videoChannelId,
  }) async {
    final formData = {
      if (displayName != null) 'displayName': displayName,
      if (privacy != null) 'privacy': privacy,
      if (thumbnailPath != null) 'thumbnailPath': thumbnailPath,
      if (description != null) 'description': description,
      if (videoChannelId != null) 'videoChannelId': videoChannelId,
    };

    await httpService.post(
      path: currentPlatform.getApiBaseUrl('/video-playlists/$playlistId'),
      body: FormData.fromMap(formData),
      expectedStatusCode: 204,
    );
  }

  Future<int> addVideoInPlaylist({
    required int playlistId,
    required int videoId,
    int? startTimestamp,
    int? stopTimestamp,
  }) async {
    final formData = {
      'videoId': videoId,
      if (startTimestamp != null) 'startTimestamp': startTimestamp,
      if (stopTimestamp != null) 'stopTimestamp': stopTimestamp,
    };

    final response = await httpService.post(
      path:
          currentPlatform.getApiBaseUrl('/video-playlists/$playlistId/videos'),
      body: FormData.fromMap(formData),
      expectedStatusCode: HttpStatus.ok,
    );

    return response.data['videoPlaylistElement']['id'];
  }

  Future<void> updatePlaylistElement({
    required int playlistId,
    required int playlistElementId,
    required int startTimestamp,
    required int stopTimestamp,
  }) async {
    final formData = {
      'startTimestamp': startTimestamp,
      'stopTimestamp': stopTimestamp,
    };

    await httpService.put(
      path: currentPlatform.getApiBaseUrl(
        '/video-playlists/$playlistId/videos/$playlistElementId',
      ),
      body: FormData.fromMap(formData),
      expectedStatusCode: 204,
    );
  }

  Future<void> reorderPlaylist({
    required int playlistId,
    required int startPosition,
    required int insertAfterPosition,
    required int reorderLength,
  }) async {
    final formData = {
      'startPosition': startPosition,
      'insertAfterPosition': insertAfterPosition,
      'reorderLength': reorderLength,
    };

    await httpService.post(
      path: currentPlatform
          .getApiBaseUrl('/video-playlists/$playlistId/videos/reorder'),
      body: FormData.fromMap(formData),
      expectedStatusCode: 204,
    );
  }

  Future<void> deletePlaylist(
    int playlistId,
  ) async =>
      await httpService.delete(
        path: currentPlatform.getApiBaseUrl(
          '/video-playlists/$playlistId',
        ),
        expectedStatusCode: 204,
      );

  Future<void> deleteVideoFromPlaylist(
    int playlistId,
    int playlistElementId,
  ) async =>
      await httpService.delete(
        path: currentPlatform.getApiBaseUrl(
          '/video-playlists/$playlistId/videos/$playlistElementId',
        ),
        expectedStatusCode: 204,
      );

  Future<List<Playlist>> _fetchPlaylistLists(
    PlaylistParams? commonParams,
    String path,
  ) async {
    final params = commonParams ?? const PlaylistParams();
    final queryParams = params.toJson();

    final response = await httpService.get(
      path: currentPlatform.getApiBaseUrl(path),
      queryParameters: queryParams,
      expectedStatusCode: HttpStatus.ok,
    );

    return response.data['data'].map<Playlist>((json) {
      try {
        return Playlist.fromJson(json);
      } catch (e) {
        throw Exception('Error parsing playlist: $e');
      }
    }).toList();
  }
}
