import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:infinite_scroll_pagination/infinite_scroll_pagination.dart';
import 'package:intl/intl.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/features/video/video_list/video_miniature.dart';
import 'package:peertube_mobile/features/video/video_model.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/button.dart';
import 'package:peertube_mobile/shared/widgets/dialog.dart';

class VideoListView extends ConsumerStatefulWidget {
  final Future<List<Video>> Function(int pageKey, int pageSize) fetchPage;
  final bool isHorizontal;
  final bool showChannel;
  final bool isNested;
  final bool wantKeepAlive;
  final StateProvider? counterProvider;
  final StateProvider<bool>? refreshTriggerProvider;
  final String? domain;
  final bool isOnSepiaSearch;

  const VideoListView({
    super.key,
    required this.fetchPage,
    this.domain,
    this.isHorizontal = false,
    this.showChannel = true,
    this.isNested = false,
    this.wantKeepAlive = false,
    this.counterProvider,
    this.refreshTriggerProvider,
    this.isOnSepiaSearch = false,
  });

  @override
  ConsumerState<VideoListView> createState() => _VideoListViewState();
}

class _VideoListViewState extends ConsumerState<VideoListView>
    with AutomaticKeepAliveClientMixin<VideoListView> {
  final PagingController<int, Video> _pagingController =
      PagingController(firstPageKey: 0);
  late int _pageKey;
  bool _isFetching = false;
  int _totalLoadedVideos = 0;

  @override
  bool get wantKeepAlive => widget.wantKeepAlive;

  @override
  void initState() {
    _pagingController.addPageRequestListener((pageKey) {
      _pageKey = pageKey;
      if (_pageKey == 0 || _isFetching) {
        _fetchPage(pageKey);
      }
    });
    super.initState();
  }

  Future<void> _fetchPage(int pageKey) async {
    int pageSize = pageKey == 0
        ? 10
        : pageKey == 10
            ? 20
            : 30;

    try {
      final newItems = await widget.fetchPage(pageKey, pageSize);

      if (!mounted) {
        return;
      }

      final isLastPage = newItems.length < pageSize;

      setState(() {
        _totalLoadedVideos += newItems.length;
        _isFetching = false;
      });

      if (widget.counterProvider != null) {
        ref.read(widget.counterProvider!.notifier).state = _totalLoadedVideos;
      }

      if (isLastPage) {
        _pagingController.appendLastPage(newItems);
      } else {
        final nextPageKey = pageKey + newItems.length;
        _pagingController.appendPage(newItems, nextPageKey);
      }
    } catch (error) {
      _pagingController.error = error;
    }
  }

  Future<void> _refresh() async {
    setState(() {
      _isFetching = false;
      _totalLoadedVideos = 0;
    });
    _pagingController.refresh();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    if (widget.refreshTriggerProvider != null) {
      ref.listen<bool>(widget.refreshTriggerProvider!, (previous, next) {
        if (next) {
          _refresh();
          ref.read(widget.refreshTriggerProvider!.notifier).state = false;
        }
      });
    }
    return !widget.isHorizontal
        ? Semantics(
            label: t(context)!.videoList,
            onTap: _refresh,
            onTapHint: t(context)!.refreshList,
            child: RefreshIndicator(
              onRefresh: _refresh,
              child: PagedListView<int, Video>(
                pagingController: _pagingController,
                physics: widget.isNested
                    ? const NeverScrollableScrollPhysics()
                    : null,
                shrinkWrap: widget.isNested,
                builderDelegate: PagedChildBuilderDelegate<Video>(
                  itemBuilder: (context, item, index) => VideoMiniature(
                    video: item,
                    showChannel: widget.showChannel,
                    isOnSepiaSearch: widget.isOnSepiaSearch,
                    domain: widget.domain ?? 'https://${item.account.host}',
                  ),
                  newPageProgressIndicatorBuilder: loadMoreItemsBuilder,
                  noItemsFoundIndicatorBuilder: (_) => const SizedBox.shrink(),
                ),
              ),
            ),
          )
        : Column(
            children: [
              SizedBox(
                height: _totalLoadedVideos == 0 ? 0 : 200,
                child: PagedListView<int, Video>(
                  scrollDirection: Axis.horizontal,
                  pagingController: _pagingController,
                  builderDelegate: PagedChildBuilderDelegate<Video>(
                    itemBuilder: (context, item, index) => VideoMiniature(
                      video: item,
                      hasPadding: true,
                      isHorizontal: true,
                      showChannel: widget.showChannel,
                      isOnSepiaSearch: widget.isOnSepiaSearch,
                      domain: widget.domain ?? 'https://${item.account.host}',
                    ),
                    newPageProgressIndicatorBuilder: loadMoreItemsBuilder,
                    noItemsFoundIndicatorBuilder: (_) =>
                        const SizedBox.shrink(),
                  ),
                ),
              ),
            ],
          );
  }

  @override
  void dispose() {
    _pagingController.dispose();
    super.dispose();
  }

  Widget loadMoreItemsBuilder(BuildContext context) {
    return Column(
      children: [
        const SizedBox(height: 10),
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Text(
              t(context)!.totalVideosDisplayed(
                NumberFormat.decimalPattern(
                  ref.watch(localeProvider).languageCode,
                ).format(_totalLoadedVideos),
              ),
              style: Theme.of(context)
                  .textTheme
                  .displayMedium!
                  .copyWith(color: Theme.of(context).colorScheme.onSecondary),
            ),
            SizedBox(
              height: 48.0,
              width: 48.0,
              child: IconButton(
                padding: const EdgeInsets.all(2.0),
                iconSize: 16.0,
                icon: Icon(
                  LucideIcons.helpCircle,
                  semanticLabel: t(context)!.showInformation,
                ),
                onPressed: () => showDialog(
                  context: context,
                  builder: (BuildContext context) {
                    return TitleContentDialog(
                      content: t(context)!.infiniteScrollingDescription1,
                      secondContent: t(context)!.infiniteScrollingDescription2,
                      title: t(context)!.infiniteScrollingTitle,
                    );
                  },
                ),
              ),
            ),
          ],
        ),
        const SizedBox(height: 10),
        _isFetching
            ? const Column(
                children: [
                  SizedBox(height: 10),
                  CircularProgressIndicator(),
                  SizedBox(height: 10),
                ],
              )
            : CustomElevatedButton(
                onPressed: () {
                  setState(() {
                    _isFetching = true;
                  });
                  _pagingController.notifyPageRequestListeners(_pageKey);
                },
                child: Text(t(context)!.showMoreVideos),
              ),
        const SizedBox(height: 70),
      ],
    );
  }
}
