import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:intl/intl.dart';
import 'package:lucide_icons/lucide_icons.dart';
import 'package:share_plus/share_plus.dart';

import 'package:peertube_mobile/features/account/connected_account/connected_account_providers.dart';
import 'package:peertube_mobile/features/account/connected_account/local_account/local_account_model.dart';
import 'package:peertube_mobile/features/account/connected_account/local_account/local_account_providers.dart';
import 'package:peertube_mobile/features/platform/platform_miniature.dart';
import 'package:peertube_mobile/features/video/playlist/playlist_checkbox.dart';
import 'package:peertube_mobile/features/video/video_model.dart';
import 'package:peertube_mobile/routes/router.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/time.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/avatar_builder.dart';
import 'package:peertube_mobile/shared/widgets/image.dart';

class VideoMiniature extends ConsumerWidget {
  final Video video;
  final String domain;
  final bool hasPadding;
  final bool isHorizontal;
  final bool showChannel;
  final bool isOnSepiaSearch;
  final VoidCallback? onTap;

  const VideoMiniature({
    super.key,
    required this.video,
    required this.domain,
    this.hasPadding = false,
    this.isHorizontal = false,
    this.showChannel = true,
    this.isOnSepiaSearch = false,
    this.onTap,
  });

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    ColorScheme colorScheme = Theme.of(context).colorScheme;

    return SizedBox(
      width: isHorizontal ? 200 : double.infinity,
      child: Semantics(
        label: t(context)!.videoItem,
        onTapHint: t(context)!.watchThisVideo,
        child: GestureDetector(
          onTap: () async {
            if (isOnSepiaSearch || ref.watch(currentPlatformProvider) == null) {
              final success = await ref
                  .read(currentPlatformServiceProvider)
                  .loadPlatformFromLocalAccount(
                    context,
                    ref,
                    domain,
                  );
              if (context.mounted) {
                if (success) {
                  GoRouter.of(context).push(
                    '/video/${video.uuid}',
                  );
                } else {
                  await showPlatformNotRespondingDialog(
                    context,
                    video.account.host,
                  );
                }
              }
            } else {
              if (context.mounted) {
                GoRouter.of(context).push('/video/${video.uuid}');
              }
            }

            if (onTap != null) onTap!();
          },
          child: Padding(
            padding: EdgeInsets.symmetric(
              horizontal: hasPadding ? 4.0 : 0,
            ),
            child: AsyncValueWidget(
              value: ref.watch(currentConnectedAccountProvider),
              dataBuilder: (account) => Stack(
                children: [
                  Column(
                    children: <Widget>[
                      Stack(
                        children: <Widget>[
                          CustomCachedNetworkImage(
                            imageUrl: Uri.parse(
                              domain,
                            )
                                .resolve(
                                  video.previewPath ?? '',
                                )
                                .toString(),
                            height: isHorizontal ? 98.0 : 220.0,
                            width: double.infinity,
                          ),
                          if (account.isLocal())
                            FutureBuilder<LocalHistoryElementSQL?>(
                              future: ref
                                  .read(localAccountServiceProvider)
                                  .getHistoryElement(videoUuid: video.uuid),
                              builder: (context, snapshot) {
                                if (snapshot.connectionState ==
                                    ConnectionState.waiting) {
                                  return const Center(
                                    child: CircularProgressIndicator(),
                                  );
                                }

                                if (snapshot.hasError) {
                                  return Center(
                                    child: Text('Error: ${snapshot.error}'),
                                  );
                                }
                                final historyPosition =
                                    snapshot.data?.currentPosition ?? 0;
                                final position = (video.duration > 0)
                                    ? historyPosition / video.duration
                                    : 0.0;

                                return historyPosition != 0
                                    ? LinearProgressIndicator(
                                        value: position,
                                      )
                                    : const SizedBox.shrink();
                              },
                            ),
                          Positioned(
                            bottom: 0,
                            right: 0,
                            child: Container(
                              padding: const EdgeInsets.symmetric(
                                vertical: 3.0,
                                horizontal: 8.0,
                              ),
                              decoration: BoxDecoration(
                                borderRadius: const BorderRadius.only(
                                  topLeft: Radius.circular(5),
                                ),
                                color:
                                    colorScheme.surface.withValues(alpha: 0.8),
                              ),
                              child: video.isLive
                                  ? Row(
                                      children: [
                                        Center(
                                          child: Container(
                                            width: 4,
                                            height: 4,
                                            decoration: BoxDecoration(
                                              shape: BoxShape.circle,
                                              color: colorScheme.primary,
                                            ),
                                          ),
                                        ),
                                        const SizedBox(
                                          width: 5,
                                        ),
                                        Text(
                                          t(context)!.live.toUpperCase(),
                                          style: Theme.of(context)
                                              .textTheme
                                              .labelSmall!
                                              .copyWith(
                                                fontWeight: FontWeight.bold,
                                                color: colorScheme
                                                    .onSurfaceVariant,
                                              ),
                                        ),
                                      ],
                                    )
                                  : Text(
                                      formatDuration(
                                        Duration(seconds: video.duration),
                                      ),
                                      semanticsLabel:
                                          Duration(seconds: video.duration)
                                              .toString(),
                                      style: Theme.of(context)
                                          .textTheme
                                          .labelMedium!
                                          .copyWith(
                                            color: colorScheme.onSurfaceVariant,
                                          ),
                                    ),
                            ),
                          ),
                        ],
                      ),
                      Padding(
                        padding: const EdgeInsets.only(
                          top: 5.0,
                          left: 12.0,
                          right: 8.0,
                          bottom: 20.0,
                        ),
                        child: Row(
                          mainAxisAlignment: MainAxisAlignment.spaceBetween,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: <Widget>[
                            if (!isHorizontal && showChannel)
                              Transform.translate(
                                offset: const Offset(0, -30.0),
                                child: AvatarBuilder(
                                  domain: domain,
                                  name: video.channel.displayName,
                                  actorImageList:
                                      video.channel.avatars.isNotEmpty
                                          ? video.channel.avatars
                                          : video.account.avatars,
                                  size: 50.0,
                                  isDark: true,
                                  borderWidth: 2.0,
                                  borderRadius: 4.0,
                                ),
                              ),
                            if (!isHorizontal) const SizedBox(width: 5.0),
                            Expanded(
                              child: Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                mainAxisSize: MainAxisSize.min,
                                children: <Widget>[
                                  if (showChannel)
                                    Flexible(
                                      child: Text(
                                        '${video.channel.name}@${video.channel.host}',
                                        maxLines: 1,
                                        overflow: TextOverflow.ellipsis,
                                        style: isHorizontal
                                            ? Theme.of(context)
                                                .textTheme
                                                .displaySmall!
                                                .copyWith(
                                                  color:
                                                      colorScheme.onSecondary,
                                                )
                                            : Theme.of(context)
                                                .textTheme
                                                .displayLarge!
                                                .copyWith(
                                                  color:
                                                      colorScheme.onSecondary,
                                                ),
                                      ),
                                    ),
                                  if (showChannel) const SizedBox(height: 5.0),
                                  Flexible(
                                    child: Text(
                                      video.name,
                                      maxLines: 2,
                                      overflow: TextOverflow.ellipsis,
                                      style: isHorizontal
                                          ? Theme.of(context)
                                              .textTheme
                                              .headlineMedium
                                          : Theme.of(context)
                                              .textTheme
                                              .headlineMedium,
                                    ),
                                  ),
                                  const SizedBox(height: 5.0),
                                  Flexible(
                                    child: Text(
                                      video.isLive
                                          ? t(context)!.totalViewers(
                                              NumberFormat.decimalPattern(
                                                ref
                                                    .watch(localeProvider)
                                                    .countryCode,
                                              ).format(video.viewers),
                                            )
                                          : '${timeago(video.publishedAt, context)} • ${t(context)!.totalViews(NumberFormat.decimalPattern(ref.watch(localeProvider).languageCode).format(video.views))}',
                                      maxLines: 2,
                                      overflow: TextOverflow.ellipsis,
                                      style: isHorizontal
                                          ? Theme.of(context)
                                              .textTheme
                                              .displaySmall!
                                              .copyWith(
                                                color: colorScheme.onTertiary,
                                              )
                                          : Theme.of(context)
                                              .textTheme
                                              .displayMedium!
                                              .copyWith(
                                                color: colorScheme.onTertiary,
                                              ),
                                    ),
                                  ),
                                ],
                              ),
                            ),
                            if (!isHorizontal)
                              PopupMenuButton<String>(
                                offset: const Offset(-10, 42),
                                iconSize: 24,
                                onSelected: (value) => {
                                  if (value == 'report')
                                    {
                                      GoRouter.of(context).push(
                                        '/report/videos/${video.uuid}?isLocalAccount=${account.isLocal()}&host=${video.account.host}',
                                      ),
                                    },
                                },
                                itemBuilder: (context) =>
                                    <PopupMenuEntry<String>>[
                                  PopupMenuItem<String>(
                                    value: 'Watch later',
                                    child: account.isLocal()
                                        ? WatchLaterCheckboxListTile(
                                            domain: domain,
                                            video: video,
                                          )
                                        : const Placeholder(),
                                  ),
                                  PopupMenuItem<String>(
                                    value: 'report',
                                    child: ListTile(
                                      leading: const Icon(LucideIcons.flag),
                                      title: Text(t(context)!.report),
                                    ),
                                  ),
                                  PopupMenuItem<String>(
                                    value: 'share',
                                    child: ListTile(
                                      onTap: () => Share.shareUri(
                                        Uri.parse(domain)
                                            .resolve('/w/${video.shortUUID}'),
                                      ),
                                      title: Text(t(context)!.share),
                                      leading: const Icon(LucideIcons.share2),
                                    ),
                                  ),
                                ],
                              ),
                          ],
                        ),
                      ),
                    ],
                  ),
                  if (!isHorizontal && showChannel)
                    Positioned(
                      top: 190,
                      left: 10,
                      child: Semantics(
                        label: t(context)!.channelName,
                        onTapHint: t(context)!.navigateToChannel,
                        child: GestureDetector(
                          onTap: () async {
                            if (isOnSepiaSearch ||
                                ref.watch(currentPlatformProvider) == null) {
                              final success = await ref
                                  .read(currentPlatformServiceProvider)
                                  .loadPlatformFromLocalAccount(
                                    context,
                                    ref,
                                    domain,
                                  );
                              if (context.mounted) {
                                if (success) {
                                  GoRouter.of(context).push(
                                    '/video-channel/${video.channel.name}${video.isLocal == true ? '' : '@${video.channel.host}'}',
                                  );
                                } else {
                                  await showPlatformNotRespondingDialog(
                                    context,
                                    video.account.host,
                                  );
                                }
                              }
                            } else {
                              ref.read(goRouterProvider).push(
                                    '/video-channel/${video.channel.name}${video.isLocal == true ? '' : '@${video.channel.host}'}',
                                  );
                            }
                          },
                          child: Container(
                            width: 60.0,
                            height: 60.0,
                            color: Colors.transparent,
                          ),
                        ),
                      ),
                    ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }
}
