import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:intl/intl.dart';
import 'package:lucide_icons/lucide_icons.dart';
import 'package:markdown/markdown.dart' as md;
import 'package:share_plus/share_plus.dart';

import 'package:peertube_mobile/features/account/connected_account/connected_account_providers.dart';
import 'package:peertube_mobile/features/video/playlist/playlist_checkbox.dart';
import 'package:peertube_mobile/features/video/rating/rating_widget.dart';
import 'package:peertube_mobile/features/video/video_model.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/time.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/button.dart';
import 'package:peertube_mobile/shared/widgets/html.dart';

class VideoInfo extends ConsumerWidget {
  final VideoDetails video;
  final bool isUserLoggedIn;

  const VideoInfo({
    super.key,
    required this.video,
    required this.isUserLoggedIn,
  });

  @override
  Widget build(context, ref) {
    final colorScheme = Theme.of(context).colorScheme;
    final textTheme = Theme.of(context).textTheme;
    final platform = ref.watch(currentPlatformNonNullableProvider);
    return Padding(
      padding: const EdgeInsets.all(16.0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          Row(
            children: [
              Expanded(
                flex: 2,
                child: Text(
                  video.name,
                  style: textTheme.headlineLarge,
                ),
              ),
              Expanded(
                flex: 1,
                child: TextButton(
                  onPressed: () async => await Share.shareUri(
                    Uri.parse(platform.domain).resolve('/w/${video.shortUUID}'),
                  ),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      const Icon(LucideIcons.share2),
                      const SizedBox(
                        width: 6,
                      ),
                      Text(t(context)!.share),
                    ],
                  ),
                ),
              ),
            ],
          ),
          const SizedBox(height: 8.0),
          Text(
            '${timeago(video.publishedAt, context)}  •  ${t(context)!.totalViews(NumberFormat.decimalPattern(ref.watch(localeProvider).languageCode).format(video.views))}',
            style: textTheme.displayMedium!
                .copyWith(color: colorScheme.onSecondary),
          ),
          const SizedBox(height: 8.0),
          if (video.description != null)
            _VideoDescriptionWidget(description: video.description!),
          _ActionsRow(video: video, isUserLoggedIn: isUserLoggedIn),
        ],
      ),
    );
  }
}

class _VideoDescriptionWidget extends StatefulWidget {
  final String description;

  const _VideoDescriptionWidget({
    required this.description,
  });

  @override
  _VideoDescriptionWidgetState createState() => _VideoDescriptionWidgetState();
}

class _VideoDescriptionWidgetState extends State<_VideoDescriptionWidget> {
  bool _isExpanded = false;

  @override
  Widget build(BuildContext context) {
    final textTheme = Theme.of(context).textTheme;

    final processedHtml = md.markdownToHtml(widget.description);

    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        ClipRect(
          child: AnimatedSize(
            duration: const Duration(milliseconds: 75),
            child: ConstrainedBox(
              constraints: BoxConstraints(
                maxHeight: _isExpanded ? double.infinity : 40.0,
              ),
              child: LayoutBuilder(
                builder: (context, constraints) {
                  return SingleChildScrollView(
                    physics: _isExpanded
                        ? null
                        : const NeverScrollableScrollPhysics(),
                    child: CustomHtmlWidget(
                      rawHtml: processedHtml,
                    ),
                  );
                },
              ),
            ),
          ),
        ),
        if (processedHtml.length > 125) ...{
          const SizedBox(
            height: 12,
          ),
          UnderlinedTextButton(
            onPressed: () {
              setState(() {
                _isExpanded = !_isExpanded;
              });
            },
            text: _isExpanded ? 'Show Less' : 'Show More',
            textStyle: textTheme.displayMedium!,
          ),
        },
      ],
    );
  }
}

class _ActionsRow extends ConsumerWidget {
  final VideoDetails video;
  final bool isUserLoggedIn;

  const _ActionsRow({
    required this.video,
    required this.isUserLoggedIn,
  });

  @override
  Widget build(context, ref) {
    final platform = ref.watch(currentPlatformNonNullableProvider);
    return AsyncValueWidget(
      value: ref.watch(currentConnectedAccountProvider),
      dataBuilder: (account) => Row(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        children: <Widget>[
          RatingActions(
            video: video,
            isUserLoggedIn: isUserLoggedIn,
          ),
          const Spacer(),
          PopupMenuButton<String>(
            icon: const Icon(LucideIcons.listPlus),
            offset: const Offset(-10, 42),
            iconSize: 24,
            onSelected: (value) => {},
            itemBuilder: (context) => <PopupMenuEntry<String>>[
              if (!video.isLive)
                PopupMenuItem<String>(
                  value: 'Watch later',
                  child: account.isLocal()
                      ? WatchLaterCheckboxListTile(
                          domain: platform.domain,
                          video: video,
                        )
                      : const Placeholder(),
                ),
            ],
          ),
          IconButton(
            onPressed: () => GoRouter.of(context).push(
              '/report/videos/${video.uuid}?isLocalAccount=${account.isLocal()}&host=${video.account.host}',
            ),
            icon: const Icon(LucideIcons.flag),
          ),
        ],
      ),
    );
  }
}
