import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import 'package:accessibility_tools/accessibility_tools.dart';
import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:logging/logging.dart';
import 'package:sentry_flutter/sentry_flutter.dart';
import 'package:sentry_logging/sentry_logging.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'package:peertube_mobile/features/platform/history/platform_history_model.dart';
import 'package:peertube_mobile/features/platform/history/platform_history_providers.dart';
import 'package:peertube_mobile/routes/router.dart';
import 'package:peertube_mobile/routes/utils/loading_screen.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/database/database.dart';
import 'package:peertube_mobile/shared/environment/environment.dart';
import 'package:peertube_mobile/shared/logger/logger.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/storage/storage_providers.dart';
import 'package:peertube_mobile/shared/themes/peertube_theme.dart';
import 'package:peertube_mobile/shared/themes/theme_provider.dart';
import 'package:peertube_mobile/shared/utils/observer.dart';

Future<void> main() async {
  runApp(const LoadingScreen());

  kDebugMode || Environment.instance.buildType == BuildType.fdroid
      ? await _runApp()
      : await SentryFlutter.init(
          (options) {
            options
              ..dsn = Environment.instance.sentryDSN
              ..environment = Environment.instance.envName
              ..addIntegration(
                LoggingIntegration(
                  minEventLevel:
                      CustomLogger.fromInt(Environment.instance.logLevel),
                ),
              );
          },
          appRunner: _runApp,
        );
}

Future<void> _runApp() async {
  Logger.root.level = CustomLogger.fromInt(Environment.instance.logLevel);

  if (kDebugMode) {
    Logger.root.onRecord.listen((record) {
      debugPrint(
        '${record.level.name}: ${record.time}: ${record.message}',
      );
    });
  }

  final SharedPreferences prefs = await SharedPreferences.getInstance();

  // DatabaseHelper.removeDatabase();
  final db = await DatabaseHelper.initializeDatabase();

  const app = MyApp();

  runApp(
    ProviderScope(
      overrides: <Override>[
        sharedPreferencesProvider.overrideWithValue(prefs),
        databaseProvider.overrideWithValue(db),
      ],
      observers: [
        MyObserver(),
      ],
      child: Sentry.isEnabled == false
          ? app
          : DefaultAssetBundle(
              bundle: SentryAssetBundle(),
              child: app,
            ),
    ),
  );
}

class MyApp extends ConsumerStatefulWidget {
  const MyApp({super.key});
  @override
  ConsumerState<MyApp> createState() => _MyAppState();
}

class _MyAppState extends ConsumerState<MyApp> {
  late PlatformHistoryElementSQL? lastPlatform;

  @override
  Widget build(BuildContext context) {
    WidgetsFlutterBinding.ensureInitialized();
    SystemChrome.setPreferredOrientations(
      [DeviceOrientation.portraitUp],
    );

    return MaterialApp.router(
      localizationsDelegates: AppLocalizations.localizationsDelegates,
      supportedLocales: AppLocalizations.supportedLocales,
      locale: ref.watch(localeProvider),
      title: 'PeerTube Mobile',
      debugShowCheckedModeBanner: false,
      // showSemanticsDebugger: true,
      builder: (context, child) => AccessibilityTools(
        logLevel: LogLevel.warning,
        child: child,
      ),
      theme: PeerTubeTheme.getThemeData(
        Brightness.light,
      ),
      darkTheme: PeerTubeTheme.getThemeData(
        Brightness.dark,
      ),
      themeMode: ref.watch(themeProvider),
      routerConfig: ref.watch(goRouterProvider),
    );
  }

  Future<void> clearStorage() async {
    await ref.watch(flutterSecureStorageProvider).deleteAll();
    await ref.watch(sharedPreferencesProvider).clear();
  }

  @override
  void didChangeDependencies() {
    // clearStorage();
    loadLastPlatform();
    super.didChangeDependencies();
  }

  Future<void> loadLastPlatform() async {
    final lastPlatform =
        await ref.read(platformHistoryServiceProvider).getLastPlatform();

    if (lastPlatform != null) {
      await ref
          .read(currentPlatformProvider.notifier)
          .loadCurrentPlatform(Uri.parse(lastPlatform.domain));
    }
  }
}
