import 'package:flutter/material.dart';

import 'package:duration_picker/duration_picker.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:sentry_flutter/sentry_flutter.dart';

import 'package:peertube_mobile/features/moderation/abuse_providers.dart';
import 'package:peertube_mobile/features/moderation/predefined_reasons_enum.dart';
import 'package:peertube_mobile/features/video/video_model.dart';
import 'package:peertube_mobile/features/video/video_providers.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/text_field.dart';

enum ReportType { video, account, comment }

class ReportAbuseScreen extends ConsumerStatefulWidget {
  final String reportId;
  final ReportType reportType;
  final bool isLocalAccount;
  final String host;

  const ReportAbuseScreen({
    super.key,
    required this.reportId,
    required this.reportType,
    required this.isLocalAccount,
    required this.host,
  });

  @override
  ConsumerState<ReportAbuseScreen> createState() => _ReportAbuseScreenState();
}

class _ReportAbuseScreenState extends ConsumerState<ReportAbuseScreen> {
  int _currentStep = 0;

  Duration _startDuration = const Duration();
  Duration _endDuration = const Duration();

  final TextEditingController _descriptionController = TextEditingController();

  final List<PredefinedReasons> _selectedCategories = [];
  bool _isStartChecked = false;
  bool _isEndChecked = false;
  VideoDetails? videoDetails;

  @override
  void initState() {
    super.initState();
    if (widget.reportType == ReportType.video) loadVideo();
  }

  Future<void> loadVideo() async {
    try {
      if (ref.read(currentPlatformProvider) != null) {
        videoDetails =
            await ref.read(videoDatasourceProvider).fetchVideo(widget.reportId);
      }
    } catch (e) {
      videoDetails = null;
    }
    setState(() {});
  }

  final int _totalSteps = 3;

  void _nextStep() {
    if (_currentStep == 2 && _descriptionController.text.isEmpty) {
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text(t(context)!.descriptionIsRequired)),
      );
      return;
    }
    if (_currentStep < _totalSteps - 1) {
      setState(() {
        widget.reportType == ReportType.video
            ? _currentStep++
            : _currentStep = _totalSteps - 1;
      });
    } else {
      _submitReport();
    }
  }

  void _submitReport() async {
    final abuseService = ref.read(abuseServiceProvider);
    String reason = _descriptionController.text;
    List<PredefinedReasons> predefinedReasons = _selectedCategories;

    try {
      if (widget.isLocalAccount) {
        await Sentry.captureMessage(
          '''
Abuse Report Submission:
  
  Report Info:
    - Report ID: ${widget.reportId}
    - Report Type: ${widget.reportType}
    - Host: ${widget.host}
  
  Submission Details:
    - Start Time: ${_isStartChecked ? _startDuration.toString() : 'Not specified'}
    - End Time: ${_isEndChecked ? _endDuration.toString() : 'Not specified'}
    - Description: ${_descriptionController.text.isEmpty ? 'Empty' : _descriptionController.text}
    - Selected Categories: ${_selectedCategories.map((e) => e.toString()).join(', ')}
  
  Device Info:
    - Platform: ${Theme.of(context).platform}
    - Locale: ${Localizations.localeOf(context)}
''',
          level: SentryLevel.warning,
        );
      } else {
        switch (widget.reportType) {
          case ReportType.video:
            await abuseService.reportVideo(
              reason: reason,
              videoId: int.parse(widget.reportId),
              predefinedReasons: predefinedReasons,
            );
            break;
          case ReportType.account:
            await abuseService.reportAccount(
              reason: reason,
              accountId: int.parse(widget.reportId),
              predefinedReasons: predefinedReasons,
            );
            break;
          case ReportType.comment:
            await abuseService.reportComment(
              reason: reason,
              commentId: int.parse(widget.reportId),
              predefinedReasons: predefinedReasons,
            );
            break;
        }
      }
      if (mounted) GoRouter.of(context).replace('/report/confirmation');
    } catch (error) {
      if (mounted) {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text('Error: $error')),
        );
      }
    }
  }

  Map<String, String> _getReasonStrings(
    PredefinedReasons reason,
    BuildContext context,
  ) {
    switch (reason) {
      case PredefinedReasons.violentOrAbusive:
        return {
          'title': t(context)!.violentOrRepulsive,
          'description': t(context)!.violentOrRepulsiveDescription,
        };
      case PredefinedReasons.hatefulOrAbusive:
        return {
          'title': t(context)!.hatefulOrAbusive,
          'description': t(context)!.hatefulOrAbusiveDescription,
        };
      case PredefinedReasons.spamOrMisleading:
        return {
          'title': t(context)!.spamAdOrFalseNews,
          'description': t(context)!.spamAdOrFalseNewsDescription,
        };
      case PredefinedReasons.privacy:
        return {
          'title': t(context)!.privacyBreachOrDoxxing,
          'description': t(context)!.privacyBreachOrDoxxingDescription,
        };
      case PredefinedReasons.rights:
        return {
          'title': t(context)!.copyright,
          'description': t(context)!.copyrightDescription,
        };
      case PredefinedReasons.serverRules:
        return {
          'title': t(context)!.breaksServerRules,
          'description': t(context)!.breaksServerRulesDescription,
        };
      case PredefinedReasons.thumbnails:
        return {
          'title': t(context)!.thumbnails,
          'description': t(context)!.thumbnailsReportDescription,
        };
      case PredefinedReasons.captions:
        return {
          'title': t(context)!.captions,
          'description': t(context)!.captionReportDescription,
        };
    }
  }

  Widget _buildCategoryTile(PredefinedReasons reason) {
    final reasonDetails = _getReasonStrings(reason, context);
    return ListTile(
      leading: Checkbox(
        value: _selectedCategories.contains(reason),
        onChanged: (bool? value) {
          setState(() {
            if (value == true) {
              _selectedCategories.add(reason);
            } else {
              _selectedCategories.remove(reason);
            }
          });
        },
      ),
      title: Text(reasonDetails['title']!),
      subtitle: Text(reasonDetails['description']!),
      onTap: () {
        setState(() {
          if (_selectedCategories.contains(reason)) {
            _selectedCategories.remove(reason);
          } else {
            _selectedCategories.add(reason);
          }
        });
      },
    );
  }

  Widget _buildStepContent() {
    switch (_currentStep) {
      case 0:
        return ListView(
          children: PredefinedReasons.values
              .map(
                (reason) => (widget.reportType != ReportType.video ||
                        (reason != PredefinedReasons.thumbnails))
                    ? _buildCategoryTile(reason)
                    : const SizedBox.shrink(),
              )
              .toList(),
        );

      case 1:
        return Column(
          children: [
            ListTile(
              leading: Checkbox(
                value: _isStartChecked,
                onChanged: (bool? value) {
                  setState(() {
                    _isStartChecked = value ?? false;
                    if (!_isStartChecked) {
                      _isEndChecked = false;
                    }
                  });
                },
              ),
              title: Text(t(context)!.startAt),
              trailing: _isStartChecked
                  ? IconButton(
                      icon: const Icon(Icons.timer),
                      onPressed: () async {
                        final resultingDuration = await showDurationPicker(
                          context: context,
                          initialTime: _startDuration,
                          baseUnit: BaseUnit.second,
                          upperBound: const Duration(minutes: 5),
                          lowerBound: const Duration(seconds: 10),
                        );
                        if (resultingDuration != null) {
                          setState(() {
                            _startDuration = resultingDuration;
                          });
                        }
                      },
                    )
                  : null,
            ),
            if (_isStartChecked) ...[
              ListTile(
                leading: Checkbox(
                  value: _isEndChecked,
                  onChanged: _isStartChecked
                      ? (bool? value) {
                          setState(() {
                            _isEndChecked = value ?? false;
                          });
                        }
                      : null,
                ),
                title: Text(t(context)!.stopAt),
                trailing: _isEndChecked
                    ? IconButton(
                        icon: const Icon(Icons.timer),
                        onPressed: () async {
                          final resultingDuration = await showDurationPicker(
                            context: context,
                            initialTime: _endDuration,
                            baseUnit: BaseUnit.second,
                            upperBound: const Duration(minutes: 5),
                            lowerBound: const Duration(seconds: 10),
                          );
                          if (resultingDuration != null) {
                            setState(() {
                              _endDuration = resultingDuration;
                            });
                          }
                        },
                      )
                    : null,
              ),
            ],
          ],
        );
      case 2:
        final currentHost = Uri.parse(
          ref.watch(currentPlatformProvider)?.domain ?? 'https://framasoft.org',
        ).host;
        final isLocal = currentHost == widget.host;
        return Column(
          children: [
            Text(
              isLocal || currentHost == 'framasoft.org'
                  ? t(context)!.yourReportWillBeSentToModeratorsOfCurrenthost(
                      currentHost,
                    )
                  : t(context)!
                      .yourReportWillBeSentToModeratorsOfCurrenthostAndWillBeForwardedToTheVideoOriginVideodetailsAccountHostToo(
                      currentHost,
                      videoDetails!.account.host,
                    ),
            ),
            const SizedBox(
              height: 16,
            ),
            CustomTextFormField(
              controller: _descriptionController,
              hintText: t(context)!.pleaseDescribeTheIssue,
              maxLines: 5,
            ),
          ],
        );
      default:
        return const SizedBox.shrink();
    }
  }

  String _getReportString(
    BuildContext context,
    ReportType reportType,
    String? name,
  ) {
    final hasName = (name != null && name.trim().isNotEmpty);

    switch (reportType) {
      case ReportType.video:
        return hasName
            ? t(context)!.reportVideo(name)
            : t(context)!.reportAVideo;
      case ReportType.account:
        return t(context)!.reportAnAccount;
      case ReportType.comment:
        return t(context)!.reportAComment;
    }
  }

  @override
  Widget build(BuildContext context) {
    double progress = (_currentStep + 0.1) / (_totalSteps - 1);

    return (widget.reportType == ReportType.video &&
            videoDetails == null &&
            ref.read(currentPlatformProvider) != null)
        ? const Scaffold(
            body: Center(
              child: CircularProgressIndicator(),
            ),
          )
        : Scaffold(
            appBar: AppBar(
              title: Text(
                _getReportString(
                  context,
                  widget.reportType,
                  videoDetails?.name,
                ),
                overflow: TextOverflow.ellipsis,
              ),
            ),
            body: Padding(
              padding: const EdgeInsets.all(16.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.stretch,
                children: [
                  LinearProgressIndicator(
                    value: progress,
                  ),
                  const SizedBox(height: 20),
                  Expanded(
                    child: _buildStepContent(),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Expanded(
                        child: Padding(
                          padding: const EdgeInsets.all(8.0),
                          child: ElevatedButton(
                            onPressed: () {
                              setState(() {
                                _currentStep == 0
                                    ? GoRouter.of(context).pop()
                                    : widget.reportType != ReportType.video &&
                                            _currentStep == _totalSteps - 1
                                        ? _currentStep = _currentStep - 2
                                        : _currentStep--;
                              });
                            },
                            child: Text(
                              _currentStep == 0
                                  ? t(context)!.cancel
                                  : t(context)!.back,
                            ),
                          ),
                        ),
                      ),
                      Expanded(
                        child: Padding(
                          padding: const EdgeInsets.all(8.0),
                          child: ElevatedButton(
                            onPressed: _nextStep,
                            child: Text(
                              _currentStep == _totalSteps - 1
                                  ? t(context)!.submit
                                  : t(context)!.next,
                            ),
                          ),
                        ),
                      ),
                    ],
                  ),
                ],
              ),
            ),
          );
  }
}

class ConfirmationScreen extends StatelessWidget {
  const ConfirmationScreen({super.key});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(t(context)!.confirmation),
      ),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Text(t(context)!.thankYouForYourReport),
            const SizedBox(height: 20),
            ElevatedButton(
              onPressed: () {
                GoRouter.of(context).pop();
              },
              child: Text(t(context)!.returnToHome),
            ),
          ],
        ),
      ),
    );
  }
}
