import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:intl/intl.dart';
import 'package:lucide_icons/lucide_icons.dart';
import 'package:readmore/readmore.dart';

import 'package:peertube_mobile/features/account/account_providers.dart';
import 'package:peertube_mobile/features/account/connected_account/connected_account_providers.dart';
import 'package:peertube_mobile/features/platform/platform_banner.dart';
import 'package:peertube_mobile/features/video/channel/channel_list_view.dart';
import 'package:peertube_mobile/features/video/video_list/video_list_view.dart';
import 'package:peertube_mobile/features/video/video_providers.dart';
import 'package:peertube_mobile/shared/config/config_providers.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/filters/account/account_params_model.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/app_bar.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/avatar_builder.dart';
import 'package:peertube_mobile/shared/widgets/tabbed_page_scaffold.dart';

class AccountScreen extends ConsumerWidget {
  final String name;

  const AccountScreen({
    super.key,
    required this.name,
  });

  @override
  Widget build(context, ref) {
    final platform = ref.watch(currentPlatformNonNullableProvider);
    final accountDetails = ref.watch(accountDetailsProvider(name));
    final asyncConfig = accountDetails.when(
      data: (account) {
        return ref.watch(configServerProvider(account.host));
      },
      loading: () => const AsyncValue.loading(),
      error: (error, stackTrace) => AsyncValue.error(error, stackTrace),
    );
    final asyncTotalVideos = ref.watch(
      totalVideosProvider(
        '/accounts/$name/videos',
      ),
    );

    final asyncTotalChannels = ref.watch(
      totalVideoChannelsProvider(name),
    );

    return TabbedPageScaffold(
      key: ValueKey('${asyncTotalVideos.value}-${asyncTotalChannels.value}'),
      appBar: AppBar(
        title: Text(
          platform.name,
          overflow: TextOverflow.ellipsis,
        ),
        titleSpacing: 0,
        actions: const [
          AccountAction(),
        ],
      ),
      headerContent: AsyncValueWidget(
        value: accountDetails,
        dataBuilder: (accountDetails) {
          bool isLocal = ref
              .watch(
                currentPlatformNonNullableProvider,
              )
              .domain
              .contains(accountDetails.host);

          final colorScheme = Theme.of(context).colorScheme;
          final textTheme = Theme.of(context).textTheme;
          return Stack(
            children: [
              Positioned(
                top: -630,
                left: -200,
                child: Container(
                  width: 800,
                  height: 700,
                  decoration: BoxDecoration(
                    color: colorScheme.surfaceContainerHigh,
                    shape: BoxShape.circle,
                  ),
                ),
              ),
              Positioned(
                right: 18,
                top: 18,
                child: AsyncValueWidget(
                  value: ref.watch(currentConnectedAccountProvider),
                  dataBuilder: (account) => CircleAvatar(
                    backgroundColor: colorScheme.surfaceContainerHighest,
                    child: IconButton(
                      onPressed: () => GoRouter.of(context).push(
                        '/report/accounts/${accountDetails.id}?isLocalAccount=${account.isLocal()}&host=${accountDetails.host}',
                      ),
                      icon: const Icon(LucideIcons.flag),
                    ),
                  ),
                ),
              ),
              Center(
                child: Column(
                  children: [
                    const SizedBox(
                      height: 24,
                    ),
                    AvatarBuilder(
                      domain: ref
                          .watch(
                            currentPlatformNonNullableProvider,
                          )
                          .domain,
                      name: accountDetails.displayName,
                      size: 96,
                      borderRadius: 50,
                      actorImageList: accountDetails.avatars,
                    ),
                    Padding(
                      padding: const EdgeInsets.symmetric(
                        vertical: 10,
                      ),
                      child: Column(
                        children: [
                          Text(
                            accountDetails.displayName,
                            style: textTheme.headlineLarge,
                            maxLines: 1,
                            overflow: TextOverflow.ellipsis,
                          ),
                          Text(
                            '@${accountDetails.name}',
                            style: textTheme.displaySmall!.copyWith(
                              color: colorScheme.onSecondary,
                            ),
                          ),
                          const SizedBox(
                            height: 10,
                          ),
                        ],
                      ),
                    ),
                    const SizedBox(
                      height: 16,
                    ),
                    if (accountDetails.description != null)
                      ReadMoreText(
                        accountDetails.description!,
                        style: TextStyle(
                          fontSize: 14,
                          color: colorScheme.onSecondary,
                        ),
                        textAlign: TextAlign.center,
                        trimMode: TrimMode.Line,
                        trimLines: 2,
                        trimCollapsedText: 'Show more',
                        trimExpandedText: '\nShow less',
                        moreStyle: textTheme.displayMedium!.copyWith(
                          color: colorScheme.onSurfaceVariant,
                          decoration: TextDecoration.underline,
                        ),
                        lessStyle: textTheme.displayMedium!.copyWith(
                          color: colorScheme.onSurfaceVariant,
                          decoration: TextDecoration.underline,
                        ),
                      ),
                    const SizedBox(height: 8),
                    if (!isLocal)
                      AsyncValueWidget(
                        value: asyncConfig,
                        dataBuilder: (config) => PlatformBannerFromConfig(
                          host: accountDetails.host,
                          config: config,
                          entityName: 'Account',
                        ),
                      ),
                  ],
                ),
              ),
            ],
          );
        },
      ),
      tabTitles: [
        asyncTotalChannels.when(
          data: (channelsCount) => t(context)!.totalChannels(
            NumberFormat.decimalPattern(
              ref.watch(localeProvider).languageCode,
            ).format(channelsCount),
          ),
          loading: () => t(context)!.channels,
          error: (_, __) => t(context)!.channels,
        ),
        asyncTotalVideos.when(
          data: (videosCount) => t(context)!.totalVideos(
            NumberFormat.decimalPattern(
              ref.watch(localeProvider).languageCode,
            ).format(videosCount),
          ),
          loading: () => t(context)!.videos,
          error: (_, __) => t(context)!.videos,
        ),
      ],
      tabViews: [
        AsyncValueWidget(
          value: asyncConfig,
          dataBuilder: (config) => VideoChannelListView(
            config: config,
            fetchVideoChannels: (AccountParams params) {
              return ref.watch(accountDatasourceProvider).fetchVideoChannels(
                    name,
                    commonParams: params,
                  );
            },
          ),
        ),
        SingleChildScrollView(
          child: Column(
            children: [
              // Padding(
              //   padding: const EdgeInsets.all(16.0),
              //   child: Row(
              //     children: [
              //       Container(
              //         decoration: BoxDecoration(
              //           border: Border.all(
              //             color: Theme.of(context)
              //                 .colorScheme
              //                 .surfaceContainerHighest,
              //           ),
              //           shape: BoxShape.circle,
              //         ),
              //         child: IconButton(
              //           onPressed: () => {},
              //           icon: const Icon(
              //             LucideIcons.settings2,
              //             semanticLabel: 'Filter',
              //           ),
              //         ),
              //       ),
              //       const SizedBox(
              //         width: 8.0,
              //       ),
              //       const Expanded(
              //         child: CustomTextField(
              //           labelText: 'Search on',
              //         ),
              //       ),
              //     ],
              //   ),
              // ),
              VideoListView(
                fetchPage: (pageKey, pageSize) =>
                    ref.read(videoServiceProvider).getAccountVideos(
                          name,
                          pageKey,
                          pageSize,
                        ),
                isNested: true,
                domain: platform.domain,
              ),
            ],
          ),
        ),
      ],
    );
  }
}
