import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/features/filters/filters_widget.dart';
import 'package:peertube_mobile/features/platform/history/platform_history_providers.dart';
import 'package:peertube_mobile/features/platform/joinpeertube/widgets/platform_list.dart';
import 'package:peertube_mobile/features/video/search/search_providers.dart';
import 'package:peertube_mobile/features/video/video_list/video_list_view.dart';
import 'package:peertube_mobile/shared/filters/filters_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/app_bar.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/button.dart';
import 'package:peertube_mobile/shared/widgets/text_field.dart';

final exploreRefreshTriggerProvider = StateProvider<bool>((ref) => false);

final explorePlatformNumberProvider = StateProvider<int>((ref) => 0);

final showOnboardingDialogProvider = StateProvider<bool>((ref) => false);

class ExploreScreen extends ConsumerStatefulWidget {
  const ExploreScreen({super.key});

  @override
  ConsumerState<ExploreScreen> createState() => _ExploreScreenState();
}

class _ExploreScreenState extends ConsumerState<ExploreScreen> {
  late TextEditingController _searchController;
  String _prevSearchText = '';
  @override
  void initState() {
    super.initState();
    _searchController = TextEditingController();

    _searchController.addListener(() {
      if (_prevSearchText != _searchController.text) {
        _prevSearchText = _searchController.text;

        ref
            .read(explorePlatformFilterProvider.notifier)
            .update(search: _searchController.text);

        ref
            .read(exploreSearchVideoFilterProvider.notifier)
            .update(search: _searchController.text);
      }
    });
  }

  @override
  void dispose() {
    _searchController.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (ref.watch(showOnboardingDialogProvider)) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        _showOnboardingDialog(context);
        ref.read(showOnboardingDialogProvider.notifier).state = false;
      });
    }

    final textTheme = Theme.of(context).textTheme;
    return Scaffold(
      appBar: AppBar(
        title: Text(
          t(context)!.explore,
        ),
        actions: const [
          AccountAction(),
        ],
      ),
      body: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            Padding(
              padding:
                  const EdgeInsets.only(top: 18.0, left: 18.0, right: 18.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  CustomTextFormField(
                    controller: _searchController,
                    hintText: t(context)!.searchPeertubeContent,
                    searchTheme: true,
                    maxLines: 1,
                  ),
                  const FilterByMyInterestsSwitch(),
                  if (ref.watch(explorePlatformNumberProvider) > 0) ...{
                    Text(
                      t(context)!.platforms,
                      style: textTheme.headlineLarge,
                    ),
                    PlatformFilterChipsWidget(
                      params: ref.watch(explorePlatformFilterProvider),
                      paramsNotifier:
                          ref.watch(explorePlatformFilterProvider.notifier),
                    ),
                  },
                ],
              ),
            ),
            Container(
              height: ref.watch(explorePlatformNumberProvider) == 0 ? 0 : 225,
              padding: const EdgeInsets.only(left: 18.0),
              child: PlatformJPTListHorizontal(
                counterProvider: explorePlatformNumberProvider,
              ),
            ),
            Padding(
              padding: const EdgeInsets.symmetric(horizontal: 18.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  if (ref.watch(explorePlatformNumberProvider) > 0) ...{
                    const SizedBox(
                      height: 8,
                    ),
                    Center(
                      child: TextButton(
                        onPressed: () => GoRouter.of(context).push(
                          '/explore/platforms',
                        ),
                        child: Padding(
                          padding: const EdgeInsets.symmetric(
                            horizontal: 16,
                            vertical: 8.5,
                          ),
                          child: Text(
                            t(context)!.showMorePlatforms,
                            style: textTheme.displayLarge!
                                .copyWith(fontWeight: FontWeight.bold),
                          ),
                        ),
                      ),
                    ),
                    const SizedBox(
                      height: 24,
                    ),
                  },
                  Text(
                    t(context)!.content,
                    style: textTheme.headlineLarge,
                  ),
                  const ContentFilterChipsWidget(),
                ],
              ),
            ),
            AsyncValueWidget(
              value: ref.watch(platformHistoryProvider),
              dataBuilder: (platformHistory) => VideoListView(
                fetchPage: (pageKey, pageSize) =>
                    ref.read(searchServiceProvider).searchOnSepia(
                          pageKey,
                          pageSize,
                          ref.watch(exploreSearchVideoFilterProvider),
                          platformHistory,
                        ),
                isNested: true,
                refreshTriggerProvider: exploreRefreshTriggerProvider,
                isOnSepiaSearch: true,
              ),
            ),
          ],
        ),
      ),
    );
  }

  Future<void> _showOnboardingDialog(BuildContext context) async {
    return showDialog<void>(
      context: context,
      barrierDismissible: false,
      builder: (BuildContext context) {
        var textTheme = Theme.of(context).textTheme;
        return Stack(
          children: [
            Positioned(
              top: 20,
              left: MediaQuery.of(context).size.width * 0.05,
              child: Material(
                child: Container(
                  width: MediaQuery.of(context).size.width * 0.9,
                  height: MediaQuery.of(context).size.height * 0.8,
                  decoration: BoxDecoration(
                    color:
                        Theme.of(context).colorScheme.surfaceContainerHighest,
                    borderRadius: BorderRadius.circular(10),
                    boxShadow: const [
                      BoxShadow(
                        color: Colors.black26,
                        blurRadius: 10,
                        offset: Offset(0, 2),
                      ),
                    ],
                  ),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.center,
                    children: <Widget>[
                      Row(
                        mainAxisAlignment: MainAxisAlignment.spaceBetween,
                        children: [
                          const SizedBox(width: 20),
                          Padding(
                            padding: const EdgeInsets.all(8.0),
                            child: IconButton(
                              icon: const Icon(
                                LucideIcons.x,
                                size: 32,
                              ),
                              onPressed: () {
                                Navigator.of(context).pop();
                              },
                            ),
                          ),
                        ],
                      ),
                      const SizedBox(height: 20),
                      Padding(
                        padding: const EdgeInsets.symmetric(horizontal: 50.0),
                        child: Column(
                          children: [
                            Text(
                              t(context)!.explorePeertube,
                              style: textTheme.titleLarge,
                            ),
                            const SizedBox(height: 40),
                            Text(
                              t(context)!.selectionOfPopularContent,
                              textAlign: TextAlign.center,
                              style: textTheme.titleSmall,
                            ),
                            const SizedBox(height: 20),
                            Text(
                              t(context)!.setYourPreferences,
                              textAlign: TextAlign.center,
                              style: textTheme.titleSmall,
                            ),
                          ],
                        ),
                      ),
                      const Spacer(),
                      Padding(
                        padding: const EdgeInsets.symmetric(horizontal: 20.0),
                        child: Column(
                          children: [
                            CustomElevatedButton(
                              child: Text(t(context)!.defineMyInterests),
                              onPressed: () {
                                Navigator.of(context).pop();
                                GoRouter.of(context).push(
                                  '/settings/preferences',
                                );
                              },
                            ),
                            const SizedBox(
                              height: 40,
                            ),
                            UnderlinedTextButton(
                              text: t(context)!.notNow,
                              alignment: Alignment.center,
                              textStyle:
                                  Theme.of(context).textTheme.headlineLarge!,
                              onPressed: () {
                                Navigator.of(context).pop();
                              },
                            ),
                            const SizedBox(
                              height: 20,
                            ),
                          ],
                        ),
                      ),
                      const SizedBox(height: 20),
                    ],
                  ),
                ),
              ),
            ),
          ],
        );
      },
    );
  }
}
