import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:grouped_list/grouped_list.dart';
import 'package:intl/intl.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/features/account/connected_account/local_account/local_account_model.dart';
import 'package:peertube_mobile/features/account/connected_account/local_account/local_account_providers.dart';
import 'package:peertube_mobile/features/account/connected_account/local_account/local_video_list_tile.dart';
import 'package:peertube_mobile/features/platform/platform_miniature.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/dialog.dart';
import 'package:peertube_mobile/shared/widgets/text_field.dart';

class LocalHistoryScreen extends ConsumerStatefulWidget {
  const LocalHistoryScreen({super.key});

  @override
  ConsumerState<LocalHistoryScreen> createState() => _LocalHistoryScreenState();
}

class _LocalHistoryScreenState extends ConsumerState<LocalHistoryScreen> {
  final TextEditingController _searchController = TextEditingController();
  List<LocalHistoryElementSQL> _filteredHistoryList = [];
  bool _isEditing = false;

  @override
  Widget build(BuildContext context) {
    final localHistoryAsyncValue = ref.watch(localHistoryProvider);

    return Scaffold(
      appBar: AppBar(
        title: Text(t(context)!.history),
        actions: [
          if (_isEditing)
            IconButton(
              icon: const Icon(Icons.delete_sweep),
              onPressed: () => showDialog<bool>(
                context: context,
                builder: (BuildContext context) {
                  return CustomAlertDialog(
                    title: t(context)!.deleteAll,
                    content: t(context)!.deleteAllConfirmation,
                    onConfirm: () =>
                        ref.read(localHistoryProvider.notifier).clearHistory(),
                  );
                },
              ),
            ),
          IconButton(
            icon: Icon(
              _isEditing ? LucideIcons.check : LucideIcons.pencil,
              semanticLabel: t(context)!.edit,
            ),
            onPressed: _toggleEditing,
          ),
        ],
      ),
      body: AsyncValueWidget(
        value: localHistoryAsyncValue,
        dataBuilder: (historyList) {
          if (_searchController.text.isEmpty) {
            _filteredHistoryList = historyList;
          }

          return Column(
            children: [
              Padding(
                padding: const EdgeInsets.all(8.0),
                child: CustomTextFormField(
                  controller: _searchController,
                  labelText: 'Search',
                  searchTheme: true,
                ),
              ),
              Expanded(
                child: _filteredHistoryList.isEmpty
                    ? Center(child: Text(t(context)!.noVideosInHistoryYet))
                    : GroupedListView<LocalHistoryElementSQL, String>(
                        elements: _filteredHistoryList,
                        groupBy: (element) {
                          DateTime date = element.watchedAt;
                          return DateFormat.yMMMMEEEEd(
                            ref.watch(localeProvider).toString(),
                          ).format(date);
                        },
                        groupSeparatorBuilder: (String date) => Container(
                          color:
                              Theme.of(context).colorScheme.surfaceContainerLow,
                          child: Padding(
                            padding: const EdgeInsets.all(8.0),
                            child: Row(
                              children: [
                                Text(
                                  t(context)!.onDate(date),
                                  style:
                                      Theme.of(context).textTheme.displayLarge,
                                ),
                              ],
                            ),
                          ),
                        ),
                        itemBuilder:
                            (context, LocalHistoryElementSQL element) =>
                                LocalVideoListTile(
                          onTap: () async {
                            final success = await ref
                                .read(currentPlatformServiceProvider)
                                .loadPlatformFromLocalAccount(
                                  context,
                                  ref,
                                  element.domain,
                                );
                            if (context.mounted) {
                              if (success) {
                                GoRouter.of(context).push(
                                  '/video/${element.videoUuid}',
                                );
                              } else {
                                await showPlatformNotRespondingDialog(
                                  context,
                                  element.domain,
                                );
                              }
                            }
                          },
                          imageUrl: Uri.parse(element.domain)
                              .resolve(element.previewPath ?? '')
                              .toString(),
                          title: element.name,
                          subtitle: element.channelDisplayName,
                          isEditing: _isEditing,
                          onDelete: () => showDialog<bool>(
                            context: context,
                            builder: (BuildContext context) {
                              return CustomAlertDialog(
                                title: t(context)!.deleteElement(element.name),
                                content: t(context)!.deleteElementConfirmation,
                                onConfirm: () {
                                  ref
                                      .read(localHistoryProvider.notifier)
                                      .removeFromHistory(element);
                                },
                              );
                            },
                          ),
                        ),
                        itemComparator: (item1, item2) =>
                            item1.watchedAt.compareTo(item2.watchedAt),
                        useStickyGroupSeparators: true,
                        floatingHeader: true,
                        order: GroupedListOrder.ASC,
                      ),
              ),
            ],
          );
        },
      ),
    );
  }

  @override
  void dispose() {
    _searchController.removeListener(_onSearchChanged);
    _searchController.dispose();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    _searchController.addListener(_onSearchChanged);
  }

  void _onSearchChanged() {
    setState(() {
      final historyList = ref.read(localHistoryProvider).valueOrNull;
      if (historyList != null) {
        _filteredHistoryList = historyList.where((element) {
          final searchQuery = _searchController.text.toLowerCase();
          return element.name.toLowerCase().contains(searchQuery) ||
              element.channelDisplayName.toLowerCase().contains(searchQuery);
        }).toList();
      }
    });
  }

  void _toggleEditing() {
    setState(() {
      _isEditing = !_isEditing;
    });
  }
}
