import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/shared/auth/auth_providers.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/text_field.dart';

class LoginScreen extends ConsumerStatefulWidget {
  const LoginScreen({super.key});

  @override
  ConsumerState createState() => _LoginScreenState();
}

class _LoginScreenState extends ConsumerState<LoginScreen> {
  final TextEditingController usernameController = TextEditingController();
  final TextEditingController passwordController = TextEditingController();

  String? usernameError;
  String? passwordError;
  String? loginError;

  @override
  Widget build(BuildContext context) {
    final colorScheme = Theme.of(context).colorScheme;
    final platform = ref.watch(currentPlatformNonNullableProvider);
    final isKeyboard = MediaQuery.of(context).viewInsets.bottom != 0;

    return Stack(
      children: [
        if (!isKeyboard)
          Positioned(
            top: -580,
            left: -200,
            child: Container(
              width: 800,
              height: 700,
              decoration: BoxDecoration(
                color: colorScheme.surfaceContainerHigh,
                shape: BoxShape.circle,
              ),
            ),
          ),
        Padding(
          padding: const EdgeInsets.all(16.0),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.center,
            children: <Widget>[
              if (!isKeyboard)
                Image.asset(
                  width: 200,
                  'assets/sepia/happy.png',
                  semanticLabel: t(context)!.sepiaPeertubeMascot,
                ),
              Text(
                t(context)!.login,
                style: Theme.of(context).textTheme.titleLarge,
              ),
              Text(
                t(context)!.toInstanceName(platform.name),
                style: Theme.of(context).textTheme.titleSmall!.copyWith(
                      fontWeight: FontWeight.w600,
                      color: colorScheme.onSecondary,
                    ),
              ),
              const SizedBox(height: 20.0),
              CustomTextFormField(
                controller: usernameController,
                labelText: t(context)!.username,
                helperText: t(context)!.helperTextLogin,
                errorText: usernameError,
              ),
              const SizedBox(height: 16.0),
              CustomTextFormField(
                controller: passwordController,
                labelText: 'Password',
                obscureText: true,
                errorText: passwordError,
              ),
              const SizedBox(height: 16.0),
              if (loginError != null)
                Text(
                  loginError!,
                  style: TextStyle(color: colorScheme.error),
                ),
              TextButton(
                onPressed: () {},
                child: Text(
                  t(context)!.forgotMyPassword,
                  style: Theme.of(context).textTheme.displayLarge!.copyWith(
                        decoration: TextDecoration.underline,
                        color: Theme.of(context).colorScheme.onSecondary,
                      ),
                ),
              ),
              if (!isKeyboard) const Spacer(),
              ElevatedButton(
                onPressed: () async {
                  setState(() {
                    usernameError = usernameController.text.isEmpty
                        ? 'Please enter your username'
                        : null;
                    passwordError = passwordController.text.isEmpty
                        ? 'Please enter your password'
                        : null;
                    loginError = null;
                  });

                  if (usernameError == null && passwordError == null) {
                    bool loginSuccess =
                        await ref.read(authServiceProvider).login(
                              usernameController.text,
                              passwordController.text,
                              platform,
                            );
                    if (context.mounted && loginSuccess) {
                      GoRouter.of(context).go('/');
                    } else {
                      setState(() {
                        loginError =
                            'Login failed. Please check your credentials and try again.';
                      });
                    }
                  }
                },
                style: ButtonStyle(
                  backgroundColor: WidgetStateProperty.all(
                    Theme.of(context).colorScheme.primary,
                  ),
                  overlayColor: WidgetStateProperty.resolveWith<Color>(
                    (Set<WidgetState> states) {
                      if (states.contains(WidgetState.pressed)) {
                        return Theme.of(context)
                            .colorScheme
                            .primary
                            .withGreen(60);
                      }
                      return Colors.transparent;
                    },
                  ),
                ),
                child: SizedBox(
                  width: double.infinity,
                  height: 60.0,
                  child: Center(
                    child: Text(
                      t(context)!.login,
                      style: Theme.of(context).textTheme.titleSmall!.copyWith(
                            color: Theme.of(context).colorScheme.surface,
                            fontWeight: FontWeight.bold,
                          ),
                      textAlign: TextAlign.center,
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
        IconButton(
          onPressed: () => GoRouter.of(context).pop(),
          icon: CircleAvatar(
            radius: 24.0,
            backgroundColor: colorScheme.surfaceContainerHigh,
            child: Icon(
              LucideIcons.arrowLeft,
              semanticLabel: t(context)!.back,
            ),
          ),
        ),
      ],
    );
  }
}
