import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:markdown/markdown.dart' as md;

import 'package:peertube_mobile/shared/config/config_providers.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/utils/tools.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/html.dart';
import 'package:peertube_mobile/shared/widgets/image.dart';

class AboutPlatformScreen extends ConsumerWidget {
  const AboutPlatformScreen({super.key});

  @override
  Widget build(context, ref) {
    final platform = ref.watch(currentPlatformNonNullableProvider);

    final bannerPath = platform.banners.isEmpty
        ? null
        : getImagePathForSize(
            platform.banners,
            MediaQuery.of(context).size.width,
          );
    var textTheme = Theme.of(context).textTheme;

    return SafeArea(
      child: Stack(
        children: [
          if (bannerPath != null)
            CustomCachedNetworkImage(
              imageUrl: Uri.parse(
                platform.domain,
              )
                  .resolve(
                    platform.banners[0].path,
                  )
                  .toString(),
            ),
          Scaffold(
            backgroundColor: Colors.transparent,
            appBar: AppBar(
              backgroundColor: Colors.transparent,
              elevation: 0.0,
              toolbarHeight: 70.0,
              leading: Container(
                margin: const EdgeInsets.all(8.0),
                decoration: BoxDecoration(
                  color: Theme.of(context).colorScheme.surfaceContainerHighest,
                  shape: BoxShape.circle,
                ),
                child: IconButton(
                  icon: const Icon(Icons.arrow_back),
                  color: Colors.white,
                  onPressed: () {
                    Navigator.of(context).pop();
                  },
                ),
              ),
            ),
            body: SingleChildScrollView(
              child: Padding(
                padding: const EdgeInsets.all(20.0),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                      t(context)!.aboutInstanceName(platform.name),
                      style: textTheme.titleLarge,
                    ),
                    const SizedBox(height: 16),
                    Text(
                      platform.shortDescription,
                      style: textTheme.displayMedium,
                    ),
                    const Padding(
                      padding: EdgeInsets.symmetric(vertical: 4.0),
                      child: Divider(thickness: 2),
                    ),
                    ConditionalSection(
                      title: 'Information',
                      subtitles: {
                        'Description': platform.description,
                      },
                    ),
                    ConditionalSection(
                      title: 'Administrators & sustainability',
                      subtitles: {
                        'Who we are': platform.administrator,
                        'Why we created this platform': platform.creationReason,
                        'How long we plan to maintain this platform':
                            platform.maintenanceLifetime,
                        'How we will pay for keeping our platform running':
                            platform.businessModel,
                      },
                    ),
                    ConditionalSection(
                      title: 'Moderation',
                      subtitles: {
                        'Moderation information':
                            platform.moderationInformation,
                        'Code of Conduct': platform.codeOfConduct,
                        'Terms': platform.terms,
                      },
                    ),
                    ConditionalSection(
                      title: 'Other information',
                      subtitles: {
                        'Hardware information': platform.hardwareInformation,
                      },
                    ),
                    const SizedBox(height: 16),
                    ListTile(
                      title: Text(
                        t(context)!.showPlatformFeatures,
                        style: textTheme.headlineLarge,
                      ),
                      trailing: const Icon(Icons.arrow_forward),
                      onTap: () {
                        GoRouter.of(context).push(
                          '/about/features',
                        );
                      },
                    ),
                  ],
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class ConditionalSection extends StatelessWidget {
  final String? title;
  final Map<String?, String?> subtitles;

  const ConditionalSection({
    super.key,
    required this.title,
    required this.subtitles,
  });

  @override
  Widget build(BuildContext context) {
    bool hasContent = subtitles.values
        .any((subtitle) => subtitle != null && subtitle.isNotEmpty);

    if (!hasContent) {
      return const SizedBox.shrink();
    }

    final textTheme = Theme.of(context).textTheme;
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        if (title != null && title!.isNotEmpty)
          Text(
            title!,
            style: textTheme.titleMedium,
          ),
        ...subtitles.entries.map((entry) {
          final subtitleTitle = entry.key;
          final content = entry.value;

          if (content != null && content.isNotEmpty) {
            return Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                if (subtitleTitle != null && subtitleTitle.isNotEmpty)
                  Padding(
                    padding: const EdgeInsets.only(top: 8.0),
                    child: Text(
                      subtitleTitle,
                      style: textTheme.titleSmall,
                    ),
                  ),
                Padding(
                  padding: const EdgeInsets.only(top: 4.0),
                  child: CustomHtmlWidget(
                    rawHtml: md.markdownToHtml(content),
                  ),
                ),
              ],
            );
          }
          return const SizedBox.shrink();
        }),
        const SizedBox(height: 8),
      ],
    );
  }
}

class PlatformFeaturesScreen extends ConsumerWidget {
  const PlatformFeaturesScreen({super.key});

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final platform = ref.watch(currentPlatformNonNullableProvider);
    final asyncConfig =
        ref.watch(configServerProvider(Uri.parse(platform.domain).host));

    return Scaffold(
      appBar: AppBar(
        title: Text(t(context)!.featuresFoundOnThisPlatform),
      ),
      body: AsyncValueWidget(
        value: asyncConfig,
        dataBuilder: (config) {
          return ListView(
            padding: const EdgeInsets.all(16.0),
            children: [
              _buildFeatureRow(t(context)!.version, config.serverVersion),
              // _buildFeatureRow('Signup', config.signup ? '✔️' : '❌'),
              // _buildFeatureRow('Other Feature', config.otherFeature ? '✔️' : '❌'),
            ],
          );
        },
      ),
    );
  }

  Widget _buildFeatureRow(String feature, String value) {
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 4.0),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Text(
            feature,
            style: const TextStyle(fontWeight: FontWeight.bold),
          ),
          Text(value),
        ],
      ),
    );
  }
}
