import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:sentry_flutter/sentry_flutter.dart';

import 'package:peertube_mobile/features/filters/filters_widget.dart';
import 'package:peertube_mobile/features/video/player/video_player.dart';
import 'package:peertube_mobile/routes/abuse/abuse_screen.dart';
import 'package:peertube_mobile/routes/account_screen.dart';
import 'package:peertube_mobile/routes/explore/add_platform_screen.dart';
import 'package:peertube_mobile/routes/explore/explore_screen.dart';
import 'package:peertube_mobile/routes/explore/platform_jpt_list_screen.dart';
import 'package:peertube_mobile/routes/library/library_screen.dart';
import 'package:peertube_mobile/routes/library/local_history_screen.dart';
import 'package:peertube_mobile/routes/library/local_watch_later_screen.dart';
import 'package:peertube_mobile/routes/login_screen.dart';
import 'package:peertube_mobile/routes/onboarding_screen.dart';
import 'package:peertube_mobile/routes/platform/about_platform_screen.dart';
import 'package:peertube_mobile/routes/platform/platform_home_screen.dart';
import 'package:peertube_mobile/routes/settings/bug_report_screen.dart';
import 'package:peertube_mobile/routes/settings/interests_screen.dart';
import 'package:peertube_mobile/routes/settings/language_settings_screen.dart';
import 'package:peertube_mobile/routes/settings/playback_settings_screen.dart';
import 'package:peertube_mobile/routes/settings/settings_screen.dart';
import 'package:peertube_mobile/routes/utils/scaffold_with_navbar.dart';
import 'package:peertube_mobile/routes/video/video_channel_screen.dart';
import 'package:peertube_mobile/routes/video/video_screen.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';

final goRouterProvider = Provider<GoRouter>(
  (ref) {
    return GoRouter(
      initialLocation: (ref.read(onboardingProvider.notifier).state == false)
          ? '/onboarding'
          : '/explore',
      observers: [
        if (Sentry.isEnabled) SentryNavigatorObserver(),
      ],
      routes: [
        GoRoute(
          path: '/onboarding',
          builder: (_, __) => const OnboardingScreen(),
        ),
        StatefulShellRoute.indexedStack(
          builder: (
            BuildContext context,
            GoRouterState state,
            StatefulNavigationShell navigationShell,
          ) {
            return ScaffoldWithNavbar(
              navigationShell: navigationShell,
            );
          },
          branches: <StatefulShellBranch>[
            StatefulShellBranch(
              routes: [
                GoRoute(
                  path: '/explore',
                  builder: (_, __) => const ExploreScreen(),
                  routes: [
                    GoRoute(
                      path: 'filters/contents',
                      builder: (_, __) => const AllContentFiltersScreen(),
                    ),
                    GoRoute(
                      path: 'platforms',
                      builder: (_, __) => const PlatformJPTListScreen(),
                      routes: [
                        GoRoute(
                          path: 'add',
                          builder: (_, __) => const AddPlatformScreen(),
                        ),
                      ],
                    ),
                  ],
                ),
              ],
            ),
            StatefulShellBranch(
              routes: [
                GoRoute(
                  path: '/library',
                  builder: (_, __) => const LibraryScreen(),
                  routes: [
                    GoRoute(
                      path: 'history',
                      builder: (_, __) => const LocalHistoryScreen(),
                    ),
                    GoRoute(
                      path: 'playlists/:id',
                      builder: (_, __) => const WatchLaterScreen(),
                    ),
                  ],
                ),
              ],
            ),
            StatefulShellBranch(
              routes: [
                GoRoute(
                  path: '/',
                  redirect: (context, state) {
                    if (state.fullPath == '/') {
                      if (ref.read(currentPlatformProvider) != null) {
                        return '/platform';
                      }
                      return '/no-platform';
                    }
                    return null;
                  },
                  routes: [
                    GoRoute(
                      path: 'platform',
                      builder: (_, __) => const PlatformHomeScreen(),
                    ),
                    GoRoute(
                      path: 'no-platform',
                      builder: (_, __) => const NoPlatformScreen(),
                    ),
                    GoRoute(
                      path: 'video/:uuid',
                      pageBuilder: (context, state) =>
                          CustomTransitionPage<void>(
                        key: state.pageKey,
                        child: VideoScreen(
                          videoUuid: state.pathParameters['uuid']!,
                        ),
                        transitionsBuilder: (_, animation, __, child) {
                          var begin = const Offset(0.0, 1.0);
                          var end = Offset.zero;
                          var tween = Tween(begin: begin, end: end);
                          var offsetAnimation = animation.drive(tween);
                          return SlideTransition(
                            position: offsetAnimation,
                            child: child,
                          );
                        },
                      ),
                    ),
                    GoRoute(
                      path: 'about',
                      builder: (_, __) => const AboutPlatformScreen(),
                      routes: [
                        GoRoute(
                          path: 'features',
                          builder: (_, __) => const PlatformFeaturesScreen(),
                        ),
                      ],
                    ),
                    GoRoute(
                      path: 'video-channel/:channelHandle',
                      builder: (context, state) => VideoChannelScreen(
                        channelHandle: state.pathParameters['channelHandle']!,
                      ),
                    ),
                    GoRoute(
                      path: 'accounts/:name',
                      builder: (context, state) => AccountScreen(
                        name: state.pathParameters['name']!,
                      ),
                    ),
                  ],
                ),
              ],
            ),
            StatefulShellBranch(
              routes: [
                GoRoute(
                  path: '/login',
                  builder: (_, __) => const LoginScreen(),
                ),
              ],
            ),
          ],
        ),
        GoRoute(
          path: '/settings',
          builder: (_, __) => const SettingsScreen(),
          routes: [
            GoRoute(
              path: 'playback',
              builder: (_, __) => const PlaybackSettingsScreen(),
            ),
            GoRoute(
              path: 'languages',
              builder: (_, __) => const LanguageSettingsScreen(),
            ),
            GoRoute(
              path: 'preferences',
              builder: (_, __) => const InterestsScreen(),
            ),
            GoRoute(
              path: 'bug-report',
              builder: (_, __) => const ReportBugScreen(),
            ),
          ],
        ),
        GoRoute(
          path: '/full-screen',
          builder: (context, state) => const FullScreenPlayer(),
        ),
        GoRoute(
          path: '/report',
          builder: (context, state) => Container(),
          routes: [
            GoRoute(
              path: 'videos/:uuid',
              builder: (context, state) => ReportAbuseScreen(
                reportId: state.pathParameters['uuid']!,
                reportType: ReportType.video,
                host: state.uri.queryParameters['host']!,
                isLocalAccount: bool.parse(
                  state.uri.queryParameters['isLocalAccount']!,
                ),
              ),
            ),
            GoRoute(
              path: 'comments/:id',
              builder: (context, state) => ReportAbuseScreen(
                reportId: state.pathParameters['id']!,
                reportType: ReportType.comment,
                host: state.uri.queryParameters['host']!,
                isLocalAccount: bool.parse(
                  state.uri.queryParameters['isLocalAccount']!,
                ),
              ),
            ),
            GoRoute(
              path: 'accounts/:id',
              builder: (context, state) => ReportAbuseScreen(
                reportId: state.pathParameters['id']!,
                reportType: ReportType.account,
                host: state.uri.queryParameters['host']!,
                isLocalAccount: bool.parse(
                  state.uri.queryParameters['isLocalAccount']!,
                ),
              ),
            ),
            GoRoute(
              path: 'confirmation',
              builder: (context, state) => const ConfirmationScreen(),
            ),
          ],
        ),
      ],
    );
  },
  name: 'goRouterProvider',
);
