import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';

import 'package:peertube_mobile/shared/preferences/user_preferences.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/button.dart';

class InterestsScreen extends ConsumerStatefulWidget {
  const InterestsScreen({super.key});

  @override
  ConsumerState<InterestsScreen> createState() => _InterestsScreenState();
}

class _InterestsScreenState extends ConsumerState<InterestsScreen> {
  late Map<String, bool> _selectedCategories;
  late Map<String, bool> _selectedTags;
  bool _hasChanged = false;

  @override
  void initState() {
    super.initState();

    final sharedUtility = ref.read(sharedUtilityProvider);

    _selectedCategories = {
      for (var index in categoriesMap.keys)
        index: sharedUtility.selectedCategories.contains(index),
    };

    _selectedTags = {
      for (var tag in tags) tag: sharedUtility.selectedTags.contains(tag),
    };
  }

  void _onTagTapped(String tag) {
    setState(() {
      _selectedTags[tag] = !_selectedTags[tag]!;
      _hasChanged = true;
    });
  }

  void _onCategoryTapped(String index) {
    setState(() {
      _selectedCategories[index] = !_selectedCategories[index]!;
      _hasChanged = true;
    });
  }

  void _applyChanges() {
    final sharedUtility = ref.read(sharedUtilityProvider);

    sharedUtility.setSelectedCategories(
      _selectedCategories.entries
          .where((entry) => entry.value)
          .map((entry) => entry.key)
          .toList(),
    );

    sharedUtility.setSelectedTags(
      _selectedTags.entries
          .where((entry) => entry.value)
          .map((entry) => entry.key)
          .toList(),
    );

    setState(() {
      _hasChanged = false;
    });
  }

  @override
  Widget build(BuildContext context) {
    var textTheme = Theme.of(context).textTheme;
    var colorScheme = Theme.of(context).colorScheme;
    return Scaffold(
      appBar: AppBar(
        title: Text(t(context)!.myInterests),
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: Stack(
          children: [
            SingleChildScrollView(
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Text(
                    t(context)!.categories,
                    style: textTheme.displayMedium!
                        .copyWith(color: colorScheme.onSecondaryContainer),
                  ),
                  const SizedBox(height: 16),
                  Wrap(
                    spacing: 8.0,
                    children: categoriesMap.keys.map((index) {
                      return FilterChip(
                        label: Text(getCategoryString(context, index)),
                        selected: _selectedCategories[index]!,
                        onSelected: (bool selected) {
                          _onCategoryTapped(index);
                        },
                      );
                    }).toList(),
                  ),
                  const SizedBox(height: 32),
                  if (tags.isNotEmpty)
                    Text(
                      t(context)!.tags,
                      style: textTheme.displayMedium!
                          .copyWith(color: colorScheme.onSecondaryContainer),
                    ),
                  const SizedBox(height: 16),
                  Wrap(
                    spacing: 8.0,
                    children: tags.map((String tag) {
                      return FilterChip(
                        label: Text(tag),
                        selected: _selectedTags[tag]!,
                        onSelected: (bool selected) {
                          _onTagTapped(tag);
                        },
                      );
                    }).toList(),
                  ),
                  const SizedBox(height: 100),
                ],
              ),
            ),
            Stack(
              children: [
                if (_hasChanged)
                  Positioned(
                    bottom: 16,
                    child: CustomElevatedButton(
                      onPressed: _applyChanges,
                      child: Text(t(context)!.apply),
                    ),
                  ),
              ],
            ),
          ],
        ),
      ),
    );
  }
}

const Map<String, String> categoriesMap = {
  '1': 'Music',
  '2': 'Films',
  '3': 'Vehicles',
  '4': 'Art',
  '5': 'Sports',
  '6': 'Travels',
  '7': 'Gaming',
  '8': 'People',
  '9': 'Comedy',
  '10': 'Entertainment',
  '11': 'News & Politics',
  '12': 'How To',
  '13': 'Education',
  '14': 'Activism',
  '15': 'Science & Technology',
  '16': 'Animals',
  '17': 'Kids',
  '18': 'Food',
};

String getCategoryString(BuildContext context, String index) {
  switch (index) {
    case '1':
      return t(context)!.music;
    case '2':
      return t(context)!.films;
    case '3':
      return t(context)!.vehicles;
    case '4':
      return t(context)!.art;
    case '5':
      return t(context)!.sports;
    case '6':
      return t(context)!.travels;
    case '7':
      return t(context)!.gaming;
    case '8':
      return t(context)!.people;
    case '9':
      return t(context)!.comedy;
    case '10':
      return t(context)!.entertainment;
    case '11':
      return t(context)!.newsPolitics;
    case '12':
      return t(context)!.howTo;
    case '13':
      return t(context)!.education;
    case '14':
      return t(context)!.activism;
    case '15':
      return t(context)!.scienceTechnology;
    case '16':
      return t(context)!.animals;
    case '17':
      return t(context)!.kids;
    case '18':
      return t(context)!.food;
    default:
      return categoriesMap[index]!;
  }
}

const List<String> tags = [];
