import 'package:flutter/material.dart';

import 'package:flutter_gen/gen_l10n/app_localizations.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/routes/settings/settings_screen.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/button.dart';

class LanguageSettingsScreen extends ConsumerWidget {
  final bool isDarkTheme = false;

  const LanguageSettingsScreen({super.key});

  @override
  Widget build(context, ref) {
    var textTheme = Theme.of(context).textTheme;
    var colorScheme = Theme.of(context).colorScheme;
    return Scaffold(
      appBar: AppBar(
        title: Text(t(context)!.languagesSettings),
      ),
      body: ListView(
        children: <Widget>[
          Padding(
            padding: const EdgeInsets.all(16.0),
            child: Text(
              t(context)!.languagesSettings,
              style: textTheme.displayMedium!
                  .copyWith(color: colorScheme.onSecondaryContainer),
            ),
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.languages,
            title: 'Language of the app',
            subtitle: t(context)!.language,
            onTap: () => _showLanguageDialog(context, ref),
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.globe,
            title: 'Content languages',
            subtitle:
                'Only display videos in the following languages/subtitles',
            openNewPage: true,
            onTap: () {
              Navigator.push(
                context,
                MaterialPageRoute(
                  builder: (context) => const ContentLanguageSettingsScreen(),
                ),
              );
            },
          ),
        ],
      ),
    );
  }
}

void _showLanguageDialog(BuildContext context, WidgetRef ref) {
  showDialog(
    context: context,
    builder: (BuildContext context) {
      return AlertDialog(
        title: Text(t(context)!.selectLanguage),
        content: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            for (var locale in AppLocalizations.supportedLocales)
              Localizations.override(
                context: context,
                locale: locale,
                child: Builder(
                  builder: (context) {
                    if (locale.languageCode != 'en' &&
                        t(context)!.language == 'English') {
                      return SizedBox.shrink();
                    }
                    return RadioListTile<Locale>(
                      title: Text(
                        '${locale.toString()} - ${t(context)!.language}',
                      ),
                      value: locale,
                      groupValue: ref.watch(localeProvider),
                      onChanged: (newLocale) => {
                        if (newLocale != null)
                          {
                            ref.read(localeProvider.notifier).state = newLocale,
                          },
                        GoRouter.of(context).pop(true),
                      },
                    );
                  },
                ),
              ),
          ],
        ),
      );
    },
  );
}

class ContentLanguageSettingsScreen extends ConsumerStatefulWidget {
  const ContentLanguageSettingsScreen({super.key});

  @override
  ConsumerState<ConsumerStatefulWidget> createState() =>
      _ContentLanguageSettingsScreenState();
}

class _ContentLanguageSettingsScreenState
    extends ConsumerState<ContentLanguageSettingsScreen> {
  List<String> selectedLanguages = [];
  List<String> filteredLanguages = [];
  String searchQuery = '';
  bool selectAll = false;
  bool showApplyButton = false;

  @override
  void initState() {
    super.initState();
    filteredLanguages = languages;
    final sharedUtility = ref.read(sharedUtilityProvider);

    selectedLanguages = sharedUtility.selectedLanguages;

    if (selectedLanguages.isEmpty) toggleSelectAll();
  }

  void toggleSelectAll() {
    setState(() {
      if (selectAll) {
        selectedLanguages.clear();
      } else {
        selectedLanguages = List.from(filteredLanguages);
      }
      selectAll = !selectAll;
      showApplyButton = true;
    });
  }

  void filterLanguages(String query) {
    setState(() {
      searchQuery = query.toLowerCase();
      filteredLanguages = languages
          .where(
            (language) => getLanguageString(context, language)
                .toLowerCase()
                .contains(searchQuery),
          )
          .toList();
    });
  }

  @override
  Widget build(BuildContext context) {
    var textTheme = Theme.of(context).textTheme;
    var colorScheme = Theme.of(context).colorScheme;

    return Scaffold(
      appBar: AppBar(
        title: Text(t(context)!.selectContentLanguages),
        actions: [
          TextButton(
            onPressed: toggleSelectAll,
            child: Text(
              selectAll ? t(context)!.deselectAll : t(context)!.selectAll,
              style: const TextStyle(color: Colors.white),
            ),
          ),
        ],
      ),
      body: Column(
        children: [
          Padding(
            padding: const EdgeInsets.all(16.0),
            child: Text(
              t(context)!.contentLanguages,
              style: textTheme.headlineSmall!
                  .copyWith(color: colorScheme.onSecondaryContainer),
            ),
          ),
          Padding(
            padding: const EdgeInsets.symmetric(horizontal: 16.0),
            child: TextField(
              decoration: const InputDecoration(
                labelText: 'Search language',
                prefixIcon: Icon(Icons.search),
              ),
              onChanged: (value) => filterLanguages(value),
            ),
          ),
          const SizedBox(height: 8),
          Expanded(
            child: ListView(
              children: filteredLanguages.map((language) {
                return CheckboxListTile(
                  title: Text(getLanguageString(context, language)),
                  value: selectedLanguages.contains(language),
                  onChanged: (bool? value) {
                    setState(() {
                      if (value == true) {
                        selectedLanguages.add(language);
                      } else {
                        selectedLanguages.remove(language);
                      }
                      showApplyButton = true;
                    });
                  },
                );
              }).toList(),
            ),
          ),
        ],
      ),
      bottomNavigationBar: showApplyButton
          ? Padding(
              padding: const EdgeInsets.all(16.0),
              child: CustomElevatedButton(
                onPressed: () async {
                  await ref
                      .read(sharedUtilityProvider)
                      .setSelectedLanguages(selectedLanguages);

                  if (context.mounted) Navigator.pop(context);
                },
                child: Text(t(context)!.apply),
              ),
            )
          : const SizedBox.shrink(),
    );
  }
}

const List<String> languages = [
  '_unknown',
  'ab',
  'af',
  'ak',
  'am',
  'ar',
  'an',
  'ase',
  'as',
  'asq',
  'av',
  'avk',
  'ay',
  'az',
  'ba',
  'bm',
  'be',
  'bn',
  'bfi',
  'bi',
  'bo',
  'bs',
  'br',
  'bg',
  'bzs',
  'ca',
  'cs',
  'ch',
  'ce',
  'cv',
  'kw',
  'co',
  'cr',
  'cse',
  'csl',
  'cy',
  'da',
  'de',
  'dv',
  'dsl',
  'dz',
  'el',
  'en',
  'eo',
  'et',
  'eu',
  'ee',
  'fo',
  'fa',
  'fj',
  'fi',
  'fr',
  'fy',
  'fse',
  'fsl',
  'ff',
  'gcf',
  'gd',
  'ga',
  'gl',
  'gv',
  'gn',
  'gsg',
  'gu',
  'ht',
  'ha',
  'sh',
  'he',
  'hz',
  'hi',
  'ho',
  'hr',
  'hu',
  'hy',
  'ig',
  'ii',
  'iu',
  'id',
  'ik',
  'is',
  'it',
  'jv',
  'jbo',
  'ja',
  'jsl',
  'kab',
  'kl',
  'kn',
  'ks',
  'ka',
  'kr',
  'kk',
  'km',
  'ki',
  'rw',
  'ky',
  'kv',
  'kg',
  'ko',
  'kj',
  'ku',
  'lo',
  'la',
  'lv',
  'li',
  'ln',
  'lt',
  'lb',
  'lu',
  'lg',
  'mh',
  'ml',
  'mr',
  'mk',
  'mg',
  'mt',
  'mn',
  'mi',
  'ms',
  'my',
  'na',
  'nv',
  'nr',
  'nd',
  'ng',
  'ne',
  'nl',
  'nn',
  'nb',
  'no',
  'ny',
  'oc',
  'oj',
  'or',
  'om',
  'os',
  'pa',
  'pks',
  'pl',
  'pt',
  'ps',
  'qu',
  'rm',
  'ro',
  'rsl',
  'rn',
  'ru',
  'sg',
  'sdl',
  'sfs',
  'si',
  'sk',
  'sl',
  'se',
  'sm',
  'sn',
  'sd',
  'so',
  'st',
  'es',
  'sq',
  'sc',
  'sr',
  'ss',
  'su',
  'sw',
  'sv',
  'swl',
  'ty',
  'ta',
  'tt',
  'te',
  'tg',
  'tl',
  'th',
  'ti',
  'tlh',
  'to',
  'tn',
  'ts',
  'tk',
  'tr',
  'tw',
  'ug',
  'uk',
  'ur',
  'uz',
  've',
  'vi',
  'wa',
  'wo',
  'xh',
  'yi',
  'yo',
  'za',
  'zh',
  'zu',
  'zxx',
  'tok',
  'pt-PT',
  'es-419',
  'zh-Hans',
  'zh-Hant',
  'ca-valencia',
];

String getLanguageString(BuildContext context, String code) {
  switch (code) {
    case '_unknown':
      return t(context)!.unknownLanguage;
    case 'aa':
      return t(context)!.afar;
    case 'ab':
      return t(context)!.abkhazian;
    case 'af':
      return t(context)!.afrikaans;
    case 'ak':
      return t(context)!.akan;
    case 'am':
      return t(context)!.amharic;
    case 'ar':
      return t(context)!.arabic;
    case 'an':
      return t(context)!.aragonese;
    case 'ase':
      return t(context)!.americanSignLanguage;
    case 'as':
      return t(context)!.assamese;
    case 'asq':
      return t(context)!.austrianSignLanguage;
    case 'av':
      return t(context)!.avaric;
    case 'avk':
      return t(context)!.kotava;
    case 'ay':
      return t(context)!.aymara;
    case 'az':
      return t(context)!.azerbaijani;
    case 'ba':
      return t(context)!.bashkir;
    case 'bm':
      return t(context)!.bambara;
    case 'be':
      return t(context)!.belarusian;
    case 'bn':
      return t(context)!.bengali;
    case 'bfi':
      return t(context)!.britishSignLanguage;
    case 'bi':
      return t(context)!.bislama;
    case 'bo':
      return t(context)!.tibetan;
    case 'bs':
      return t(context)!.bosnian;
    case 'br':
      return t(context)!.breton;
    case 'bg':
      return t(context)!.bulgarian;
    case 'bzs':
      return t(context)!.brazilianSignLanguage;
    case 'ca':
      return t(context)!.catalan;
    case 'cs':
      return t(context)!.czech;
    case 'ch':
      return t(context)!.chamorro;
    case 'ce':
      return t(context)!.chechen;
    case 'cv':
      return t(context)!.chuvash;
    case 'kw':
      return t(context)!.cornish;
    case 'co':
      return t(context)!.corsican;
    case 'cr':
      return t(context)!.cree;
    case 'cse':
      return t(context)!.czechSignLanguage;
    case 'csl':
      return t(context)!.chineseSignLanguage;
    case 'cy':
      return t(context)!.welsh;
    case 'da':
      return t(context)!.danish;
    case 'de':
      return t(context)!.german;
    case 'dv':
      return t(context)!.dhivehi;
    case 'dsl':
      return t(context)!.danishSignLanguage;
    case 'dz':
      return t(context)!.dzongkha;
    case 'el':
      return t(context)!.greek;
    case 'en':
      return t(context)!.english;
    case 'eo':
      return t(context)!.esperanto;
    case 'et':
      return t(context)!.estonian;
    case 'eu':
      return t(context)!.basque;
    case 'ee':
      return t(context)!.ewe;
    case 'fo':
      return t(context)!.faroese;
    case 'fa':
      return t(context)!.persian;
    case 'fj':
      return t(context)!.fijian;
    case 'fi':
      return t(context)!.finnish;
    case 'fr':
      return t(context)!.french;
    case 'fy':
      return t(context)!.westernFrisian;
    case 'fse':
      return t(context)!.finnishSignLanguage;
    case 'fsl':
      return t(context)!.frenchSignLanguage;
    case 'ff':
      return t(context)!.fulah;
    case 'gcf':
      return t(context)!.guadeloupeanCreoleFrench;
    case 'gd':
      return t(context)!.scottishGaelic;
    case 'ga':
      return t(context)!.irish;
    case 'gl':
      return t(context)!.galician;
    case 'gv':
      return t(context)!.manx;
    case 'gn':
      return t(context)!.guarani;
    case 'gsg':
      return t(context)!.germanSignLanguage;
    case 'gu':
      return t(context)!.gujarati;
    case 'ht':
      return t(context)!.haitian;
    case 'ha':
      return t(context)!.hausa;
    case 'sh':
      return t(context)!.serboCroatian;
    case 'he':
      return t(context)!.hebrew;
    case 'hz':
      return t(context)!.herero;
    case 'hi':
      return t(context)!.hindi;
    case 'ho':
      return t(context)!.hiriMotu;
    case 'hr':
      return t(context)!.croatian;
    case 'hu':
      return t(context)!.hungarian;
    case 'hy':
      return t(context)!.armenian;
    case 'ig':
      return t(context)!.igbo;
    case 'ii':
      return t(context)!.sichuanYi;
    case 'iu':
      return t(context)!.inuktitut;
    case 'id':
      return t(context)!.indonesian;
    case 'ik':
      return t(context)!.inupiaq;
    case 'is':
      return t(context)!.icelandic;
    case 'it':
      return t(context)!.italian;
    case 'jv':
      return t(context)!.javanese;
    case 'jbo':
      return t(context)!.lojban;
    case 'ja':
      return t(context)!.japanese;
    case 'jsl':
      return t(context)!.japaneseSignLanguage;
    case 'kab':
      return t(context)!.kabyle;
    case 'kl':
      return t(context)!.kalaallisut;
    case 'kn':
      return t(context)!.kannada;
    case 'ks':
      return t(context)!.kashmiri;
    case 'ka':
      return t(context)!.georgian;
    case 'kr':
      return t(context)!.kanuri;
    case 'kk':
      return t(context)!.kazakh;
    case 'km':
      return t(context)!.khmer;
    case 'ki':
      return t(context)!.kikuyu;
    case 'rw':
      return t(context)!.kinyarwanda;
    case 'ky':
      return t(context)!.kirghiz;
    case 'kv':
      return t(context)!.komi;
    case 'kg':
      return t(context)!.kongo;
    case 'ko':
      return t(context)!.korean;
    case 'kj':
      return t(context)!.kuanyama;
    case 'ku':
      return t(context)!.kurdish;
    case 'lo':
      return t(context)!.lao;
    case 'la':
      return t(context)!.latin;
    case 'lv':
      return t(context)!.latvian;
    case 'li':
      return t(context)!.limburgan;
    case 'ln':
      return t(context)!.lingala;
    case 'lt':
      return t(context)!.lithuanian;
    case 'lb':
      return t(context)!.luxembourgish;
    case 'lu':
      return t(context)!.lubaKatanga;
    case 'lg':
      return t(context)!.ganda;
    case 'mh':
      return t(context)!.marshallese;
    case 'ml':
      return t(context)!.malayalam;
    case 'mr':
      return t(context)!.marathi;
    case 'mk':
      return t(context)!.macedonian;
    case 'mg':
      return t(context)!.malagasy;
    case 'mt':
      return t(context)!.maltese;
    case 'mn':
      return t(context)!.mongolian;
    case 'mi':
      return t(context)!.maori;
    case 'ms':
      return t(context)!.malaymacrolanguage;
    case 'my':
      return t(context)!.burmese;
    case 'na':
      return t(context)!.nauru;
    case 'nv':
      return t(context)!.navajo;
    case 'nr':
      return t(context)!.southNdebele;
    case 'nd':
      return t(context)!.northNdebele;
    case 'ng':
      return t(context)!.ndonga;
    case 'ne':
      return t(context)!.nepalimacrolanguage;
    case 'nl':
      return t(context)!.dutch;
    case 'nn':
      return t(context)!.norwegianNynorsk;
    case 'nb':
      return t(context)!.norwegianBokmal;
    case 'no':
      return t(context)!.norwegian;
    case 'ny':
      return t(context)!.nyanja;
    case 'oc':
      return t(context)!.occitan;
    case 'oj':
      return t(context)!.ojibwa;
    case 'or':
      return t(context)!.oriyamacrolanguage;
    case 'om':
      return t(context)!.oromo;
    case 'os':
      return t(context)!.ossetian;
    case 'pa':
      return t(context)!.panjabi;
    case 'pks':
      return t(context)!.pakistanSignLanguage;
    case 'pl':
      return t(context)!.polish;
    case 'pt':
      return t(context)!.portugueseBrazilian;
    case 'ps':
      return t(context)!.pushto;
    case 'qu':
      return t(context)!.quechua;
    case 'rm':
      return t(context)!.romansh;
    case 'ro':
      return t(context)!.romanian;
    case 'rsl':
      return t(context)!.russianSignLanguage;
    case 'rn':
      return t(context)!.rundi;
    case 'ru':
      return t(context)!.russian;
    case 'sg':
      return t(context)!.sango;
    case 'sdl':
      return t(context)!.saudiArabianSignLanguage;
    case 'sfs':
      return t(context)!.southAfricanSignLanguage;
    case 'si':
      return t(context)!.sinhala;
    case 'sk':
      return t(context)!.slovak;
    case 'sl':
      return t(context)!.slovenian;
    case 'se':
      return t(context)!.northernSami;
    case 'sm':
      return t(context)!.samoan;
    case 'sn':
      return t(context)!.shona;
    case 'sd':
      return t(context)!.sindhi;
    case 'so':
      return t(context)!.somali;
    case 'st':
      return t(context)!.southernSotho;
    case 'es':
      return t(context)!.spanishSpain;
    case 'sq':
      return t(context)!.albanian;
    case 'sc':
      return t(context)!.sardinian;
    case 'sr':
      return t(context)!.serbian;
    case 'ss':
      return t(context)!.swati;
    case 'su':
      return t(context)!.sundanese;
    case 'sw':
      return t(context)!.swahilimacrolanguage;
    case 'sv':
      return t(context)!.swedish;
    case 'swl':
      return t(context)!.swedishSignLanguage;
    case 'ty':
      return t(context)!.tahitian;
    case 'ta':
      return t(context)!.tamil;
    case 'tt':
      return t(context)!.tatar;
    case 'te':
      return t(context)!.telugu;
    case 'tg':
      return t(context)!.tajik;
    case 'tl':
      return t(context)!.tagalog;
    case 'th':
      return t(context)!.thai;
    case 'ti':
      return t(context)!.tigrinya;
    case 'tlh':
      return t(context)!.klingon;
    case 'to':
      return t(context)!.tongaTongaIslands;
    case 'tn':
      return t(context)!.tswana;
    case 'ts':
      return t(context)!.tsonga;
    case 'tk':
      return t(context)!.turkmen;
    case 'tr':
      return t(context)!.turkish;
    case 'tw':
      return t(context)!.twi;
    case 'ug':
      return t(context)!.uighur;
    case 'uk':
      return t(context)!.ukrainian;
    case 'ur':
      return t(context)!.urdu;
    case 'uz':
      return t(context)!.uzbek;
    case 've':
      return t(context)!.venda;
    case 'vi':
      return t(context)!.vietnamese;
    case 'wa':
      return t(context)!.walloon;
    case 'wo':
      return t(context)!.wolof;
    case 'xh':
      return t(context)!.xhosa;
    case 'yi':
      return t(context)!.yiddish;
    case 'yo':
      return t(context)!.yoruba;
    case 'za':
      return t(context)!.zhuang;
    case 'zh':
      return t(context)!.chinese;
    case 'zu':
      return t(context)!.zulu;
    case 'zxx':
      return t(context)!.nolinguisticcontent;
    case 'tok':
      return t(context)!.tokiPona;
    case 'pt-PT':
      return t(context)!.portuguesePortugal;
    case 'es-419':
      return t(context)!.spanishLatinAmerica;
    case 'zh-Hans':
      return t(context)!.simplifiedChinese;
    case 'zh-Hant':
      return t(context)!.traditionalChinese;
    case 'ca-valencia':
      return t(context)!.valencian;
    default:
      return code;
  }
}
