import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:lucide_icons/lucide_icons.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:sentry_flutter/sentry_flutter.dart';
import 'package:url_launcher/url_launcher.dart';

import 'package:peertube_mobile/shared/themes/theme_provider.dart';
import 'package:peertube_mobile/shared/user/nsfw_policy_enum.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';

class SettingsScreen extends ConsumerWidget {
  final bool isDarkTheme = false;

  const SettingsScreen({super.key});

  @override
  Widget build(context, ref) {
    var textTheme = Theme.of(context).textTheme;
    var colorScheme = Theme.of(context).colorScheme;
    return Scaffold(
      appBar: AppBar(
        title: Text(t(context)!.settings),
      ),
      body: ListView(
        children: <Widget>[
          Padding(
            padding: const EdgeInsets.all(16.0),
            child: Text(
              t(context)!.phoneSettings,
              style: textTheme.displayMedium!
                  .copyWith(color: colorScheme.onSecondaryContainer),
            ),
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.play,
            title: t(context)!.playbackSettings,
            openNewPage: true,
            onTap: () => GoRouter.of(context).push(
              '/settings/playback',
            ),
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.languages,
            title: t(context)!.languagesSettings,
            openNewPage: true,
            onTap: () => GoRouter.of(context).push(
              '/settings/languages',
            ),
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.sunMoon,
            title: t(context)!.theme,
            subtitle: _getThemeString(
              context,
              ref.watch(themeProvider),
            ),
            onTap: () => _showThemeDialog(context, ref),
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.heart,
            title: t(context)!.myInterests,
            openNewPage: true,
            onTap: () => GoRouter.of(context).push(
              '/settings/preferences',
            ),
          ),
          const Padding(
            padding: EdgeInsets.only(top: 8.0, bottom: 18.0),
            child: Divider(),
          ),
          Padding(
            padding: const EdgeInsets.all(16.0),
            child: Text(
              'PeerTube',
              style: textTheme.displayMedium!
                  .copyWith(color: colorScheme.onSecondaryContainer),
            ),
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.info,
            title: t(context)!.aboutPeertube,
            onTap: () async {
              final url = Uri.parse('https://joinpeertube.org/');
              await launchUrl(
                url,
                mode: LaunchMode.externalApplication,
              );
            },
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.messageCircle,
            title: t(context)!.giveMyOpinion,
            onTap: () async {
              final url = Uri.parse(
                'https://asso.framasoft.org/dolo/h/peertube-app-beta-feedback/',
              );
              await launchUrl(url);
            },
          ),
          buildSettingItem(
            context,
            icon: LucideIcons.bug,
            title: t(context)!.reportBug,
            onTap: () async {
              if (Sentry.isEnabled) {
                GoRouter.of(context).push('/settings/bug-report');
              } else {
                ScaffoldMessenger.of(context).showSnackBar(
                  SnackBar(
                    content: Text(
                      t(context)!.sentryIsNotActivatedPleaseTryAgainLater,
                    ),
                    duration: const Duration(seconds: 3),
                  ),
                );
              }
            },
          ),
          FutureBuilder<String>(
            future: _getAppVersion(),
            builder: (context, snapshot) {
              if (snapshot.connectionState == ConnectionState.waiting) {
                return const CircularProgressIndicator();
              } else {
                return buildSettingItem(
                  context,
                  title: 'Version',
                  subtitle: snapshot.hasError || snapshot.data != null
                      ? snapshot.data
                      : t(context)!.unknownVersion,
                );
              }
            },
          ),
          // _buildSettingItem(
          //   context,
          //   icon: LucideIcons.logOut,
          //   title: 'Logout',
          //   value: 'Log out of your account',
          //   onTap: () async {
          //     await ref.read(authServiceProvider).logout(
          //           ref.watch(currentPlatformNonNullableProvider),
          //           ref.watch(peertubeClientProvider),
          //         );

          //     if (context.mounted) GoRouter.of(context).go('/');
          //   },
          // ),
        ],
      ),
    );
  }

  Future<String> _getAppVersion() async {
    final packageInfo = await PackageInfo.fromPlatform();
    return '${packageInfo.version}+${packageInfo.buildNumber}';
  }
}

Widget buildSettingItem(
  BuildContext context, {
  IconData? icon,
  required String title,
  String? subtitle,
  bool? switchValue,
  bool openNewPage = false,
  ValueChanged<bool>? onSwitchChanged,
  GestureTapCallback? onTap,
}) {
  return ListTile(
    contentPadding: const EdgeInsets.symmetric(vertical: 0.0, horizontal: 16.0),
    visualDensity: subtitle != null
        ? const VisualDensity(horizontal: 0, vertical: -4)
        : null,
    leading: icon != null ? Icon(icon) : null,
    title: Text(title),
    subtitle: subtitle != null ? Text(subtitle) : null,
    trailing: switchValue != null
        ? Switch(
            value: switchValue,
            onChanged: onSwitchChanged,
          )
        : openNewPage
            ? const Icon(
                LucideIcons.arrowRight,
                size: 32,
              )
            : null,
    onTap: () {
      if (onSwitchChanged != null && switchValue != null) {
        onSwitchChanged(!switchValue);
      }
      if (onTap != null) {
        onTap();
      }
    },
  );
}

void _showThemeDialog(BuildContext context, WidgetRef ref) {
  final themeNotifier = ref.read(themeProvider.notifier);
  showDialog(
    context: context,
    builder: (BuildContext context) {
      return AlertDialog(
        title: Text(t(context)!.selectTheme),
        content: Column(
          mainAxisSize: MainAxisSize.min,
          children: <Widget>[
            for (var themeMode in ThemeMode.values)
              RadioListTile<ThemeMode>(
                title: Text(_getThemeString(context, themeMode)),
                value: themeMode,
                groupValue: themeNotifier.state,
                onChanged: (newTheme) {
                  themeNotifier.state = newTheme!;
                  GoRouter.of(context).pop(true);
                },
              ),
          ],
        ),
      );
    },
  );
}

String _getThemeString(BuildContext context, ThemeMode mode) {
  switch (mode) {
    case ThemeMode.system:
      return t(context)!.systemTheme;
    case ThemeMode.light:
      return t(context)!.lightTheme;
    case ThemeMode.dark:
      return t(context)!.darkTheme;
  }
}

String getNsfwPolicyString(BuildContext context, NsfwPolicy policy) {
  switch (policy) {
    // case NsfwPolicy.blur:
    //   return 'Blur thumbnails';
    case NsfwPolicy.display:
      return t(context)!.display;
    case NsfwPolicy.doNotList:
      return t(context)!.hide;
  }
}

final nsfwPolicyProvider = Provider<NsfwPolicy>(
  (ref) => NsfwPolicy.doNotList,
  name: 'nsfwPolicyProvider',
);
