import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:flutter_svg/svg.dart';
import 'package:go_router/go_router.dart';

import 'package:peertube_mobile/features/video/miniplayer/miniplayer.dart';
import 'package:peertube_mobile/shared/themes/color_schemes.dart';

const double playerMinHeight = 60.0;

final ValueNotifier<double> playerExpandProgress =
    ValueNotifier<double>(playerMinHeight);

class ScaffoldWithNavbar extends ConsumerWidget {
  final StatefulNavigationShell navigationShell;

  const ScaffoldWithNavbar({
    super.key,
    required this.navigationShell,
  });

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final colorScheme = Theme.of(context).colorScheme;
    final currentString = GoRouterState.of(context).uri.toString();
    return Semantics(
      blockUserActions: true,
      child: GestureDetector(
        onTap: () => FocusScope.of(context).unfocus(),
        child: Scaffold(
          body: Container(
            color: colorScheme.surfaceContainerLowest,
            child: Stack(
              children: [
                navigationShell,
                Column(
                  children: [
                    const Spacer(),
                    Offstage(
                      offstage: currentString.contains('/video/'),
                      child: const PeerTubeMiniPlayer(),
                    ),
                  ],
                ),
              ],
            ),
          ),
          bottomNavigationBar:
              CustomBottomNavigationBar(navigationShell: navigationShell),
        ),
      ),
    );
  }
}

class CustomBottomNavigationBar extends StatelessWidget {
  const CustomBottomNavigationBar({
    super.key,
    required this.navigationShell,
  });

  final StatefulNavigationShell navigationShell;

  @override
  Widget build(BuildContext context) {
    return BottomNavigationBar(
      type: BottomNavigationBarType.fixed,
      currentIndex: navigationShell.currentIndex,
      onTap: (index) => navigationShell.goBranch(index, initialLocation: true),
      items: const <BottomNavigationBarItem>[
        BottomNavigationBarItem(
          icon: BottomNavigationBarIcon(
            assetName: 'assets/explore.svg',
            label: 'Explore',
          ),
          activeIcon: BottomNavigationBarIcon(
            assetName: 'assets/explore_selected.svg',
            label: 'Explore',
            isSelected: true,
          ),
          label: 'Explore',
        ),
        BottomNavigationBarItem(
          icon: BottomNavigationBarIcon(
            assetName: 'assets/library.svg',
            label: 'Library',
          ),
          activeIcon: BottomNavigationBarIcon(
            assetName: 'assets/library_selected.svg',
            label: 'Library',
            isSelected: true,
          ),
          label: 'Library',
        ),
        BottomNavigationBarItem(
          icon: BottomNavigationBarIcon(
            assetName: 'assets/home.svg',
            label: 'Platforms',
          ),
          activeIcon: BottomNavigationBarIcon(
            assetName: 'assets/home_selected.svg',
            label: 'Platforms',
            isSelected: true,
          ),
          label: 'Platforms',
        ),
      ],
    );
  }
}

class BottomNavigationBarIcon extends StatelessWidget {
  final String assetName;
  final String label;
  final bool isSelected;

  const BottomNavigationBarIcon({
    super.key,
    required this.assetName,
    required this.label,
    this.isSelected = false,
  });

  @override
  Widget build(BuildContext context) {
    final colorScheme = Theme.of(context).colorScheme;
    return Stack(
      alignment: Alignment.topCenter,
      children: [
        Container(
          width: 44,
          height: 44,
          decoration: BoxDecoration(
            shape: BoxShape.circle,
            color:
                isSelected ? colorScheme.surfaceContainer : Colors.transparent,
          ),
        ),
        Padding(
          padding: const EdgeInsets.all(3.0),
          child: Column(
            children: [
              SvgPicture(
                width: 24,
                SvgAssetLoader(
                  assetName,
                  colorMapper: MyColorMapper(
                    baseColor: isSelected
                        ? colorScheme.onPrimary
                        : colorScheme.onSecondary,
                    accentColor: colorScheme.primary,
                  ),
                ),
              ),
              Text(
                label,
                style: Theme.of(context).textTheme.displaySmall!.copyWith(
                      color: isSelected
                          ? colorScheme.onPrimary
                          : colorScheme.onSecondary,
                      fontWeight:
                          isSelected ? FontWeight.bold : FontWeight.normal,
                    ),
              ),
            ],
          ),
        ),
      ],
    );
  }
}

class MyColorMapper implements ColorMapper {
  const MyColorMapper({
    required this.baseColor,
    this.accentColor,
  });

  static const _rawBaseColor = beige300;
  static const _rawAccentColor = orange400;

  final Color baseColor;
  final Color? accentColor;

  @override
  Color substitute(
    String? id,
    String elementName,
    String attributeName,
    Color color,
  ) {
    if (color == _rawBaseColor) return baseColor;

    final accentColor = this.accentColor;
    if (accentColor != null && color == _rawAccentColor) return accentColor;

    return color;
  }
}
