import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:intl/intl.dart';
import 'package:lucide_icons/lucide_icons.dart';
import 'package:markdown/markdown.dart' as md;
import 'package:readmore/readmore.dart';

import 'package:peertube_mobile/features/platform/platform_banner.dart';
import 'package:peertube_mobile/features/video/channel/channel_providers.dart';
import 'package:peertube_mobile/features/video/video_list/video_list_view.dart';
import 'package:peertube_mobile/features/video/video_providers.dart';
import 'package:peertube_mobile/shared/config/config_providers.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/tools.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/app_bar.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/avatar_builder.dart';
import 'package:peertube_mobile/shared/widgets/dialog.dart';
import 'package:peertube_mobile/shared/widgets/html.dart';
import 'package:peertube_mobile/shared/widgets/image.dart';
import 'package:peertube_mobile/shared/widgets/tabbed_page_scaffold.dart';

class VideoChannelScreen extends ConsumerWidget {
  final String channelHandle;

  const VideoChannelScreen({
    super.key,
    required this.channelHandle,
  });

  @override
  Widget build(context, ref) {
    final platform = ref.watch(currentPlatformNonNullableProvider);

    final asyncTotalVideos = ref.watch(
      totalVideosProvider(
        '/video-channels/$channelHandle/videos',
      ),
    );
    return TabbedPageScaffold(
      key: ValueKey(asyncTotalVideos),
      appBar: AppBar(
        title: Text(
          platform.name,
          overflow: TextOverflow.ellipsis,
        ),
        titleSpacing: 0,
        actions: const [
          AccountAction(),
        ],
      ),
      tabTitles: [
        asyncTotalVideos.when(
          data: (count) => t(context)!.totalVideos(
            NumberFormat.decimalPattern(
              ref.watch(localeProvider).languageCode,
            ).format(count),
          ),
          loading: () => t(context)!.videos,
          error: (_, __) => t(context)!.videos,
        ),
        // 'Playlists'
      ],
      headerContent: AsyncValueWidget(
        value: ref.watch(videoChannelDetailsProvider(channelHandle)),
        dataBuilder: (channelDetails) {
          final asyncConfig =
              ref.watch(configServerProvider(channelDetails.host));
          final colorScheme = Theme.of(context).colorScheme;
          final textTheme = Theme.of(context).textTheme;
          return Stack(
            children: [
              if (channelDetails.banners.isNotEmpty)
                Semantics(
                  label: t(context)!.videoChannelBanner,
                  child: CustomCachedNetworkImage(
                    imageUrl: Uri.parse(
                      ref
                          .read(
                            currentPlatformNonNullableProvider,
                          )
                          .domain,
                    )
                        .resolve(
                          getImagePathForSize(
                            channelDetails.banners,
                            MediaQuery.of(context).size.width,
                          )!,
                        )
                        .toString(),
                  ),
                ),
              Padding(
                padding: const EdgeInsets.symmetric(horizontal: 16.0),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    const SizedBox(
                      height: 50,
                    ),
                    Row(
                      crossAxisAlignment: CrossAxisAlignment.end,
                      children: [
                        AvatarBuilder(
                          domain: ref
                              .watch(
                                currentPlatformNonNullableProvider,
                              )
                              .domain,
                          name: channelDetails.displayName,
                          size: 96,
                          actorImageList: channelDetails.avatars,
                        ),
                        Padding(
                          padding: const EdgeInsets.symmetric(
                            horizontal: 10,
                          ),
                          child: Column(
                            crossAxisAlignment: CrossAxisAlignment.start,
                            children: [
                              Text(
                                channelDetails.displayName,
                                style: textTheme.headlineLarge,
                                overflow: TextOverflow.ellipsis,
                              ),
                              Text(
                                '@${channelDetails.name}',
                                style: textTheme.displaySmall!.copyWith(
                                  color: colorScheme.onSecondary,
                                ),
                              ),
                              const SizedBox(
                                height: 10,
                              ),
                              Row(
                                children: [
                                  Icon(
                                    LucideIcons.users,
                                    size: 16,
                                    color: colorScheme.onSurfaceVariant,
                                  ),
                                  const SizedBox(
                                    width: 5,
                                  ),
                                  Text(
                                    t(context)!.totalFollowers(
                                      NumberFormat.decimalPattern(
                                        ref.watch(localeProvider).countryCode,
                                      ).format(
                                        channelDetails.followersCount,
                                      ),
                                    ),
                                    style: textTheme.displaySmall!.copyWith(
                                      color: colorScheme.onSurfaceVariant,
                                    ),
                                  ),
                                ],
                              ),
                            ],
                          ),
                        ),
                      ],
                    ),
                    const SizedBox(
                      height: 16,
                    ),
                    Row(
                      children: [
                        if (channelDetails.description != null)
                          Expanded(
                            flex: 2,
                            child: ReadMoreText(
                              channelDetails.description!,
                              style: TextStyle(
                                fontSize: 14,
                                color: colorScheme.onSecondary,
                              ),
                              textAlign: TextAlign.center,
                              trimMode: TrimMode.Line,
                              trimLines: 3,
                              trimCollapsedText: 'Show more',
                              trimExpandedText: '\nShow less',
                              moreStyle: textTheme.displayMedium!.copyWith(
                                shadows: [
                                  Shadow(
                                    color: colorScheme.onSurfaceVariant,
                                    offset: const Offset(0, -4),
                                  ),
                                ],
                                color: Colors.transparent,
                                decoration: TextDecoration.underline,
                              ),
                              lessStyle: textTheme.displayMedium!.copyWith(
                                shadows: [
                                  Shadow(
                                    color: colorScheme.onSurfaceVariant,
                                    offset: const Offset(0, -4),
                                  ),
                                ],
                                color: Colors.transparent,
                                decoration: TextDecoration.underline,
                              ),
                            ),
                          ),
                        if (channelDetails.support != null) ...{
                          const SizedBox(
                            width: 16,
                          ),
                          Expanded(
                            flex: 1,
                            child: TextButton(
                              child: SizedBox(
                                height: 48,
                                child: Center(
                                  child: Text(t(context)!.support),
                                ),
                              ),
                              onPressed: () => showDialog(
                                context: context,
                                builder: (context) => CustomDialog(
                                  content: Padding(
                                    padding: const EdgeInsets.only(top: 4.0),
                                    child: CustomHtmlWidget(
                                      rawHtml: md.markdownToHtml(
                                        channelDetails.support!,
                                      ),
                                    ),
                                  ),
                                ),
                              ),
                            ),
                          ),
                        },
                      ],
                    ),
                    const SizedBox(height: 16),
                    Text(
                      t(context)!.videoChannelManagedBy,
                      style: textTheme.displayMedium!.copyWith(
                        color: colorScheme.onSurfaceVariant,
                      ),
                    ),
                    const SizedBox(height: 4),
                    Semantics(
                      label: channelDetails.ownerAccount.name,
                      onTapHint: t(context)!.navigateToAccount,
                      child: GestureDetector(
                        onTap: () => GoRouter.of(context).push(
                          '/accounts/${channelDetails.ownerAccount.name}${channelDetails.isLocal ? '' : '@${channelDetails.host}'}',
                        ),
                        child: ConstrainedBox(
                          constraints: const BoxConstraints(
                            minHeight: 48.0,
                          ),
                          child: Row(
                            children: [
                              AvatarBuilder(
                                domain: ref
                                    .watch(
                                      currentPlatformNonNullableProvider,
                                    )
                                    .domain,
                                name: channelDetails.ownerAccount.displayName,
                                actorImageList:
                                    channelDetails.ownerAccount.avatars,
                                size: 41,
                                borderRadius: 25,
                              ),
                              const SizedBox(width: 8),
                              Column(
                                crossAxisAlignment: CrossAxisAlignment.start,
                                children: [
                                  Text(
                                    channelDetails.ownerAccount.displayName,
                                    style: textTheme.headlineMedium!.copyWith(
                                      color: colorScheme.onSurfaceVariant,
                                      fontWeight: FontWeight.w600,
                                    ),
                                  ),
                                  Text(
                                    '@${channelDetails.ownerAccount.name}',
                                    style: textTheme.displaySmall!.copyWith(
                                      color: colorScheme.onSecondary,
                                    ),
                                  ),
                                ],
                              ),
                            ],
                          ),
                        ),
                      ),
                    ),
                    const SizedBox(
                      height: 8,
                    ),
                    if (channelDetails.isLocal == false)
                      AsyncValueWidget(
                        value: asyncConfig,
                        dataBuilder: (config) => PlatformBannerFromConfig(
                          host: channelDetails.host,
                          config: config,
                          entityName: 'Video channel',
                        ),
                      ),
                  ],
                ),
              ),
            ],
          );
        },
      ),
      tabViews: [
        SingleChildScrollView(
          child: Column(
            children: [
              // Padding(
              //   padding: const EdgeInsets.all(16.0),
              //   child: Row(
              //     children: [
              //       Container(
              //         decoration: BoxDecoration(
              //           border: Border.all(
              //             color: Theme.of(context)
              //                 .colorScheme
              //                 .surfaceContainerHighest,
              //           ),
              //           shape: BoxShape.circle,
              //         ),
              //         child: IconButton(
              //           onPressed: () => {},
              //           icon: const Icon(
              //             LucideIcons.settings2,
              //             semanticLabel: 'Filter',
              //           ),
              //         ),
              //       ),
              //       const SizedBox(
              //         width: 8.0,
              //       ),
              //       const Expanded(
              //         child: CustomTextField(
              //           labelText: 'Search on',
              //         ),
              //       ),
              //     ],
              //   ),
              // ),
              VideoListView(
                fetchPage: (pageKey, pageSize) =>
                    ref.read(videoServiceProvider).getVideoChannelVideos(
                          channelHandle,
                          pageKey,
                          pageSize,
                        ),
                isNested: true,
                showChannel: false,
                domain: platform.domain,
              ),
            ],
          ),
        ),
        // const Center(
        //   child: Text('Playlist 🚧'),
        // ),
      ],
    );
  }
}
