import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:intl/intl.dart';

import 'package:peertube_mobile/features/account/connected_account/connected_account_providers.dart';
import 'package:peertube_mobile/features/account/connected_account/local_account/local_account_providers.dart';
import 'package:peertube_mobile/features/video/channel/channel_miniature.dart';
import 'package:peertube_mobile/features/video/comments/comment_list_view.dart';
import 'package:peertube_mobile/features/video/comments/comment_providers.dart';
import 'package:peertube_mobile/features/video/player/video_player.dart';
import 'package:peertube_mobile/features/video/video_model.dart';
import 'package:peertube_mobile/features/video/video_providers.dart';
import 'package:peertube_mobile/features/video/video_watch/video_info.dart';
import 'package:peertube_mobile/shared/config/config_models.dart';
import 'package:peertube_mobile/shared/config/config_providers.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_providers.dart';
import 'package:peertube_mobile/shared/filters/comment/comment_params_model.dart';
import 'package:peertube_mobile/shared/preferences/user_preferences_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/app_bar.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/future_builder.dart';
import 'package:peertube_mobile/shared/widgets/tabbed_page_scaffold.dart';

class VideoScreen extends ConsumerStatefulWidget {
  final String videoUuid;

  const VideoScreen({required this.videoUuid, super.key});

  @override
  ConsumerState<VideoScreen> createState() => _VideoScreenState();
}

class _VideoScreenState extends ConsumerState<VideoScreen> {
  Future<VideoDetails?> loadVideo(bool isLocal) async {
    final String? currentId = ref.read(selectedVideoProvider)?.uuid;
    if (currentId != widget.videoUuid) {
      final video = await ref
          .read(selectedVideoProvider.notifier)
          .loadSelectedVideo(widget.videoUuid);
      if (video != null) {
        final videoPlayerController = ref.read(videoPlayerControllerProvider);
        if (videoPlayerController != null) {
          if (currentId != null) {
            if (isLocal) {
              await ref
                  .read(localHistoryProvider.notifier)
                  .updateCurrentPosition(
                    currentId,
                    videoPlayerController.value.position.inSeconds,
                  );
            }
          }
          videoPlayerController.dispose();
        }
        ref.invalidate(videoPlayerControllerProvider);
        // ignore: use_build_context_synchronously
        await initializePlayer(context, ref, video);
        return video;
      } else {
        return null;
      }
    }
    return ref.read(selectedVideoProvider);
  }

  @override
  Widget build(BuildContext context) {
    final asyncTotalComments = ref.watch(
      totalCommentsProvider(widget.videoUuid),
    );
    final platform = ref.watch(currentPlatformNonNullableProvider);
    return AsyncValueWidget(
      value: ref.watch(currentConnectedAccountProvider),
      dataBuilder: (account) {
        return Dismissible(
          direction: DismissDirection.down,
          key: Key(widget.videoUuid),
          onDismissed: (_) => Navigator.of(context).pop(),
          child: CustomFutureBuilder<VideoDetails?>(
            future: loadVideo(account.isLocal()),
            onData: (video) {
              return TabbedPageScaffold(
                key: ValueKey(asyncTotalComments),
                appBar: AppBar(
                  leading: IconButton(
                    iconSize: 30.0,
                    icon: const Icon(Icons.keyboard_arrow_down),
                    onPressed: () => GoRouter.of(context).pop(),
                  ),
                  title: Text(
                    platform.name,
                    overflow: TextOverflow.ellipsis,
                  ),
                  actions: const [
                    AccountAction(),
                  ],
                ),
                tabTitles: [
                  asyncTotalComments.when(
                    data: (channelsCount) => channelsCount == 0
                        ? t(context)!.comments
                        : t(context)!.totalComments(
                            NumberFormat.decimalPattern(
                              ref.watch(localeProvider).languageCode,
                            ).format(channelsCount),
                          ),
                    loading: () => t(context)!.comments,
                    error: (_, __) => t(context)!.comments,
                  ),
                  t(context)!.similarVideos,
                ],
                tabViews: [
                  VideoCommentListView(
                    params: CommentParams(),
                    isLocalAccount: account.isLocal(),
                    videoUuid: widget.videoUuid,
                  ),
                  Container(height: 50),
                ],
                headerContent: Container(
                  color: Theme.of(context).scaffoldBackgroundColor,
                  child: SingleChildScrollView(
                    child: Column(
                      children: [
                        ref.watch(videoPlayerControllerProvider) != null
                            ? PeerTubeVideoPlayer(
                                videoPlayerController: ref.watch(
                                  videoPlayerControllerProvider,
                                )!,
                                isLive: video!.isLive,
                              )
                            : const AspectRatio(
                                aspectRatio: 16 / 9,
                                child: Center(
                                  child: CircularProgressIndicator(),
                                ),
                              ),
                        VideoInfo(
                          video: video!,
                          isUserLoggedIn: !account.isLocal(),
                        ),
                        CustomFutureBuilder<ConfigServer>(
                          future: ref.read(configDatasourceProvider).config(
                                Uri.parse(
                                  'https://${video.account.host}',
                                ),
                              ),
                          onData: (config) {
                            return VideoChannelMiniature(
                              videoChannel: video.channel,
                              config: config,
                              isLocalAccount: account.isLocal(),
                              showStatsAndDescription: false,
                            );
                          },
                        ),
                      ],
                    ),
                  ),
                ),
              );
            },
          ),
        );
      },
    );
  }
}
