import 'dart:io';

import 'package:dio/dio.dart';

import 'package:peertube_mobile/shared/auth/token_api_model.dart';
import 'package:peertube_mobile/shared/auth/token_model.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_models.dart';

class AuthDatasource {
  final CurrentPlatform currentPlatform;
  final Dio httpClient;

  AuthDatasource({
    required this.currentPlatform,
    required this.httpClient,
  });

  Future<Token> requestToken(
    String username,
    String password,
  ) async {
    String clientId, clientSecret;
    try {
      final clientResponse = await httpClient.get(
        currentPlatform.getApiBaseUrl('/oauth-clients/local'),
      );
      dynamic data = clientResponse.data;
      clientId = data['client_id'];
      clientSecret = data['client_secret'];
    } on DioException catch (_) {
      throw Exception('Failed to retrieve client identifiers');
    }

    final formData = {
      'client_id': clientId,
      'client_secret': clientSecret,
      'grant_type': 'password',
      'username': username,
      'password': password,
    };

    final options = Options(
      headers: <String, dynamic>{
        HttpHeaders.contentTypeHeader: Headers.formUrlEncodedContentType,
      },
    );
    try {
      final response = await httpClient.post(
        currentPlatform.getApiBaseUrl('/users/token'),
        data: formData,
        options: options,
      );

      dynamic data = response.data;
      final tokenServer = TokenAPI.fromJson(data);

      return Token(
        clientId: clientId,
        clientSecret: clientSecret,
        accessToken: tokenServer.accessToken,
        refreshToken: tokenServer.refreshToken,
        tokenType: tokenServer.tokenType,
      );
    } on DioException catch (e) {
      throw Exception(e);
    }
  }

  Future<Token> refreshToken(
    Token token,
  ) async {
    try {
      final formData = {
        'client_id': token.clientId,
        'client_secret': token.clientSecret,
        'grant_type': 'refresh_token',
        'refresh_token': token.refreshToken,
      };

      final response = await httpClient.post(
        currentPlatform.getApiBaseUrl('/users/token'),
        data: FormData.fromMap(formData),
        options: Options(
          contentType: Headers.formUrlEncodedContentType,
        ),
      );

      dynamic data = response.data;
      final tokenServer = TokenAPI.fromJson(data);

      return Token(
        clientId: token.clientId,
        clientSecret: token.clientSecret,
        accessToken: tokenServer.accessToken,
        refreshToken: tokenServer.refreshToken,
        tokenType: tokenServer.tokenType,
      );
    } on DioException catch (e) {
      throw Exception(e);
    }
  }

  Future<void> revokeToken(Dio peerTubeClient) async {
    try {
      await peerTubeClient.post(
        currentPlatform.getApiBaseUrl('/users/revoke-token'),
      );
    } on DioException catch (e) {
      throw Exception(e);
    }
  }
}
