import 'dart:convert';

import 'package:dio/dio.dart';

import 'package:peertube_mobile/shared/auth/auth_datasource.dart';
import 'package:peertube_mobile/shared/auth/token_model.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_models.dart';
import 'package:peertube_mobile/shared/logger/logger.dart';
import 'package:peertube_mobile/shared/storage/secure_storage_service.dart';

class AuthService {
  final AuthDatasource auth;
  final SecureStorageService storage;

  AuthService({
    required this.storage,
    required this.auth,
  });

  Future<bool> login(
    String username,
    String password,
    Platform platform,
  ) async {
    try {
      final Token token = await auth.requestToken(
        username,
        password,
      );

      await storage.setToken(
        _getStorageKey(platform),
        jsonEncode(token.toJson()),
      );
      return true;
    } catch (e, stackTrace) {
      log.warning('Login failed', e, stackTrace);
      return false;
    }
  }

  Future<String?> getAccessToken(Platform platform) async {
    final tokenJson = await storage.getToken(_getStorageKey(platform));
    if (tokenJson != null) {
      Token token = Token.fromJson(jsonDecode(tokenJson));
      return token.accessToken;
    }
    return null;
  }

  Future<bool> isLoggedIn(Platform platform) async =>
      (await getAccessToken(platform)) != null;

  Future<Token?> refreshToken(CurrentPlatform platform) async {
    final tokenJson = await storage.getToken(_getStorageKey(platform));
    if (tokenJson != null) {
      Token token = Token.fromJson(jsonDecode(tokenJson));
      try {
        final Token newToken = await auth.refreshToken(token);
        await storage.setToken(
          platform.domain.toString(),
          jsonEncode(newToken.toJson()),
        );
        return newToken;
      } catch (e) {
        throw Exception(e);
      }
    }
    return null;
  }

  Future<void> logout(Platform platform, Dio peerTubeClient) async {
    auth.revokeToken(peerTubeClient).then((_) async {
      await storage.removeToken(
        _getStorageKey(platform),
      );
    });
  }

  String _getStorageKey(Platform platform) => Uri.parse(platform.domain).host;
}
