import 'package:flutter/material.dart';

import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/shared/themes/color_schemes.dart';

class PeerTubeTheme {
  static ThemeData getThemeData(
    final Brightness brightness,
  ) {
    ColorScheme colorScheme;
    if (brightness == Brightness.light) {
      colorScheme = lightColorScheme;
    } else {
      colorScheme = darkColorScheme;
    }

    return ThemeData(
      scaffoldBackgroundColor: colorScheme.surfaceContainerLow,
      useMaterial3: true,
      colorScheme: colorScheme,
      textTheme: _TEXT_THEME,
      fontFamily: 'Source Sans Pro',
      appBarTheme: AppBarTheme(
        scrolledUnderElevation: 0,
        toolbarHeight: 65,
        titleSpacing: 24,
        titleTextStyle: _TEXT_THEME.titleMedium!.copyWith(
          color: colorScheme.onPrimary,
        ),
        actionsIconTheme: IconThemeData(
          size: 24,
          color: colorScheme.onSecondary,
        ),
        surfaceTintColor: Colors.transparent,
        color: colorScheme.surface,
      ),
      bottomNavigationBarTheme: BottomNavigationBarThemeData(
        backgroundColor: colorScheme.surface,
        selectedItemColor: colorScheme.onPrimary,
        unselectedItemColor: colorScheme.onSecondary,
        selectedLabelStyle: _TEXT_THEME.displaySmall,
        unselectedLabelStyle: _TEXT_THEME.displaySmall,
        showSelectedLabels: false,
        showUnselectedLabels: false,
      ),
      splashColor: Colors.transparent,
      tabBarTheme: TabBarTheme(
        labelStyle: _TEXT_THEME.headlineSmall,
        labelColor: colorScheme.onPrimary,
        labelPadding: const EdgeInsets.only(right: 18.0, top: 10.0),
        unselectedLabelStyle: _TEXT_THEME.headlineSmall,
        unselectedLabelColor: colorScheme.onSecondary,
        dividerColor: colorScheme.surfaceContainerHigh,
        tabAlignment: TabAlignment.start,
        overlayColor: WidgetStateProperty.resolveWith(
          (states) {
            if (states.contains(WidgetState.pressed)) {
              return colorScheme.secondary;
            }
            return Colors.transparent;
          },
        ),
      ),
      switchTheme: SwitchThemeData(
        trackColor:
            WidgetStateProperty.resolveWith<Color?>((Set<WidgetState> states) {
          if (states.contains(WidgetState.selected)) {
            return colorScheme.primary;
          }
          return colorScheme.surfaceContainerHighest;
        }),
        thumbColor:
            WidgetStateProperty.resolveWith<Color?>((Set<WidgetState> states) {
          if (states.contains(WidgetState.selected)) {
            return colorScheme.onPrimary;
          }
          return colorScheme.secondary;
        }),
        thumbIcon:
            WidgetStateProperty.resolveWith<Icon?>((Set<WidgetState> states) {
          if (states.contains(WidgetState.selected)) {
            return Icon(
              LucideIcons.check,
              size: 18,
              color: colorScheme.secondaryContainer,
            );
          }
          return Icon(
            Icons.close,
            size: 18,
            color: colorScheme.onTertiary,
          );
        }),
        trackOutlineWidth:
            WidgetStateProperty.resolveWith<double?>((Set<WidgetState> states) {
          return 0;
        }),
        trackOutlineColor:
            WidgetStateProperty.resolveWith<Color?>((Set<WidgetState> states) {
          return Colors.transparent;
        }),
      ),
      dividerTheme: DividerThemeData(color: colorScheme.surfaceContainerHigh),
      popupMenuTheme: PopupMenuThemeData(
        iconColor: colorScheme.onPrimary,
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(16.0),
        ),
      ),
      dialogTheme: DialogTheme(
        backgroundColor: colorScheme.surfaceContainerHighest,
      ),
      elevatedButtonTheme: ElevatedButtonThemeData(
        style: ButtonStyle(
          tapTargetSize: MaterialTapTargetSize.shrinkWrap,
          textStyle: WidgetStateProperty.all<TextStyle>(
            _TEXT_THEME.titleSmall!.copyWith(
              fontWeight: FontWeight.bold,
            ),
          ),
          backgroundColor: WidgetStateProperty.resolveWith<Color>(
            (Set<WidgetState> states) {
              if (states.contains(WidgetState.pressed)) {
                return colorScheme.onSecondary;
              }
              return colorScheme.primaryFixed;
            },
          ),
          foregroundColor: WidgetStateProperty.resolveWith<Color>((states) {
            if (states.contains(WidgetState.disabled)) {
              return Colors.grey;
            }
            return brown600;
          }),
          shape: WidgetStateProperty.all<OutlinedBorder>(
            RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(5.0),
              side: BorderSide(color: colorScheme.surface),
            ),
          ),
        ),
      ),
      textButtonTheme: TextButtonThemeData(
        style: ButtonStyle(
          textStyle: WidgetStateProperty.all<TextStyle>(
            _TEXT_THEME.labelLarge!.copyWith(
              fontWeight: FontWeight.bold,
            ),
          ),
          foregroundColor: WidgetStateProperty.resolveWith<Color>((states) {
            if (states.contains(WidgetState.disabled)) {
              return Colors.grey;
            }
            return colorScheme.onSurfaceVariant;
          }),
          tapTargetSize: MaterialTapTargetSize.shrinkWrap,
          shape: WidgetStateProperty.all<OutlinedBorder>(
            RoundedRectangleBorder(
              borderRadius: BorderRadius.circular(5.0),
              side: BorderSide(color: colorScheme.secondaryContainer),
            ),
          ),
        ),
      ),
      iconTheme: IconThemeData(
        color: colorScheme.onSurfaceVariant,
      ),
      chipTheme: ChipThemeData(
        backgroundColor: colorScheme.tertiary,
        selectedColor: colorScheme.primaryFixed,
        padding: const EdgeInsets.all(8.0),
        shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(20.0),
          side: BorderSide(
            color: colorScheme.onSecondary,
          ),
        ),
        labelStyle: TextStyle(
          color: ChipLabelColor(colorScheme),
        ),
        checkmarkColor: Colors.black,
      ),
    );
  }

  static const TextTheme _TEXT_THEME = TextTheme(
    titleLarge: TextStyle(
      fontSize: 28,
      fontWeight: FontWeight.bold,
      height: 1.2,
    ),
    titleMedium: TextStyle(
      fontSize: 22,
      fontWeight: FontWeight.bold,
      height: 1.2,
    ),
    titleSmall: TextStyle(
      fontSize: 18,
      fontWeight: FontWeight.normal,
    ),
    headlineLarge: TextStyle(
      fontSize: 18,
      fontFamily: 'Proza Libre',
      fontWeight: FontWeight.bold,
      height: 1.3,
    ),
    headlineMedium: TextStyle(
      fontFamily: 'Proza Libre',
      fontSize: 14,
      height: 1.3,
    ),
    headlineSmall: TextStyle(
      fontFamily: 'Proza Libre',
      fontSize: 12,
      height: 1.2,
      letterSpacing: 0.4,
    ),
    displayLarge: TextStyle(
      fontSize: 16,
    ),
    displayMedium: TextStyle(
      fontSize: 14,
    ),
    displaySmall: TextStyle(
      fontSize: 12,
    ),
    labelLarge: TextStyle(
      fontSize: 16,
      letterSpacing: 0.4,
    ),
    labelMedium: TextStyle(
      fontSize: 12,
      letterSpacing: 1.2,
    ),
    labelSmall: TextStyle(
      fontSize: 8,
      letterSpacing: 1.2,
    ),
  );
}

class ChipLabelColor extends Color implements WidgetStateColor {
  const ChipLabelColor(this.colorScheme) : super(_default);

  final ColorScheme colorScheme;

  static const int _default = 0xFF000000;

  @override
  Color resolve(Set<WidgetState> states) {
    if (states.contains(WidgetState.selected)) {
      return Colors.black;
    }
    return colorScheme.onPrimary;
  }
}
