import 'package:flutter/material.dart';

import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:go_router/go_router.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/features/account/connected_account/connected_account_providers.dart';
import 'package:peertube_mobile/shared/utils/translate.dart';
import 'package:peertube_mobile/shared/widgets/async_value.dart';
import 'package:peertube_mobile/shared/widgets/avatar_builder.dart';
import 'package:peertube_mobile/shared/widgets/dialog.dart';

class AccountAction extends ConsumerWidget {
  const AccountAction({
    super.key,
  });

  @override
  Widget build(context, ref) {
    final asyncAccount = ref.watch(currentConnectedAccountProvider);
    final colorScheme = Theme.of(context).colorScheme;
    return Padding(
      padding: const EdgeInsets.only(right: 15),
      child: Semantics(
        label: t(context)!.manageAccounts,
        child: GestureDetector(
          onTap: () => showModalBottomSheet(
            context: context,
            isScrollControlled: true,
            builder: (BuildContext context) {
              return const AccountModal();
            },
          ),
          child: SizedBox(
            height: 48,
            width: 48,
            child: Stack(
              children: [
                AsyncValueWidget(
                  value: asyncAccount,
                  dataBuilder: (account) => account.isLocal()
                      ? Container(
                          height: 36,
                          width: 36,
                          decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(20.0),
                            image: const DecorationImage(
                              image: AssetImage(
                                'assets/sepia/local_account_avatar.jpg',
                              ),
                              fit: BoxFit.cover,
                            ),
                          ),
                        )
                      : AvatarBuilder(
                          domain: '',
                          name: account.displayName,
                          size: 36,
                          avatarUrl: account.avatarUrl,
                        ),
                ),
                Positioned(
                  bottom: 2,
                  right: 2,
                  child: Container(
                    decoration: BoxDecoration(
                      color: colorScheme.secondary,
                      borderRadius: BorderRadius.circular(20),
                      border: Border.all(color: colorScheme.surface),
                    ),
                    child: Padding(
                      padding: const EdgeInsets.all(2.0),
                      child: Icon(
                        LucideIcons.menu,
                        size: 18,
                        color: colorScheme.onPrimary,
                      ),
                    ),
                  ),
                ),
              ],
            ),
          ),
        ),
      ),
    );
  }
}

class AccountModal extends ConsumerWidget {
  const AccountModal({
    super.key,
  });

  @override
  Widget build(context, ref) {
    return DraggableScrollableSheet(
      expand: false,
      snap: true,
      initialChildSize: 0.7,
      maxChildSize: 0.95,
      snapSizes: const <double>[0.7, 0.95],
      builder: (BuildContext context, ScrollController scrollController) {
        final colorsScheme = Theme.of(context).colorScheme;
        final textTheme = Theme.of(context).textTheme;
        return Container(
          color: colorsScheme.secondary,
          child: Column(
            children: <Widget>[
              Column(
                children: <Widget>[
                  SingleChildScrollView(
                    controller: scrollController,
                    child: Align(
                      alignment: Alignment.topCenter,
                      child: Container(
                        margin: const EdgeInsets.only(top: 24, bottom: 16),
                        height: 4.0,
                        width: 32.0,
                        decoration: BoxDecoration(
                          color: colorsScheme.onSecondary,
                          borderRadius: BorderRadius.circular(10.0),
                        ),
                      ),
                    ),
                  ),
                  Row(
                    children: [
                      Expanded(
                        child: Padding(
                          padding: const EdgeInsets.only(left: 48.0),
                          child: Text(
                            t(context)!.myAccounts,
                            style: textTheme.titleLarge,
                            textAlign: TextAlign.center,
                          ),
                        ),
                      ),
                      Padding(
                        padding: const EdgeInsets.all(8.0),
                        child: IconButton(
                          onPressed: () => showDialog(
                            context: context,
                            builder: (context) => TitleContentDialog(
                              title: t(context)!.accountManagement,
                              content: t(context)!.localAccountDescription,
                              secondContent: t(context)!.accountsDescription,
                            ),
                          ),
                          icon: Icon(
                            LucideIcons.helpCircle,
                            semanticLabel: t(context)!.showInformation,
                            size: 32,
                          ),
                        ),
                      ),
                    ],
                  ),
                  const SizedBox(
                    height: 16,
                  ),
                  Text(
                    t(context)!.activeAccount,
                    style: textTheme.displayMedium,
                  ),
                  const SizedBox(
                    height: 8,
                  ),
                  AsyncValueWidget(
                    value: ref.read(currentConnectedAccountProvider),
                    dataBuilder: (account) {
                      bool isLocal = account.isLocal();
                      return Stack(
                        children: [
                          Container(
                            margin: const EdgeInsets.symmetric(
                              horizontal: 16,
                              vertical: 24,
                            ),
                            height: 174,
                            width: MediaQuery.of(context).size.width * .95,
                            decoration: BoxDecoration(
                              borderRadius:
                                  const BorderRadius.all(Radius.circular(4.0)),
                              color: colorsScheme.surfaceContainerHighest,
                            ),
                            child: Column(
                              children: [
                                Padding(
                                  padding: const EdgeInsets.all(16.0),
                                  child: Align(
                                    alignment: Alignment.topRight,
                                    child: isLocal
                                        ? IconButton(
                                            onPressed: () =>
                                                GoRouter.of(context).push(
                                              '/settings',
                                            ),
                                            icon: Icon(
                                              LucideIcons.settings,
                                              semanticLabel: t(context)!
                                                  .accessAccountSettings,
                                            ),
                                          )
                                        : const Icon(LucideIcons.moreVertical),
                                  ),
                                ),
                                Text(
                                  account.displayName,
                                  style: textTheme.headlineLarge,
                                ),
                                if (isLocal)
                                  Text(
                                    t(context)!.localAccount,
                                    style: textTheme.displaySmall!.copyWith(
                                      color: colorsScheme.onSurfaceVariant,
                                    ),
                                  ),
                                const SizedBox(
                                  height: 8,
                                ),
                                if (isLocal)
                                  Text(
                                    t(context)!.localAccountShortDescription,
                                    style: textTheme.displaySmall!.copyWith(
                                      color: colorsScheme.onSecondary,
                                    ),
                                    textAlign: TextAlign.center,
                                  ),
                              ],
                            ),
                          ),
                          Center(
                            child: Container(
                              decoration: BoxDecoration(
                                border: Border.all(
                                  color: colorsScheme.surfaceContainerHighest,
                                  width: 4,
                                ),
                                borderRadius:
                                    const BorderRadius.all(Radius.circular(50)),
                              ),
                              child: Container(
                                decoration: BoxDecoration(
                                  border: Border.all(
                                    color: colorsScheme.primary,
                                    width: 2,
                                  ),
                                  borderRadius: const BorderRadius.all(
                                    Radius.circular(50),
                                  ),
                                ),
                                child: AvatarBuilder(
                                  domain: '',
                                  avatarUrl: account.avatarUrl,
                                  name: account.username,
                                  size: 60,
                                  borderRadius: 50,
                                  borderColor:
                                      colorsScheme.surfaceContainerHighest,
                                  borderWidth: 2,
                                ),
                              ),
                            ),
                          ),
                        ],
                      );
                    },
                  ),
                ],
              ),
            ],
          ),
        );
      },
    );
  }
}
