import 'package:flutter/material.dart';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:lucide_icons/lucide_icons.dart';

import 'package:peertube_mobile/shared/actor/actor_image_model.dart';
import 'package:peertube_mobile/shared/themes/color_schemes.dart';
import 'package:peertube_mobile/shared/utils/tools.dart';

class AvatarBuilder extends StatelessWidget {
  final String domain;
  final List<ActorImage>? actorImageList;
  final String? avatarUrl;
  final String name;
  final bool isDark;
  final double size;
  final double borderWidth;
  final double borderRadius;
  final Color? borderColor;

  const AvatarBuilder({
    super.key,
    this.actorImageList,
    this.avatarUrl,
    required this.domain,
    required this.name,
    this.isDark = false,
    required this.size,
    this.borderWidth = 1.2,
    this.borderRadius = 5.0,
    this.borderColor,
  });

  @override
  Widget build(BuildContext context) {
    String? path;
    if (actorImageList != null) {
      path = getImagePathForSize(actorImageList!, size);
    }
    path ??= avatarUrl ?? '';
    ColorPair colors = getColorPair(name);
    final contextBorderColor =
        borderColor ?? Theme.of(context).colorScheme.secondary;
    return ExcludeSemantics(
      child: Container(
        width: size,
        height: size,
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(borderRadius),
          border: Border.all(color: contextBorderColor, width: borderWidth),
          color: path == ''
              ? isDark
                  ? colors.primary
                  : colors.secondary
              : beige300,
        ),
        child: path == ''
            ? Center(
                child: Text(
                  getInitials(name),
                  style: Theme.of(context).textTheme.titleSmall!.copyWith(
                        color: isDark ? colors.secondary : colors.primary,
                      ),
                ),
              )
            : CachedNetworkImage(
                imageUrl: Uri.parse(domain).resolve(path).toString(),
                imageBuilder: (context, imageProvider) => Container(
                  width: size,
                  height: size,
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(borderRadius),
                    image: DecorationImage(
                      image: imageProvider,
                      fit: BoxFit.cover,
                    ),
                  ),
                ),
                placeholder: (context, url) =>
                    const CircularProgressIndicator(),
                errorWidget: (context, url, error) =>
                    const Icon(LucideIcons.alertCircle),
              ),
      ),
    );
  }

  String getInitials(String name) {
    List<String> words = name.split(' ');
    String initials = '';

    for (String word in words) {
      if (word.isNotEmpty) {
        initials += word[0].toLowerCase();
      }
    }
    return initials.length >= 2 ? initials.substring(0, 2) : initials;
  }

  ColorPair getColorPair(String name) {
    final initialLowercase = name.substring(0, 1).toLowerCase();

    final Map<String, ColorPair> themes = {
      '0123456789abc':
          ColorPair(const Color(0xFFCC4E13), const Color(0xFFFEE7C8)),
      'defgh': ColorPair(const Color(0xFFD93E25), const Color(0xFFFEDCD7)),
      'ijklm': ColorPair(const Color(0xFF725794), const Color(0xFFEDD8FD)),
      'nopqr': ColorPair(const Color(0xFF677490), const Color(0xFFEDF1F7)),
      'stvu': ColorPair(const Color(0xFF487982), const Color(0xFFB3E4E6)),
      'wxyz': ColorPair(const Color(0xFF338650), const Color(0xFF8FEBA4)),
    };

    try {
      final theme =
          themes.keys.firstWhere((chars) => chars.contains(initialLowercase));
      return themes[theme] ?? themes.entries.first.value;
    } catch (e) {
      return themes.entries.first.value;
    }
  }
}

class ColorPair {
  final Color primary;
  final Color secondary;

  ColorPair(this.primary, this.secondary);
}
