import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:http_mock_adapter/http_mock_adapter.dart';

import 'package:peertube_mobile/features/video/rating/rating_datasource.dart';
import '../../../test_utils.dart';

void main() {
  group('RatingDatasource', () {
    late Dio httpClient;
    late DioAdapter dioAdapter;
    late RatingDatasource ratingDatasource;

    setUp(() {
      httpClient = Dio();

      httpClient.interceptors.add(DioLoggingInterceptor());
      dioAdapter = DioAdapter(dio: httpClient);
      ratingDatasource = RatingDatasource(
        peertubeClient: httpClient,
        currentPlatform: mockPeertubeInstance,
      );
    });

    group('getRate', () {
      test('should return video rating', () async {
        const videoId = 123;
        const expectedRating = 'like';
        final mockResponseData = {'rating': expectedRating};

        dioAdapter.onGet(
          mockPeertubeInstance
              .getApiBaseUrl('/users/me/videos/$videoId/rating'),
          (server) => server.reply(HttpStatus.ok, mockResponseData),
        );

        final rating = await ratingDatasource.getRate(videoId);

        expect(rating, expectedRating);
      });

      test('should throw an Exception when retrieving rating fails', () async {
        const videoId = 123;
        final requestOptions = RequestOptions(
          path: mockPeertubeInstance
              .getApiBaseUrl('/users/me/videos/$videoId/rating'),
        );

        final dioException = MockDioException(
          requestOptions: requestOptions,
          error: 'mock error message',
          type: DioExceptionType.badResponse,
        );

        dioAdapter.onGet(
          mockPeertubeInstance
              .getApiBaseUrl('/users/me/videos/$videoId/rating'),
          (server) =>
              server.throws(HttpStatus.internalServerError, dioException),
        );

        expect(
          () => ratingDatasource.getRate(videoId),
          throwsA(
            isA<Exception>(),
          ),
        );
      });
    });

    group('rateVideo', () {
      test('should rate the video successfully', () async {
        const videoId = 123;
        const rating = 'like';

        dioAdapter.onPut(
          mockPeertubeInstance.getApiBaseUrl('/videos/$videoId/rate'),
          (server) => server.reply(HttpStatus.noContent, null),
          data: Matchers.any,
        );

        await ratingDatasource.rateVideo(videoId, rating);
      });

      test('should throw an Exception when rating fails', () async {
        const videoId = 123;
        const rating = 'like';

        final dioException = MockDioException(
          requestOptions: RequestOptions(
            path: mockPeertubeInstance.getApiBaseUrl('/videos/$videoId/rate'),
          ),
          error: 'mock error message',
          type: DioExceptionType.badResponse,
        );

        dioAdapter.onPut(
          mockPeertubeInstance.getApiBaseUrl('/videos/$videoId/rate'),
          (server) =>
              server.throws(HttpStatus.internalServerError, dioException),
          data: Matchers.any,
        );

        expect(
          () => ratingDatasource.rateVideo(videoId, rating),
          throwsA(
            isA<Exception>(),
          ),
        );
      });
    });
  });
}

final mockPeertubeInstance = MockedObjects.generateMockCurrentInstance();
