import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';

import 'package:peertube_mobile/features/video/rating/rating_datasource.dart';
import 'package:peertube_mobile/features/video/rating/rating_service.dart';
import 'rating_service_test.mocks.dart';

@GenerateNiceMocks([
  MockSpec<RatingDatasource>(),
])
void main() {
  group('RatingService', () {
    late RatingService service;
    late MockRatingDatasource ratingDatasource;

    setUp(() {
      ratingDatasource = MockRatingDatasource();
      service = RatingService(rating: ratingDatasource);
    });

    group('getRate', () {
      test('should return video rating', () async {
        const videoId = 123;
        const expectedRating = 'like';
        when(ratingDatasource.getRate(videoId))
            .thenAnswer((_) async => expectedRating);

        final rating = await service.getRate(videoId);

        expect(rating, expectedRating);
        verify(ratingDatasource.getRate(videoId)).called(1);
      });

      test('should throw an Exception when retrieving rating fails', () async {
        const videoId = 123;
        when(ratingDatasource.getRate(videoId)).thenThrow(Exception());

        expect(() => service.getRate(videoId), throwsException);
        verify(ratingDatasource.getRate(videoId)).called(1);
      });
    });

    group('updateRate', () {
      const videoId = 123;
      const likeCount = 10;
      const dislikeCount = 5;

      test('should update video rating successfully from "like" to "dislike"',
          () async {
        const lastRating = 'like';
        const newRating = 'dislike';

        when(ratingDatasource.rateVideo(videoId, newRating))
            .thenAnswer((_) async {});

        final result = await service.updateRate(
          videoId: videoId,
          lastRating: lastRating,
          newRating: newRating,
          likeCount: likeCount,
          dislikeCount: dislikeCount,
        );

        expect(result, [likeCount - 1, dislikeCount + 1]);
        verify(ratingDatasource.rateVideo(videoId, newRating)).called(1);
      });

      test('should update video rating successfully from "dislike" to "like"',
          () async {
        const lastRating = 'dislike';
        const newRating = 'like';

        when(ratingDatasource.rateVideo(videoId, newRating))
            .thenAnswer((_) async {});

        final result = await service.updateRate(
          videoId: videoId,
          lastRating: lastRating,
          newRating: newRating,
          likeCount: likeCount,
          dislikeCount: dislikeCount,
        );

        expect(result, [likeCount + 1, dislikeCount - 1]);
        verify(ratingDatasource.rateVideo(videoId, newRating)).called(1);
      });

      test('should remove video rating when updating from "like" to "like"',
          () async {
        const lastRating = 'like';
        const newRating = 'like';

        when(ratingDatasource.rateVideo(videoId, newRating))
            .thenAnswer((_) async {});

        final result = await service.updateRate(
          videoId: videoId,
          lastRating: lastRating,
          newRating: newRating,
          likeCount: likeCount,
          dislikeCount: dislikeCount,
        );

        expect(result, [likeCount - 1, dislikeCount]);
      });

      test(
          'should remove video rating when updating from "dislike" to "dislike"',
          () async {
        const lastRating = 'dislike';
        const newRating = 'dislike';

        when(ratingDatasource.rateVideo(videoId, newRating))
            .thenAnswer((_) async {});

        final result = await service.updateRate(
          videoId: videoId,
          lastRating: lastRating,
          newRating: newRating,
          likeCount: likeCount,
          dislikeCount: dislikeCount,
        );

        expect(result, [likeCount, dislikeCount - 1]);
      });

      test('should handle errors when updating rating fails', () async {
        const lastRating = 'like';
        const newRating = 'dislike';

        when(ratingDatasource.rateVideo(videoId, newRating))
            .thenThrow(Exception());

        expect(
          () => service.updateRate(
            videoId: videoId,
            lastRating: lastRating,
            newRating: newRating,
            likeCount: likeCount,
            dislikeCount: dislikeCount,
          ),
          throwsException,
        );

        verify(ratingDatasource.rateVideo(videoId, newRating)).called(1);
      });
    });
  });
}
