import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:http_mock_adapter/http_mock_adapter.dart';

import 'package:peertube_mobile/features/video/video_datasource.dart';
import 'package:peertube_mobile/features/video/video_model.dart';
import 'package:peertube_mobile/shared/network/http_service.dart';
import '../../test_utils.dart';

void main() {
  group('VideoDatasource', () {
    late Dio httpClient;
    late DioAdapter dioAdapter;
    late VideoDatasource videoDatasource;

    setUp(() {
      httpClient = Dio();
      dioAdapter = DioAdapter(dio: httpClient);
      videoDatasource = VideoDatasource(
        httpService: HttpService(peertubeClient: httpClient),
        currentPlatform: mockPeertubeInstance,
      );
    });

    group('fetchVideos', () {
      test('fetchVideos should return a list of Video objects', () async {
        final mockResponseData = {
          'data': [videoData],
        };

        dioAdapter.onGet(
          mockPeertubeInstance.getApiBaseUrl(VideoDatasource.videoPath),
          (server) => server.reply(HttpStatus.ok, mockResponseData),
        );

        final videos =
            await videoDatasource.fetchVideos(VideoDatasource.videoPath);

        expect(videos, isA<List<Video>>());
      });

      test('should throw an Exception when fetching videos fails', () async {
        final requestOptions = RequestOptions(
          path: mockPeertubeInstance.getApiBaseUrl(VideoDatasource.videoPath),
        );

        final dioException = MockDioException(
          requestOptions: requestOptions,
          error: 'mock error message',
          type: DioExceptionType.badResponse,
        );

        dioAdapter.onGet(
          mockPeertubeInstance.getApiBaseUrl(VideoDatasource.videoPath),
          (server) =>
              server.throws(HttpStatus.internalServerError, dioException),
        );

        expect(
          () => videoDatasource.fetchVideos(VideoDatasource.videoPath),
          throwsA(
            isA<Exception>(),
          ),
        );
      });
    });

    group('fetchVideo', () {
      test('fetchVideo should return a VideoDetails object', () async {
        const videoUuid = '12345';
        final mockResponseData = videoDetailsData;

        dioAdapter.onGet(
          mockPeertubeInstance
              .getApiBaseUrl('${VideoDatasource.videoPath}/$videoUuid'),
          (server) => server.reply(HttpStatus.ok, mockResponseData),
        );

        final video = await videoDatasource.fetchVideo(videoUuid);

        expect(video, isA<VideoDetails>());
      });

      test('should throw an Exception when fetching a video fails', () async {
        const videoUuid = '12345';
        final requestOptions = RequestOptions(
          path: mockPeertubeInstance
              .getApiBaseUrl('${VideoDatasource.videoPath}/$videoUuid'),
        );

        final dioException = MockDioException(
          requestOptions: requestOptions,
          error: 'mock error message',
          type: DioExceptionType.badResponse,
        );

        dioAdapter.onGet(
          mockPeertubeInstance
              .getApiBaseUrl('${VideoDatasource.videoPath}/$videoUuid'),
          (server) =>
              server.throws(HttpStatus.internalServerError, dioException),
        );

        expect(
          () => videoDatasource.fetchVideo(videoUuid),
          throwsA(
            isA<Exception>(),
          ),
        );
      });
    });
  });
}

final accountData = MockedObjects.generateMockAccountDetails().toJson();

final videoData = MockedObjects.generateMockVideo().toJson();

final videoDetailsData = MockedObjects.generateMockVideoDetails().toJson();

final mockPeertubeInstance = MockedObjects.generateMockCurrentInstance();
