import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:http_mock_adapter/http_mock_adapter.dart';

import 'package:peertube_mobile/shared/auth/auth_datasource.dart';
import 'package:peertube_mobile/shared/auth/token_model.dart';
import '../../test_utils.dart';

void main() {
  group('AuthDatasource', () {
    late Dio httpClient;
    late DioAdapter dioAdapter;
    late AuthDatasource authDatasource;
    const errorCode = 500;

    final mockRequestTokenPath =
        mockPeertubeInstance.getApiBaseUrl('/users/token');

    setUp(() {
      httpClient = Dio();
      dioAdapter = DioAdapter(dio: httpClient);
      authDatasource = AuthDatasource(
        httpClient: httpClient,
        currentPlatform: mockPeertubeInstance,
      );
    });

    group('requestToken', () {
      final mockLoginPrerequisitePath =
          mockPeertubeInstance.getApiBaseUrl('/oauth-clients/local');

      test('should throw an Exception when retrieving client identifiers fails',
          () async {
        final requestOptions = RequestOptions(path: mockLoginPrerequisitePath);

        final dioException = MockDioException(
          requestOptions: requestOptions,
          error: 'mock error message',
          type: DioExceptionType.badResponse,
        );

        dioAdapter.onGet(
          mockLoginPrerequisitePath,
          (server) => server.throws(errorCode, dioException),
        );

        expect(
          () => authDatasource.requestToken('username', 'password'),
          throwsA(
            isA<Exception>().having(
              (e) => e.toString(),
              'toString',
              contains('Failed to retrieve client identifiers'),
            ),
          ),
        );
      });

      test('should throw an Exception when getting token fails', () async {
        final requestOptions = RequestOptions(path: mockRequestTokenPath);

        final dioException = MockDioException(
          requestOptions: requestOptions,
          error: 'mock error message',
          type: DioExceptionType.badResponse,
        );

        dioAdapter
          ..onGet(
            mockLoginPrerequisitePath,
            (server) => server.reply(HttpStatus.ok, mockLoginPrerequisiteData),
          )
          ..onPost(
            mockRequestTokenPath,
            (server) => server.throws(errorCode, dioException),
          );

        expect(
          () => authDatasource.requestToken('username', 'password'),
          throwsA(
            isA<Exception>(),
          ),
        );
      });
      test('should return token when requests succeed', () async {
        dioAdapter
          ..onGet(
            mockLoginPrerequisitePath,
            (server) => server.reply(HttpStatus.ok, mockLoginPrerequisiteData),
          )
          ..onPost(
            mockRequestTokenPath,
            (server) => server.reply(
              HttpStatus.ok,
              mockTokenAPIData,
            ),
            data: Matchers.any,
          );

        expect(
          await authDatasource.requestToken('username', 'password'),
          isA<Token>()
              .having((p0) => p0.accessToken, 'accessToken', 'access_token')
              .having((p0) => p0.refreshToken, 'refreshToken', 'refresh_token')
              .having((p0) => p0.tokenType, 'tokenType', 'token_type')
              .having((p0) => p0.clientId, 'clientId', 'id')
              .having((p0) => p0.clientSecret, 'clientSecret', 'secret'),
        );
      });
    });

    group('refreshToken', () {
      test('should throw an Exception when refreshing token fails', () async {
        final requestOptions = RequestOptions(path: mockRequestTokenPath);

        final dioException = MockDioException(
          requestOptions: requestOptions,
          error: 'mock error message',
          type: DioExceptionType.badResponse,
        );

        dioAdapter.onPost(
          mockRequestTokenPath,
          (server) => server.throws(errorCode, dioException),
        );

        expect(
          () => authDatasource.refreshToken(
            const Token(
              clientId: 'clientId',
              clientSecret: 'clientSecret',
              accessToken: 'accessToken',
              refreshToken: 'refreshToken',
              tokenType: 'tokenType',
            ),
          ),
          throwsA(
            isA<Exception>(),
          ),
        );
      });

      test('should return token when refreshing token succeeds', () async {
        dioAdapter.onPost(
          mockRequestTokenPath,
          (server) => server.reply(
            HttpStatus.ok,
            mockTokenAPIData,
          ),
          data: Matchers.any,
        );

        expect(
          await authDatasource.refreshToken(
            const Token(
              clientId: 'id',
              clientSecret: 'secret',
              accessToken: 'access_token',
              refreshToken: 'refresh_token',
              tokenType: 'token_type',
            ),
          ),
          isA<Token>()
              .having((p0) => p0.accessToken, 'accessToken', 'access_token')
              .having((p0) => p0.refreshToken, 'refreshToken', 'refresh_token')
              .having((p0) => p0.tokenType, 'tokenType', 'token_type')
              .having((p0) => p0.clientId, 'clientId', 'id')
              .having((p0) => p0.clientSecret, 'clientSecret', 'secret'),
        );
      });
    });
  });
}

final mockPeertubeInstance = MockedObjects.generateMockCurrentInstance();

final mockLoginPrerequisiteData = <String, dynamic>{
  'client_id': 'id',
  'client_secret': 'secret',
};
final mockTokenAPIData = <String, dynamic>{
  'access_token': 'access_token',
  'refresh_token': 'refresh_token',
  'token_type': 'token_type',
};
