import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';

import 'package:peertube_mobile/shared/auth/auth_service.dart';
import 'package:peertube_mobile/shared/auth/token_model.dart';
import 'package:peertube_mobile/shared/current_platform/current_platform_models.dart';
import 'package:peertube_mobile/shared/network/dio_authentication_interceptor.dart';
import '../../test_utils.dart';
import 'dio_authentication_interceptor_test.mocks.dart';

@GenerateNiceMocks([
  MockSpec<RequestInterceptorHandler>(),
  MockSpec<ErrorInterceptorHandler>(),
  MockSpec<AuthService>(),
])
void main() {
  group('AuthenticationInterceptor', () {
    late MockRequestInterceptorHandler requestHandler;
    late MockErrorInterceptorHandler errorHandler;
    late MockAuthService authService;

    late DioAuthenticationInterceptor authenticationInterceptor;
    late Dio dio;

    late CurrentPlatform mockCurrentPeerTubeInstance;
    setUp(() {
      requestHandler = MockRequestInterceptorHandler();
      errorHandler = MockErrorInterceptorHandler();
      authService = MockAuthService();
      dio = Dio();

      dio.interceptors.add(DioLoggingInterceptor());

      mockCurrentPeerTubeInstance = MockedObjects.generateMockCurrentInstance();

      authenticationInterceptor = DioAuthenticationInterceptor(
        currentPlaform: mockCurrentPeerTubeInstance,
        auth: authService,
        errorHandlerDio: dio,
      );
    });

    group('onRequest', () {
      test(
          'should add the ${HttpHeaders.contentTypeHeader} header to the request options',
          () async {
        authenticationInterceptor.onRequest(mockOptions, requestHandler);

        // Hack to wait the Future secureStorage.getToken() is completed
        await Future.delayed(const Duration(milliseconds: 0));

        verify(
          requestHandler.next(
            argThat(
              isA<RequestOptions>().having(
                (p0) => p0.headers,
                'headers',
                containsPair(
                  HttpHeaders.contentTypeHeader,
                  'application/json',
                ),
              ),
            ),
          ),
        ).called(1);
        verifyNever(requestHandler.reject(any));
      });

      test(
          'should add the ${HttpHeaders.authorizationHeader} header to the request options when token for selected peertube instance is not null',
          () async {
        when(
          authService.getAccessToken(mockCurrentPeerTubeInstance),
        ).thenAnswer(
          (_) async => mockToken.accessToken,
        );

        authenticationInterceptor.onRequest(mockOptions, requestHandler);

        // Hack to wait the Future secureStorage.getToken() is completed
        await Future.delayed(const Duration(milliseconds: 0));

        verify(
          requestHandler.next(
            argThat(
              isA<RequestOptions>().having(
                (p0) => p0.headers,
                'headers',
                containsPair(
                  HttpHeaders.authorizationHeader,
                  '${DioAuthenticationInterceptor.bearerPrefix} ${mockToken.accessToken}',
                ),
              ),
            ),
          ),
        ).called(1);
        verifyNever(requestHandler.reject(any));
      });

      test(
          'should reject with DioException type ${DioExceptionType.cancel} when getting access token failed',
          () async {
        final mockException = Exception('get token failed');
        when(
          authService.getAccessToken(mockCurrentPeerTubeInstance),
        ).thenAnswer(
          (_) {
            throw mockException;
          },
        );

        authenticationInterceptor.onRequest(mockOptions, requestHandler);

        // Hack to wait the Future secureStorage.getToken() is completed
        await Future.delayed(const Duration(milliseconds: 0));

        verify(
          requestHandler.reject(
            argThat(
              isA<DioException>()
                  .having(
                    (p0) => p0.requestOptions,
                    'requestOptions',
                    equals(mockOptions),
                  )
                  .having(
                    (p0) => p0.type,
                    'type',
                    DioExceptionType.cancel,
                  )
                  .having(
                    (p0) => p0.error,
                    'error',
                    equals(mockException),
                  ),
            ),
          ),
        ).called(1);
        // verifyNever(requestHandler.next(any));
      });
    });

    group('onError', () {
      test('should refresh token and retry request successfully', () async {
        final mockErrorResponse = Response(
          requestOptions: mockOptions,
          statusCode: HttpStatus.unauthorized,
        );

        final mockException = DioException(
          requestOptions: mockOptions,
          response: mockErrorResponse,
        );

        const newAccessToken = 'new_access_token';
        const newToken = Token(
          accessToken: newAccessToken,
          clientId: 'id',
          clientSecret: 'secret',
          refreshToken: 'refresh',
          tokenType: '',
        );

        when(authService.refreshToken(mockCurrentPeerTubeInstance))
            .thenAnswer((_) async => newToken);

        // when(dio.fetch(mockOptions)).thenAnswer(
        //   (_) async => Response(requestOptions: mockOptions, data: {}),
        // );

        authenticationInterceptor.onError(mockException, errorHandler);

        verify(authService.refreshToken(mockCurrentPeerTubeInstance)).called(1);

        // verify(
        //   errorHandler.resolve(any),
        // ).called(1);
      });

      test('should handle error when refreshToken fails', () async {
        final mockErrorResponse = Response(
          requestOptions: mockOptions,
          statusCode: HttpStatus.unauthorized,
        );

        final mockException = DioException(
          requestOptions: mockOptions,
          response: mockErrorResponse,
        );

        when(authService.refreshToken(mockCurrentPeerTubeInstance))
            .thenThrow(Exception('Refresh token failed'));

        expect(
          () => authenticationInterceptor.onError(mockException, errorHandler),
          throwsA(
            isA<Exception>(),
          ),
        );
        verify(authService.refreshToken(mockCurrentPeerTubeInstance)).called(1);
      });

      test('should pass through other errors', () async {
        final mockException = DioException(
          requestOptions: mockOptions,
          error: Exception('Other error occurred'),
        );

        authenticationInterceptor.onError(mockException, errorHandler);

        //TODO
      });
    });
  });
}

const mockToken = Token(
  accessToken: 'mock_access_token',
  clientId: 'mock_id',
  clientSecret: 'mock_secret',
  refreshToken: 'mock_refresh',
  tokenType: '',
);

final mockOptions = RequestOptions(
  path: 'https://demo.test',
);
