import 'package:flutter/material.dart';

import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'package:peertube_mobile/shared/preferences/user_preferences.dart';
import 'user_preferences_test.mocks.dart';

@GenerateNiceMocks([
  MockSpec<SharedPreferences>(),
])
void main() {
  group('SharedUtility', () {
    late MockSharedPreferences sharedPreferences;
    late SharedUtility sharedUtility;

    setUp(() {
      sharedPreferences = MockSharedPreferences();
      sharedUtility = SharedUtility(sharedPreferences: sharedPreferences);
    });

    group('setTheme', () {
      test('should set the theme correctly', () async {
        const ThemeMode theme = ThemeMode.dark;

        await sharedUtility.setTheme(theme);

        verify(
          sharedPreferences.setInt(
            'currentThemeModeIndex',
            theme.index,
          ),
        ).called(1);
      });
    });

    group('currentTheme', () {
      test('should return the current theme correctly', () async {
        const ThemeMode theme = ThemeMode.light;

        when(sharedPreferences.getInt('currentThemeModeIndex'))
            .thenReturn(ThemeMode.light.index);

        final result = sharedUtility.currentThemeIndex;

        expect(result, equals(theme.index));
      });

      test('should return default theme if no theme is set', () async {
        const ThemeMode defaultTheme = ThemeMode.system;

        when(sharedPreferences.getInt('currentThemeModeIndex'))
            .thenReturn(null);

        final result = sharedUtility.currentThemeIndex;

        expect(result, equals(defaultTheme.index));
      });
    });

    group('setUserCountryCode', () {
      test('should set the user country code correctly', () async {
        const String countryCode = 'FR';

        await sharedUtility.setUserCountryCode(countryCode);

        verify(
          sharedPreferences.setString(
            'userCountry',
            countryCode,
          ),
        ).called(1);
      });
    });

    group('userCountryCode', () {
      test('should return the user country code correctly', () async {
        const String countryCode = 'US';

        when(sharedPreferences.getString('userCountry'))
            .thenReturn(countryCode);

        final result = sharedUtility.userCountryCode;

        expect(result, equals(countryCode));
      });

      test('should return null if no country code is set', () async {
        when(sharedPreferences.getString('userCountry')).thenReturn(null);

        final result = sharedUtility.userCountryCode;

        expect(result, isNull);
      });
    });
  });
}
